//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.framework.command;

/**
 * This type describes an object which manages contexts. The "current" context
 * is the object referred to by getContext().  The "current" context may
 * be an object as simple as DefaultContext.  Alternatively it may be
 * more complex like CommandStackManager.
 * Contexts may be added or removed from the context manager, although
 * exactly what this does depends on the concrete implementation of
 * ContextManager.
 *
 * @author Marcel Baumann
 *         <p/>
 *         Date        Chng.Rq.   Author    Changes/Enhancements
 *         1999.01.04             MB       Created
 * @see com.nexirius.framework.command.Context
 * @see com.nexirius.framework.command.DefaultContextManager
 * @see com.nexirius.framework.command.StackContextManager
 */
public interface ContextManager {
    /**
     * Return a reference to a context which represents all the commands
     * available to the end-user at this time.
     *
     * @return com.nexirius.framework.command.Context
     * @author Marcel Baumann
     */
    Context getContext();


    /**
     * Add a new context to the ContextManager. Exactly how this is implemented
     * is up to the concrete implementation of ContextManager. This may
     * involve a simple replacement of the current context. Or it may involve
     * a complex merge of the current context with the given context.
     *
     * @param context The context to be added
     * @author Marcel Baumann
     */
    void addContext(Context context);


    /**
     * Removes the given context from the context manager. See comments
     * for addContext().
     *
     * @param context The context to be removed
     * @author Marcel Baumann
     */
    void removeContext(Context context);


    /**
     * Add a listener for context change events
     *
     * @param listener A context change listenern
     * @author Marcel Baumann
     */
    void addContextChangeListener(ContextChangeListener listener);


    /**
     * Remove a listener for context change events
     *
     * @param listener A context change listenern
     * @author Marcel Baumann
     */
    void removeContextChangeListener(ContextChangeListener listener);
}
