//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.framework.dataeditor;

import com.nexirius.framework.datamodel.PasswordModel;

import javax.swing.*;

/**
 * This viewer is designed to edit PasswordModel data
 *
 * @author Marcel Baumann
 * @see com.nexirius.framework.datamodel.PasswordModel
 */

public class PasswordEditor extends SimpleEditor {



    public static final String editorName = "PasswordEditor";

    /**
     * Creates a new instance and initialises its associated password model
     */
    public PasswordEditor(PasswordModel model) {
        super(model);
    }

    /**
     * Returns the associated model
     */
    public PasswordModel getPasswordModel() {
        return (PasswordModel) getDataModel();
    }

    /**
     * Creates the actual swing component (JPasswordField)
     */
    public void create() {
        JPasswordField field = new JPasswordField();

        setJComponent(field);
        field.getDocument().addDocumentListener(new MyDocumentListener());
        field.addFocusListener(new MyFocusListener());
        field.addKeyListener(new KeyAdapter());
        field.setEchoChar(getPasswordModel().getEchoChar());
        update();
    }

    /**
     * Updates the value in the password field
     */
    public void update() {
        if (isCreated() && !getJTextComponent().hasFocus()) {
            resetTextComponent();
        }
    }

    /**
     * Reset the actual password field to the associated model value
     */
    public void resetTextComponent() {
        if (isCreated()) {
            if (getModel().isExceptional()) {
                getJTextComponent().setText("");
            } else {
                getJTextComponent().setText(getPasswordModel().getPassword());
            }
        }
    }

    /**
     * Only for debugging
     */
    public String getViewerName() {
        return editorName;
    }
}
