//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.framework.dataeditor;

import com.nexirius.framework.application.DialogManager;
import com.nexirius.framework.datamodel.DataModel;
import com.nexirius.framework.dataviewer.DataViewer;
import com.nexirius.framework.dataviewer.ViewerCreator;
import com.nexirius.framework.swing.CFJButton;

import javax.swing.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

/**
 * This viewer is designed to show any DataModel in a popup window. the actual component
 * which is displayed is a button which pops up a dialog window.
 *
 * @author Marcel Baumann
 */
public class PopupEditor extends DataViewer {



    public static final String editorName = "PopupEditor";

    ViewerCreator viewerCreator = null;

    public PopupEditor(DataModel model) {
        super(model);
    }

    /**
     * Returns true
     */
    public boolean isEditor() {
        return true;
    }

    /**
     * Need the associated ViewerCreator to create the editor in the dialog window
     */
    public void setViewerCreator(ViewerCreator viewerCreator) {
        this.viewerCreator = viewerCreator;
    }

    /**
     * Returns the associated viewer creator
     */
    public ViewerCreator getViewerCreator() {
        return viewerCreator;
    }

    /**
     * Returns the button component or null (if not created)
     */
    public JButton getShowButton() {
        return (JButton) getJComponent();
    }

    /**
     * Creates the button component
     */
    public void create() {
        setJComponent(new CFJButton(getFactory().getClientResource(), getDataModel().getFieldName()));
        getShowButton().addActionListener(new PopupListener());
    }

    /**
     * @param viewerCreator null (for default editor) or the name of the model associated viewer creator for the editor within th popup window
     */
    public void popup(ViewerCreator viewerCreator) {
        DialogManager.getPopupEditorAdaptor().noDuplicatePopupEdit(getDataModel(), viewerCreator, getLayout());
    }

    /**
     * No implementation
     */
    public void update() {
    }


    /**
     * Only for debugging
     */
    public String getViewerName() {
        return editorName;
    }

    class PopupListener implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            popup(viewerCreator);
        }
    }
}
