//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.framework.dataeditor;

import com.nexirius.framework.datamodel.DataModel;
import com.nexirius.framework.datamodel.DataModelCommandVector;
import com.nexirius.framework.dataviewer.LayoutItem;
import com.nexirius.framework.dataviewer.ViewerCreator;

public interface PopupEditorAdaptor {
    /**
     * @param model
     * @param reuse
     * @return
     * @deprecated
     */
    public boolean popupEdit(DataModel model, boolean reuse);

    /**
     * Show a popup editor with ok and cancel for the specified model
     *
     * @param model This model will be translated into a editor
     * @return
     */
    public boolean popupEdit(DataModel model);

    /**
     * Show a popup editor with ok and cancel for the specified model
     *
     * @param model   This model will be translated into a editor
     * @param creator is used to translate the model (if null then default translation from the ViewerMap is used)
     * @return true if ok was pressed on the dialog window
     */
    public boolean popupEdit(DataModel model, ViewerCreator creator);

    /**
     * @param model   This model will be translated into a editor
     * @param creator is used to translate the model (if null then default translation from the ViewerMap is used)
     * @param layout
     * @param reuse
     * @return true if ok was pressed on the dialog window
     * @deprecated
     */
    public boolean popupEdit(DataModel model, ViewerCreator creator, LayoutItem layout, boolean reuse);

    /**
     * Show a popup editor with ok and cancel for the specified model
     *
     * @param model   This model will be translated into a editor
     * @param creator is used to translate the model (if null then default translation from the ViewerMap is used)
     * @param layout  is used as the viewer creator layout (if null then default is used)
     * @return
     */
    public boolean popupEdit(DataModel model, ViewerCreator creator, LayoutItem layout);

    /**
     * Show a popup editor with ok and cancel for the specified model
     *
     * @param dataModel      the model which will be translated into a dialog window
     * @param creator        the creator (or null for default creator) which translates the model into an editor
     * @param layout         the layout (or null for default or not used by creator)
     * @param duplicateModel duplicate model (show ok and cancel button if true else only close button)
     * @param commands       only these data model commands are displayed as buttons (if null no buttons are visible on the lefthand bottom side) 
     * @return true if ok was pressed on the dialog window
     */
    public boolean popupEdit(DataModel dataModel, ViewerCreator creator, LayoutItem layout, boolean duplicateModel, DataModelCommandVector commands);

    /**
     * Show a popup editor with close for the specified model
     *
     * @param model   the model which will be translated into a dialog window
     * @param creator the creator (or null for default creator) which translates the model into an editor
     * @param layout  the layout (or null for default or not used by creator)
     * @return true if ok was pressed on the dialog window
     */
    public boolean noDuplicatePopupEdit(DataModel model, ViewerCreator creator, LayoutItem layout);
}
