//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.framework.datamodel;

import com.nexirius.util.statemachine.StateMachine;

public class AssistantFormModel extends StructModel {



    public static String STEP_PREV = "assistantStepPrev";
    public static String STEP_NEXT = "assistantStepNext";
    StructModel title;
    StructModel form;
    boolean hasControls = true;
    StepPrevCommand prevCommand;
    StepNextCommand nextCommand;

    public AssistantFormModel(String title, DataModel formModel, String name, boolean hasControls) {
        super(name);
        this.hasControls = hasControls;
        init(title, formModel);
    }

    public AssistantFormModel(String title, DataModel formModel, String name) {
        super(name);
        init(title, formModel);
    }

    public DataModelCommand getPrevCommand() {
        return prevCommand;
    }

    public DataModelCommand getNextCommand() {
        return nextCommand;
    }

    public void setNextCommandResourceKey(String resourceKey) {
        nextCommand.setResourceKey(resourceKey);
    }

    private void init(String titleText, DataModel formModel) {
        StringModel titleModel = new StringModel(titleText, "titleText");

        titleModel.setFlag(ModelFlag.TRANSLATE_VALUE, true);

        title = new StructModel("Title");

        title.append(titleModel);

        appendMethod(prevCommand = new StepPrevCommand());
        appendMethod(nextCommand = new StepNextCommand());

        prevCommand.setEnabled(hasControls);
        nextCommand.setEnabled(hasControls);
        nextCommand.setDefaultButton(hasControls);

        form = new StructModel("Form");

        form.append(formModel);

        append(title);
        append(form);
    }

    public boolean hasControls() {
        return hasControls;
    }

    public AssistantModel getAssistantModel() {
        return (AssistantModel) getParentDataModel();
    }

    public StateMachine getStateMachine() {
        return getAssistantModel().getStateMachine();
    }

    public void stepPrev() {
        try {
            getStateMachine().step(STEP_PREV);
        } catch (Exception ex) {
            throw new RuntimeException(ex);
        }
    }

    public void stepNext() {
        try {
            getStateMachine().getNextState();
        } catch (Exception ex) {
            throw new RuntimeException(ex);
        }
    }

    class StepPrevCommand extends DefaultDataModelCommand {
        public StepPrevCommand() {
            super(STEP_PREV);
        }

        public boolean doAction() {
            stepPrev();
            return(true);
        }
    }

    class StepNextCommand extends DefaultDataModelCommand {
        public StepNextCommand() {
            super(STEP_NEXT);
        }

        public boolean doAction() {
            stepNext();
            return(true);
        }
    }
}
