//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.framework.datamodel;

import com.nexirius.util.CopyPairs;

import java.awt.*;

/**
 * The color model holds an integer between 0 and 0xffffff. The value represents
 * the RGB color values like Color.getRGB() returns
 * * @author Marcel Baumann
 */
public class ColorModel extends LimitIntModel {




    /**
     * initialize color model (field name "Color" value Color.gray)
     */
    public ColorModel() {
        this("Color", Color.gray);
    }

    /**
     * initialize color model (field name "Color")
     *
     * @param col
     */
    public ColorModel(Color col) {
        this("Color", col);
    }

    /**
     * initialize the ColorModel
     *
     * @param name field name
     * @param col  the color
     */
    public ColorModel(String name, Color col) {
        super(col.getRGB() & 0xffffff, 0, 0xffffff, name);
    }

    /**
     * set the color value
     *
     * @param col
     */
    public void setColor(Color col) {
        setInt(col.getRGB() & 0xffffff);
    }

    /**
     * access the color value
     *
     * @return
     */
    public Color getColor() {
        return new Color(getInt());
    }

    /**
     * get the complementary color Color(255 - R, 255 - G, 255 - B)
     *
     * @return
     */
    public Color getComplementColor() {
        Color col = getColor();

        return new Color(255 - col.getRed(), 255 - col.getGreen(), 255 - col.getBlue());
    }

    /**
     * return white or black depending on the sum of the three colors
     * if (R+G+B <  (255+255+255) / 2) return white else black
     *
     * @return
     */
    public Color getTextColor() {
        Color col = getColor();
        int sum = col.getRed() + col.getGreen() + col.getBlue();

        return sum < (255 + 255 + 255) / 2 ? Color.white : Color.black;
    }

    /**
     * return a hex integer value which consists of exactly 6 hex digits
     * 000000 to ffffff
     */
    public String getText() {
        StringBuffer b = new StringBuffer(6);
        String s = Integer.toHexString(getInt());

        for (int i = s.length(); i < 6; ++i) {
            b.append('0');
        }

        b.append(s);

        return b.toString();
    }

    /**
     * accept hex integer value between 0 and ffffff
     * (throws Exception if format is not correct)
     */
    public void setText(String s)
            throws Exception {
        setInt(Integer.parseInt(s, 16));
    }

    /**
     * Creates an exact copy of this instance.
     *
     * @param instance  null (or a defined subclass instance when called from subclass method duplicate)
     * @param copyPairs null (not used)
     */
    public DataModel duplicate(DataModel instance, CopyPairs copyPairs) {
        if (instance == null) {
            instance = new ColorModel(getColor());
        }

        return super.duplicate(instance, copyPairs);
    }
}
