//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.framework.datamodel;

import com.nexirius.util.SortedVector;

import javax.swing.tree.TreeNode;
import java.util.Enumeration;

/**
 * The swing tree interface which eventually is attached to a DataModelContainer element.
 *
 * @author Marcel Baumann
 */
public class ContainerTreeHook extends TreeHook {
    /**
     * Creates a new instance which is hooked to the given DataModelContainer
     *
     * @param container The container class which needs a tree hook instance
     */
    public ContainerTreeHook(DataModelContainer container) {
        super(container);
    }

    /**
     * TreeItem interface implementation
     */
    public TreeNode getChildAt(int childIndex) {
        SortedVector v = getChildTreeHooks();

        if (childIndex < v.size()) {

            return (TreeNode) v.get(childIndex);
        }

        return null;
    }

    /**
     * TreeItem interface implementation
     */
    public int getChildCount() {
        SortedVector v = getChildTreeHooks();

        return v.size();
    }

    /**
     * TreeItem interface implementation
     */
    public int getIndex(TreeNode node) {
        return getChildTreeHooks().indexOf(node);
    }

    /**
     * TreeItem interface implementation
     */
    public boolean getAllowsChildren() {
        return true;
    }

    /**
     * TreeItem interface implementation
     */
    public boolean isLeaf() {
        return getChildTreeHooks().size() == 0;
    }

    /**
     * TreeItem interface implementation
     */
    public Enumeration children() {
        return getChildTreeHooks().elements();
    }

    /**
     * Removes the child at index from the receiver.
     */
    public void remove(int index) {
        throw new RuntimeException("FIX NOT IMPLEMENTED");
//            removeItemAt(index);
    }

    /**
     * Removes node from the receiver.
     */
    public void remove(TreeNode node) {
        throw new RuntimeException("FIX NOT IMPLEMENTED");
//            removeItem((DataModel)node);
    }

    public SortedVector getChildTreeHooks() {
        DataModelEnumeration e = getDataModel().getEnumeration();
        SortedVector v = new SortedVector();

        while (e.hasMore()) {
            DataModel model = e.next();

            if (!model.getFlag(ModelFlag.INVISIBLE_TREE_NODE)) {
                v.addElement(model.getTreeHook());
            }
        }

        return v;
    }
}
