//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.framework.datamodel;

import com.nexirius.util.CopyPairs;
import com.nexirius.util.DoubleRounder;
import com.nexirius.util.DoubleRounderPrecision;
import com.nexirius.util.support.CurrencyManager;

import java.text.DecimalFormat;
import java.text.NumberFormat;


public class CurrencyModel extends StructModel {

    public static final String FIELDNAME_AMOUNT = "Amount";
    public static final String FIELDNAME_CURRENCY_CODE = "CurrencyCode";

    protected RoundedDoubleModel amount;
    protected StringModel currencyCode;

    public CurrencyModel(double amount, String currencyCode, String fieldName) {
        init(amount, currencyCode);
        setFieldName(fieldName);
    }

    public CurrencyModel(double amount, String currencyCode) {
        init(amount, currencyCode);
    }

    public CurrencyModel(double amount) {
        init(amount, "NIL");
    }

    private void init(double amount, String currencyCode) {
        this.amount = new RoundedDoubleModel(amount, null, FIELDNAME_AMOUNT);
        this.currencyCode = new StringModel("", FIELDNAME_CURRENCY_CODE);

        this.currencyCode.addDataModelListener(new CurrencyCodeListener());

        append(this.amount);
        append(this.currencyCode);

        setCurrencyCode(currencyCode);
    }

    public int compare(Object o1, Object o2) {
        return new ArrayModel.AttributeSorter(FIELDNAME_AMOUNT).compare(o1, o2);
    }

    public void setCurrencyCode(String currencyCode) {
        this.currencyCode.setText(currencyCode);
    }

    public String getCurrencyCode() {
        return this.currencyCode.getText();
    }

    public void setAmount(double newAmount) {
        amount.setDouble(newAmount);
    }

    public RoundedDoubleModel getAmountModel() {
        return amount;
    }

    public double getAmount() {
        return amount.getDouble();
    }

    public void newCurrencyCode() {
        amount.setNumberFormat(getFormat());
        amount.setRounder(getRounder());
        amount.fireValueChange();
    }

    public NumberFormat getFormat() {
        NumberFormat format = null;
        String numberFormatString = null;

        try {
            numberFormatString = CurrencyManager.getInstance().getNumberFormat(currencyCode.getText());

            format = new DecimalFormat(numberFormatString);
        } catch (Exception ex) {
            throw new RuntimeException(ex);
        }

        return format;
    }

    public DoubleRounder getRounder() {
        double denominator = 0.01;

        try {
            denominator = CurrencyManager.getInstance().getRoundingDenominator(currencyCode.getText());
        } catch (Exception ex) {
            throw new RuntimeException(ex);
        }

        return new DoubleRounderPrecision(denominator);
    }

    class CurrencyCodeListener extends DataModelAdaptor {
        public void dataModelChangeValue(DataModelEvent event) {
            newCurrencyCode();
        }
    }

    /**
     * Create a copy instance
     */
    public DataModel duplicate(DataModel instance, CopyPairs<DataModel> copyPairs) {
        if (instance == null) {
            instance = new CurrencyModel(amount.getDouble(), currencyCode.getText());
        }

        return super.duplicate(instance, copyPairs);
    }

    /**
     * convenience method to allow amount column in table to display in the color of the parent DataModelContainer
     * @return
     */
    public String getBackgroundColorId() {
        DataModel parent = getParentDataModel();
        return parent != null ? parent.getBackgroundColorId() : super.getBackgroundColorId();
    }
}


