//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.framework.datamodel;

import com.nexirius.util.CopyPairs;
import com.nexirius.util.support.CurrencyManager;


public class CurrencyWithSelectorModel extends CurrencyModel {



    public static final String FIELDNAME_CURRENCY_SELECTOR = "CurrencySelector";

    protected ComboBoxModel currencySelector;

    public CurrencyWithSelectorModel(double amount, String currencyCode, String fieldName) {
        super(amount, currencyCode, fieldName);
        init(currencyCode);
    }

    public CurrencyWithSelectorModel(double amount, String currencyCode) {
        super(amount);
        init(currencyCode);
    }

    private void init(String currencyCode) {
        this.currencySelector = new ComboBoxModel(0, CurrencyManager.getInstance().getCurrencyCodes(), FIELDNAME_CURRENCY_SELECTOR);
        append(this.currencySelector);
        currencySelector.setTransient(true);
        try {
            currencySelector.setText(currencyCode);
        } catch (Exception ex) {
            throw new RuntimeException(ex);
        }
        this.currencySelector.addDataModelListener(new CurrencySelectorListener());
    }

    public void setCurrencyCode(String currencyCode) {
        super.setCurrencyCode(currencyCode);

        if (currencySelector != null) {
            try {
                currencySelector.setText(currencyCode);
            } catch (Exception ex) {
                throw new RuntimeException(ex);
            }
        }
    }

    public ComboBoxModel getCurrencySelectorModel() {
        return currencySelector;
    }

    public void updateCurrencyCode() {
        setCurrencyCode(this.currencySelector.getText());
    }

    public void newCurrencyCode() {
        try {
            super.newCurrencyCode();

            if (currencySelector != null) {
                currencySelector.setText(currencyCode.getText());
            }
        } catch (Exception ex) {
            throw new RuntimeException(ex); //FIX
        }
    }

    class CurrencySelectorListener extends DataModelAdaptor {
        public void dataModelChangeValue(DataModelEvent event) {
            updateCurrencyCode();
        }
    }

    /**
     * Create a copy instance
     */
    public DataModel duplicate(DataModel instance, CopyPairs<DataModel> copyPairs) {
        if (instance == null) {
            instance = new CurrencyWithSelectorModel(amount.getDouble(), currencyCode.getText());
        }

        return super.duplicate(instance, copyPairs);
    }

    /**
     * Create a deep copy of the children list
     *
     * @param copyPairs A list which avoids creating two copies of the same instance (not null)
     */
    public Object cloneValue(CopyPairs copyPairs) {
        return new CurrencyWithSelectorModel(amount.getDouble(), currencyCode.getText()).getValue();
    }

}


