//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.framework.datamodel;

import com.nexirius.util.SortedVector;

/**
 * A list of DataModelCommand items
 *
 * @author Marcel Baumann
 */
public class DataModelCommandVector extends SortedVector<DataModelCommand> {

    private static final long serialVersionUID = -7843027367992089929L;

    public DataModelCommandVector()
    {
    }

    public DataModelCommandVector(DataModelCommand initialCommand)
    {
        if (initialCommand != null)
        {
            append(initialCommand);
        }
    }
    
    /**
     * firstItem, nextItem, lastItem, previousItem are used to iterate the
     * List. Do not use these functions multithreaded or recursively.
     */
    public DataModelCommand firstItem() {
        return (DataModelCommand) firstObject();
    }

    /**
     * firstItem, nextItem, lastItem, previousItem are used to iterate the
     * List. Do not use these functions multithreaded or recursively.
     */
    public DataModelCommand nextItem() {
        return (DataModelCommand) nextObject();
    }

    /**
     * firstItem, nextItem, lastItem, previousItem are used to iterate the
     * List. Do not use these functions multithreaded or recursively.
     */
    public DataModelCommand lastItem() {
        return (DataModelCommand) lastObject();
    }

    /**
     * firstItem, nextItem, lastItem, previousItem are used to iterate the
     * List. Do not use these functions multithreaded or recursively.
     */
    public DataModelCommand previousItem() {
        return (DataModelCommand) previousObject();
    }

    /**
     * Get an status at a specified index.
     *
     * @param index Should be inbetween the limits 0 .. size()-1 (otherwise a runtime exception is thrown)
     * @return the appropriate command
     */
    public DataModelCommand getItem(int index) {
        return (DataModelCommand) elementAt(index);
    }

    /**
     * Add a new command element to the end of the list
     *
     * @param item The newly added command instance
     */
    public void append(DataModelCommand item) {
        addElement(item);
    }

    /**
     * Add a new command instance at a specified position
     */
    public void insert(DataModelCommand item, int position) {
        insertElementAt(item, position);
    }

    /**
     * Insert a specified command instance a the beginning of the list (same as insert(status, 0))
     */
    public void insert(DataModelCommand item) {
        insertElementAt(item, 0);
    }
}
