//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.framework.datamodel;

import java.util.Iterator;

/**
 * This class is an iterator for DataModelVector.
 *
 * @author Marcel Baumann
 */
public class DataModelEnumeration<T extends DataModel> {

    public int size = 0;
    private Iterator<T> iterator = null;

    /**
     * Creates a pseudo enumeration containing no elements
     */
    public DataModelEnumeration() 
    {
    }

    /**
     * Creates an enumeration
     *
     * @param size The number of elements in the list
     * @param e    The actual iterator
     */
    public DataModelEnumeration(int size, Iterator<T> e) {
        this.size = size;
        this.iterator = e;
    }

    /**
     * This method returns false if there are no elements in the list or if the
     * last element was reached.
     */
    public boolean hasMore() {
        if (iterator != null) {
            return iterator.hasNext();
        }

        return false;
    }

    /**
     * Returns the next element from the list (only if hasMore() is true).
     */
    public T next() {
        if (iterator != null) {
            return (T) iterator.next();
        }

        return null;
    }

    /**
     * Returns the next element from the list which has the given flag set.
     */
    public T nextWithFlag(ModelFlag flag) {
        if (iterator != null) {
            while (true) {
                T ret = (T) iterator.next();
                if (ret.getFlag(flag)) {
                    return ret;
                }
                if (!iterator.hasNext()) {
                    return null;
                }
            }
        }

        return null;
    }

    /**
     * Returns the next element from the list which has the given flag not set.
     */
    public T nextWithoutFlag(ModelFlag flag) {
        if (iterator != null) {
            while (true) {
                T ret = (T) iterator.next();
                if (!ret.getFlag(flag)) {
                    return ret;
                }
                if (!iterator.hasNext()) {
                    return null;
                }
            }
        }
        return null;
    }

    public void removeActualItem() {
        iterator.remove();
    }
}
