//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.framework.datamodel;

import java.awt.*;

/**
 * This is the event multicaster class to support the DataModelListener interface.
 */
public class DataModelEventMulticaster
        extends AWTEventMulticaster
        implements DataModelListener {



    /**
     * Constructor to support multicast events.
     *
     * @param a DataModelListener
     * @param b DataModelListener
     */
    protected DataModelEventMulticaster(DataModelListener a, DataModelListener b) {
        super(a, b);
    }

    /**
     * Add new listener to support multicast events.
     *
     * @param a DataModelListener
     * @param b DataModelListener
     * @return DataModelListener
     */
    public static DataModelListener add(DataModelListener a, DataModelListener b) {
        if (a == null) return b;
        if (b == null) return a;
        return new DataModelEventMulticaster(a, b);
    }

    /**
     * Remove listener to support multicast events.
     *
     * @param a DataModelListener
     * @param b DataModelListener
     * @return DataModelListener
     */
    public static DataModelListener remove(DataModelListener a, DataModelListener b) {
        return (DataModelListener) removeInternal(a, b);
    }

    @Override
    public void dataModelChangeValue(DataModelEvent event) {
        ((DataModelListener) a).dataModelChangeValue(event);
        ((DataModelListener) b).dataModelChangeValue(event);
    }

    @Override
    public void dataModelWantToChangeValue(VetoableDataModelEvent event)
    {
        ((DataModelListener) a).dataModelWantToChangeValue(event);
        ((DataModelListener) b).dataModelWantToChangeValue(event);
    }
    
    @Override
    public void dataModelChangeStructure(DataModelEvent event) {
        ((DataModelListener) a).dataModelChangeStructure(event);
        ((DataModelListener) b).dataModelChangeStructure(event);
    }

    @Override
    public void dataModelGrabFocus(DataModelEvent event) {
        ((DataModelListener) a).dataModelGrabFocus(event);
        ((DataModelListener) b).dataModelGrabFocus(event);
    }
}
