//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.framework.datamodel;

import java.util.EventListener;

/**
 * The listener interface for all classes which are interested in changes of data models
 * <pre>
 * event dispatching:
 private void immediatelySendTo(DataModelListener listener) {
     switch (id) {
         case VALUE_CHANGE:
         case CHILD_VALUE_CHANGE:
             listener.dataModelChangeValue(this);
             break;

         case STRUCTURE_CHANGE:
             listener.dataModelChangeStructure(this);
             break;

         case GRAB_FOCUS:
         case GRAB_CHILD_FOCUS:
             listener.dataModelGrabFocus(this);
             break;

         case START_EDIT:
         case VALUE_EDIT:
         case CANCEL_EDIT:
         case FINISH_EDIT:
         case CALL_METHOD:
         case ENABLE_METHOD:
         case DISABLE_METHOD:
         case FORMAT_ERROR:
         case HIGHLIGHT_CHANGED:
         case VIEWER_CREATED:
             if (listener instanceof DataModelEditListener) {
                 ((DataModelEditListener) listener).dataModelEdit(this);
             }
             break;

         default:
             throw new RuntimeException("Unexpected DataModelEvent ID = " + id);
     }
 }
 * </pre>
 */
public interface DataModelListener extends EventListener {
    /**
     * The value of the data model or of one of its children has changed.
     */
    public void dataModelChangeValue(DataModelEvent event);

    /**
     * The value of the data model is about to be changed
     */
    public void dataModelWantToChangeValue(VetoableDataModelEvent event);

    /**
     * The structure (number of children) of a data model structure or of one of its
     * children has changed.
     */
    public void dataModelChangeStructure(DataModelEvent event);

    /**
     * A data model tries to get the input focus.
     */
    public void dataModelGrabFocus(DataModelEvent event);

/*
	public void dataModelChangeValue(DataModelEvent event)
	{
	}

	public void dataModelChangeStructure(DataModelEvent event)
	{
	}
	
	public void dataModelGrabFocus(DataModelEvent event)
	{
	}
*/
}
