//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.framework.datamodel;

import com.nexirius.util.VariableAccessor;
import com.nexirius.util.VariableResolver;

import java.io.*;

/**
 * resolves fieldnames like $(fieldname) within a template input stream.
 * Each attribute of the data model which isappears as $(fieldname) in the input
 * stream will be replaced with its textual representation
 */
public class FormatDataModel {
    public static String DEFAULT_CAHRSET = "UTF-8";


    public static void translate(DataModel model, InputStream templateInputStream, OutputStream out) throws IOException {
        translate(model, templateInputStream, out, DEFAULT_CAHRSET);
    }

    public static void translate(DataModel model, InputStream templateInputStream, OutputStream out, String charset) throws IOException {
        VariableResolver.translate(new DataModelVariableAccessor(model), templateInputStream, out, charset);
    }

    public static String translate(DataModel model, String template) {
        return translate(model, template, DEFAULT_CAHRSET);
    }

    public static String translate(DataModel model, String template, String charset) {
        ByteArrayOutputStream out = new ByteArrayOutputStream();

        try {
            translate(model, new ByteArrayInputStream(template.getBytes(charset)), out, charset);
        } catch (IOException e) {
            throw new RuntimeException(e);
        }

        try {
            return out.toString(charset);
        } catch (UnsupportedEncodingException e) {
            throw new RuntimeException(e);
        }
    }

    static class DataModelVariableAccessor implements VariableAccessor {
        DataModel model;

        public DataModelVariableAccessor(DataModel model) {
            this.model = model;
        }

        public String getValueOf(String var) {
            return model.getChildText(var);
        }
    }
}
