//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 *
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 *
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 *
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 *
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.framework.datamodel;

import java.util.HashSet;
import java.util.Set;

/**
 * A class which represents a named flag which can be added to a data model instance.
 * The flag itself does not hold the state of the flag! It is only used to identify the flag by its name.
 *
 * @author Marcel Baumann
 */
public class ModelFlag {
    public static final ModelFlag ERROR = new ModelFlag("ERROR");
    public static final ModelFlag EMPTY = new ModelFlag("EMPTY");
    public static final ModelFlag CHANGED = new ModelFlag("CHANGED");
    public static final ModelFlag INVALID = new ModelFlag("INVALID");
    public static final ModelFlag TRANSIENT = new ModelFlag("TRANSIENT");
    public static final ModelFlag GRAY = new ModelFlag("GRAY");
    public static final ModelFlag GRAY2 = new ModelFlag("GRAY2");
    public static final ModelFlag REARRANGEABLE = new ModelFlag("REARRANGEABLE");
    public static final ModelFlag INVISIBLE = new ModelFlag("INVISIBLE");
    public static final ModelFlag NOT_EDITABLE = new ModelFlag("NOT_EDITABLE");
    public static final ModelFlag TRANSLATE_VALUE = new ModelFlag("TRANSLATE_VALUE");
    public static final ModelFlag INVISIBLE_TREE_NODE = new ModelFlag("INVISIBLE_TREE_NODE");
    public static final ModelFlag SKIP_TREE_NODE = new ModelFlag("SKIP_TREE_NODE");
    public static final ModelFlag DELETED = new ModelFlag("DELETED");
    public static final ModelFlag MANDATORY = new ModelFlag("MANDATORY");
    public static final ModelFlag LINE_WRAP = new ModelFlag("LINE_WRAP"); // used to switch on/off line wrapping in TextModels (JTextArea.setLineWrap())
    public static final ModelFlag REF = new ModelFlag("REF");
    public static final ModelFlag FLAG_WINDOW_FOCUS = new ModelFlag("WINDOW_FOCUS");

    String name;
    private static Set<String> transientFlags = new HashSet<String>();

    /**
     * Creates a new flag.
     *
     * @param name The name which identifies the flag.
     */
    public ModelFlag(String name) {
        if (name == null) {
            throw new IllegalArgumentException("name must be != null");
        }

        this.name = name;
    }

    public ModelFlag(String name, boolean isTransient) {
        this(name);
        setTransient(name, isTransient);
    }

    public static void setTransient(String flagName, boolean isTransient) {
        if (isTransient) {
            transientFlags.add(flagName);
        } else {
            transientFlags.remove(flagName);
        }
    }

    public static boolean isTransient(String flagName) {
        if (flagName == null) {
            return false;
        }
        return transientFlags.contains(flagName);
    }

    /**
     * Model flags are equal if the have the same name.
     */
    public boolean equals(Object o) {
        if (o == null || !(o instanceof ModelFlag)) {
            return false;
        }

        ModelFlag flag = (ModelFlag) o;

        return flag.getName().equals(getName());
    }

    /**
     * Returns the flag name (identifier)
     */
    public String getName() {
        return this.name;
    }

    /**
     * Returns the has code of the flag name
     */
    public int hashCode() {
        return this.name.hashCode();
    }

    /**
     * Returns the flag name.
     */
    public String toString() {
        return name;
    }
}
