//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.framework.datamodel;

/**
 * This class holds the ok, cancel, close and help comands for the popup dialiog window.
 * 
 * @author Marcel Baumann
 */
public class PopupModel extends StructModel {
    public static String OK_COMMAND = "Ok";
    public static String CANCEL_COMMAND = "Cancel";
    public static String CLOSE_COMMAND = "Close";
    public static String HELP_COMMAND = "Help";

    DefaultDataModelCommand okCommand;
    DefaultDataModelCommand cancelCommand;
    DefaultDataModelCommand closeCommand;
    DefaultDataModelCommand helpCommand;

    public PopupModel(String name) {
        super(name);
        appendMethod(helpCommand = new DialogCommand(HELP_COMMAND));
        appendMethod(closeCommand = new DialogCommand(CLOSE_COMMAND));
        appendMethod(okCommand = new DialogCommand(OK_COMMAND));
        appendMethod(cancelCommand = new DialogCommand(CANCEL_COMMAND));

        okCommand.setDefaultButton(true);
    }

    /**
     * Hide or show dialog buttons
     *
     * @param on If true then only close button will be displayed. Ok and Cancel are not.
     */
    public void setShowCloseOnly(boolean on) {
        okCommand.setVisible(!on);
        cancelCommand.setVisible(!on);
        closeCommand.setVisible(on);

        closeCommand.setDefaultButton(on);
        okCommand.setDefaultButton(!on);
    }

    /**
     * Hide or show help button
     *
     * @param on If true then the help button is visible
     */
    public void setShowHelp(boolean on) {
        helpCommand.setVisible(on);
    }

    /**
     * set the default button
     *
     * @param nameOfCommand if null, then none of the buttons will be default
     */
    public void setDefaultButton(String nameOfCommand) {
        DataModelCommandVector commands = this.getMethods();

        if (commands == null) {

            return;
        }

        for (DataModelCommand command = commands.firstItem(); command != null; command = commands.nextItem()) {
            if (command.getCommandName().equals(nameOfCommand)) {

                command.setDefaultButton(true);
            } else {
                command.setDefaultButton(false);
            }
        }
    }

    /**
     * @return
     */
    public DefaultDataModelCommand getCancelCommand() {
        return cancelCommand;
    }

    /**
     * @return
     */
    public DefaultDataModelCommand getCloseCommand() {
        return closeCommand;
    }

    /**
     * @return
     */
    public DefaultDataModelCommand getHelpCommand() {
        return helpCommand;
    }

    /**
     * @return
     */
    public DefaultDataModelCommand getOkCommand() {
        return okCommand;
    }

    /**
     * Set one of the dialog buttons enabled or not.
     *
     * @param buttonName The name of the button e.g. PopupModel.OK_COMMAND
     * @param on         if true then the button will be enabled
     */
    public void setEnabled(String buttonName, boolean on)
            throws Exception {
        ((DefaultDataModelCommand) getViewableChild(buttonName)).setEnabled(on);
    }

    class DialogCommand extends DefaultDataModelCommand {
        DialogCommand(String name) {
            super(name);
            setSynchronous(true);
        }

        public boolean doAction() {
            // handled in PopupController
            return(true);
        }
    }

    public DataModelCommand getDefaultCommand() {
        DataModelCommandVector commands = this.getMethods();

        if (commands == null) {

            return null;
        }

        for (DataModelCommand command = commands.firstItem(); command != null; command = commands.nextItem()) {
            if (command.isVisible() && command.isDefaultButton()) {

                return command;
            }
        }

        return null;
    }
}
