//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.framework.datamodel;

import com.nexirius.util.CopyPairs;
import com.nexirius.util.TextToken;
import com.nexirius.util.simpletype.SimpleType_String;

/**
 * A class that represents an String information which can be displayed in a text field.
 *
 * @author Marcel Baumann
 */
public class StringModel extends SimpleModel {
    public StringModel() {
        this("");
    }

    /**
     * Creates a new instance and initialize its value
     *
     * @param value The initial value
     */
    public StringModel(String value) {
        super(new SimpleType_String(value));
    }

    /**
     * Creates a new instance and initialize its value and field name
     *
     * @param value     The initial value
     * @param fieldName The initial field name
     */
    public StringModel(String value, String fieldName) {
        super(new SimpleType_String(value), fieldName);
    }

    /**
     * Returns the text which will be displayed in the GUI
     */
    public String getText() {
        return getSimpleType().toString();
    }

    public String getCaption() {
        return getFieldName();
    }

    public int length() {
        return getSimpleType().toString().length();
    }

    public boolean filled() {
        return getSimpleType().toString().length() != 0;
    }

    /**
     * Set the text coming from a GUI component.
     */
    public void setText(String text) {
        if (text == null) {
            setEmpty();
        } else if (text.length() > 0) {
            setEmpty(false);        
        }
        setValue(new SimpleType_String(text));
    }

    /**
     * Returns true if the given test is equal the current value.
     *
     * @param text This string is compared to the current value.
     */
    public boolean isText(String text) {
        return getSimpleType().toString().equals(text);
    }

    /**
     * Creates an exact copy of the instance.
     *
     * @param instance Can be null (normally used by subclass implementation of duplicate method)
     * @param copyPairs Can be null (not used)
     */
    public DataModel duplicate(DataModel instance, CopyPairs copyPairs) {
        if (instance == null) {
            instance = new StringModel(getText());
        }

        return super.duplicate(instance, copyPairs);
    }
}
