//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.framework.datamodel;

import java.util.Date;
import java.util.Calendar;
import java.text.SimpleDateFormat;


public class TimeModel extends IntModel {
    public static final char SEPARATOR_CHAR = ':';

    public TimeModel() {
        this(0);
    }

    /**
     *
     * @param value number of minutes since 0:00
     */
    public TimeModel(int value) {
        super(value);
    }

    /**
     *
     * @param value number of minutes since 0:00
     * @param fieldName
     */
    public TimeModel(int value, String fieldName) {
        super(value, fieldName);
    }

    /**
     *
     * @param date
     * @param fieldName
     */
    public TimeModel(Date date, String fieldName) {
        super(0, fieldName);

        setDate(date);
    }

    public void setDate(Date date) {
        Calendar calendar = Calendar.getInstance();

        calendar.setTime(date);
        int min = calendar.get(Calendar.MINUTE);
        int hour = calendar.get(Calendar.HOUR_OF_DAY);
        setTime(hour, min);
    }

    /**
     * set the time (hour and minute) of a given date and return the result. Second and millisecond will be set to 0.
     * @param date the reference date, which will be used to generate the result
     * @return
     */
    public Date getDate(Date date) {
        Calendar calendar = Calendar.getInstance();

        calendar.setTime(date);
        calendar.set(Calendar.HOUR_OF_DAY, getHours());
        calendar.set(Calendar.MINUTE, getMinutes());
        calendar.set(Calendar.SECOND, 0);
        calendar.set(Calendar.MILLISECOND, 0);

        return calendar.getTime();
    }

    private void setTime(int hour, int min) {
        int value = hour * 60 + min;

        if (value >= 60 * 24) {
            value %= 60 * 24;
        }

        setInt(value);
    }

    public String getText() {
        int v = getInt();

        int min = v % 60;
        int h = v / 60;

        StringBuffer ret = new StringBuffer();

        ret.append(h);
        ret.append(SEPARATOR_CHAR);

        if (min < 10) {
            ret.append('0');
        }

        ret.append(min);

        return ret.toString();
    }

    public void setText(String text) throws Exception {
        int value = 0;
        boolean leadingZero = false;

        for (int i = 0; i < text.length(); ++i) {
            char c = text.charAt(i);

            if (c == '0' && value == 0) {
                leadingZero = true;
            }

            if (Character.isDigit(c)) {
                value *= 10;
                value += c - '0';
            }
        }

        if (!leadingZero) {
            if (value < 24) {
                value *= 100;
            }
        }

        int h = value / 100;

        value = h * 60 + (value % 100);

        value %= 60 * 24;

        setInt(value);
    }

    public String getCaption() {
        return getText();
    }

    public int getHours() {
        return getInt() / 60;
    }

    public int getMinutes() {
        return getInt() % 60;
    }

    /**
     * Access the number of hours and minutes as double value (1:45 is 1.75)
     * @return hours + minutes / 60.0
     */
    public double getDouble() {
        return getHours() + ((double)getMinutes()) / 60.0;
    }

    public void setDouble(double v) {
        setInt((int)(v * 60.0));
    }
}
