//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.framework.datamodel;

import javax.swing.tree.TreeNode;
import javax.swing.tree.TreePath;
import java.util.Enumeration;

/**
 * This class is used to access a data model via a string tree. The associated
 * DataModel instance is just a member of this class.
 */
public class TreeHook implements DataModelTreeNode {
    DataModel dataModel;

    /**
     * Creates an instance which is hooked against the given data model.
     *
     * @param dataModel The wrapped data model instance.
     */
    TreeHook(DataModel dataModel) {
        this.dataModel = dataModel;
    }

    /**
     * Returns the wrapped dta model instance
     */
    public DataModel getDataModel() {
        return dataModel;
    }

    // TreeNode interface
    /**
     * Returns null
     *
     * @see ContainerTreeHook
     */
    public TreeNode getChildAt(int childIndex) {
        return null;
    }

    /**
     * Returns 0
     *
     * @see ContainerTreeHook
     */
    public int getChildCount() {
        return 0;
    }

    /**
     * Returns the parent tree hook or null (for the root status)
     */
    public TreeNode getParent() {
        if (getDataModel().hasParent()) {
            return getDataModel().getParentDataModel().getTreeHook();
        }
        return null;
    }

    /**
     * Returns -1
     *
     * @see ContainerTreeHook
     */
    public int getIndex(TreeNode node) {
        return -1;
    }

    /**
     * Returns false
     *
     * @see ContainerTreeHook
     */
    public boolean getAllowsChildren() {
        return false;
    }

    /**
     * Returns true
     *
     * @see ContainerTreeHook
     */
    public boolean isLeaf() {
        return true;
    }

    /**
     * Returns null
     *
     * @see ContainerTreeHook
     */
    public Enumeration children() {
        return null;
    }

    // Mutable TreeNode interface
    /**
     Adds child to the receiver at index.
     */
/*        public void insert(MutableTreeNode child, int index)
    {
        throw new RuntimeException("Can't insert node");
    }
*/
    /**
     * Removes the child at index from the receiver.
     *
     * @see ContainerTreeHook
     */
    public void remove(int index) {
        throw new RuntimeException("Can't remove node");
    }

    /**
     * Removes node from the receiver.
     *
     * @see ContainerTreeHook
     */
    public void remove(TreeNode node) {
        throw new RuntimeException("Can't remove node");
    }

    /**
     Removes the receiver from its parent.
     */
/*        public void removeFromParent()
    {
        if (hasParent()) {
            parent.getTreeHook().remove(this);
        } else {
            throw new RuntimeException("Can't remove root node");
        }
    }
*/
    /**
     Sets the parent of the receiver to newParent.
     */
/*        public void setParent(MutableTreeNode newParent)
    {
        if (newParent == null) {
            parent = null;
        } else {

            if (newParent instanceof DataModel.TreeHook) {
                parent = (DataModelContainer)((DataModel.TreeHook)newParent).getDataModel();
            } else {
                throw new RuntimeException("Can't assign new parent of type " + newParent.getClass().getName());
            }
        }
    }
*/
    /**
     * Resets the user object of the receiver to object.
     */
    public void setUserObject(Object object) {
        getDataModel().setValue(object);
    }

    // DataModelTreeNodeAttributes interface
    /**
     * @return the label (if getColumns returns <= 1)
     */
    public String getLabel() {
        return dataModel.getLabel();
    }

    /**
     * @return additional information on the status
     */
    public String getInfo() {
        return dataModel.getInfo();
    }

    /**
     * @return the TreePath of this status (an array from root to this)
     */
    public TreePath getTreePath() {
        return dataModel.getTreePath();
    }

    /**
     * @return the number of columns to be displayed
     */
    public int getColumns() {
        return dataModel.getColumns();
    }

    /**
     * @return the label at the specified column index
     */
    public String getLabel(int index) {
        return dataModel.getLabel(index);
    }

    /**
     * @return the id of the icon which will be displayed in the tree
     */
    public String getIconId() {
        return dataModel.getIconId();
    }

    /**
     * @return the overall background color id
     */
    public String getBackgroundColorId() {
        return dataModel.getBackgroundColorId();
    }

    /**
     * @return the column specific background color id
     */
    public String getBackgroundColorId(int index) {
        return dataModel.getBackgroundColorId(index);
    }

    // end DataModelTreeNode Attributes interface
}

