//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 *
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 *
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 *
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 *
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.framework.datamodel;

import com.nexirius.util.CopyPairs;


public class TrendModel extends StructModel {



    public static final String FIELDNAME_TRENDS = "Trends";
    public static final String FIELDNAME_VALUES = "Values";
    public static final String FIELDNAME_MINX = "MinX";
    public static final String FIELDNAME_MINY = "MinY";
    public static final String FIELDNAME_MAXX = "MaxX";
    public static final String FIELDNAME_MAXY = "MaxY";
    public static final String FIELDNAME_STEPX = "StepX";
    public static final String FIELDNAME_STEPY = "StepY";

    public static final Double EMPTY[] = {};
    public static final int maxTrends = 16;

    protected StructModel trends;
    protected DoubleModel minX;
    protected DoubleModel minY;
    protected DoubleModel maxX;
    protected DoubleModel maxY;
    protected DoubleModel stepX;
    protected DoubleModel stepY;

    public TrendModel(TrendModel other) {
        this(other.getFieldName()
                , other.getMinX()
                , other.getMinY()
                , other.getMaxX()
                , other.getMaxY()
                , other.getStepX()
                , other.getStepY());
    }

    public TrendModel(String fieldname, double minx, double miny, double maxx, double maxy, double stepx, double stepy) {
        super(fieldname);
        init(minx, miny, maxx, maxy, stepx, stepy);
    }

    private void init(double minx, double miny, double maxx, double maxy, double stepx, double stepy) {
        this.trends = new StructModel(FIELDNAME_TRENDS);

        for (int i = 0; i < maxTrends; ++i) {
            this.trends.append(new SimpleArrayModel(EMPTY, FIELDNAME_VALUES + i));
        }

        this.minX = new DoubleModel(minx, FIELDNAME_MINX);
        this.minY = new DoubleModel(miny, FIELDNAME_MINY);
        this.maxX = new DoubleModel(maxx, FIELDNAME_MAXX);
        this.maxY = new DoubleModel(maxy, FIELDNAME_MAXY);
        this.stepX = new DoubleModel(stepx, FIELDNAME_STEPX);
        this.stepY = new DoubleModel(stepy, FIELDNAME_STEPY);

        append(this.trends);
        append(this.minX);
        append(this.minY);
        append(this.maxX);
        append(this.maxY);
        append(this.stepX);
        append(this.stepY);
    }

    public double getMinX() {
        return minX.getDouble();
    }

    public double getMinY() {
        return minY.getDouble();
    }

    public double getMaxX() {
        return maxX.getDouble();
    }

    public double getMaxY() {
        return maxY.getDouble();
    }

    public double getStepX() {
        return stepX.getDouble();
    }

    public double getStepY() {
        return stepY.getDouble();
    }

    public void setRangeX(double min, double max, double step) {
        minX.setDouble(min);
        maxX.setDouble(max);
        stepX.setDouble(step);
    }

    public void setRangeY(double min, double max, double step) {
        minY.setDouble(min);
        maxY.setDouble(max);
        stepY.setDouble(step);
    }

    public void setValues(Double v[], int trendNumber) {
        if (trendNumber > maxTrends || trendNumber < 0) {

            return;
        }

        getValues(trendNumber).setArray((Object[])v);
    }

    protected SimpleArrayModel getValues(int n) {
        return (SimpleArrayModel) trends.getChildren().getItem(n);
    }

    public int getMaxTrends() {
        return maxTrends;
    }

    public double[] getNormalized(int trendNumber) {
        if (trendNumber > maxTrends || trendNumber < 0) {

            return new double[0];
        }

        Double a[] = (Double[]) getValues(trendNumber).getArray();
        double m = minY.getDouble();
        double range = (maxY.getDouble() - minY.getDouble());
        int n = a.length;
        double ret[] = new double[n];

        if (range == 0.0) {
            return ret;
        }

        for (int i = 0; i < n; ++i) {
            ret[i] = (a[i].doubleValue() - m) / range;
        }

        return ret;
    }

    /**
     * Creates an exact copy of this.
     *
     * @param instance  null (eventually used by subclasses which define the duplicate method)
     * @param copyPairs A list which avoids creating two copies of the same instance (can be null)
     */
    @Override
    public DataModel duplicate(DataModel instance, CopyPairs<DataModel> copyPairs) {
        if (instance == null) {
            instance = new TrendModel(this);
        }

        return super.duplicate(instance, copyPairs);
    }

}


