package com.nexirius.framework.datamodel.json.format;

import com.nexirius.framework.datamodel.DataModel;
import com.nexirius.framework.datamodel.BooleanModel;
import com.nexirius.framework.datamodel.ModelFlag;
import com.nexirius.framework.datamodel.json.IJsonFormat;
import com.nexirius.framework.datamodel.json.JsonParseException;
import com.nexirius.framework.datamodel.json.DefaultJsonFormatMap;
import com.nexirius.util.TextToken;

import java.io.IOException;
import java.io.OutputStream;
import java.io.PushbackInputStream;

public class BooleanJsonFormat implements IJsonFormat {
    public void parseJson(PushbackInputStream in, DataModel target) throws IOException, JsonParseException {
        BooleanModel booleanModel = (BooleanModel) target;
        TextToken token = TextToken.nextToken(in);

        if (token.isIdentifier("true")) {
            booleanModel.setBoolean(true);
        } else if (token.isIdentifier("false")) {
            booleanModel.setBoolean(false);
        } else if (token.isIdentifier("null")) {
            booleanModel.setEmpty();
        } else {
            throw new JsonParseException(token, "boolean value", "");
        }
    }

    public void writeJson(DataModel source, OutputStream out) throws Exception {
        BooleanModel booleanModel = (BooleanModel)source;

        if (source.getFlag(ModelFlag.EMPTY)) {
            out.write("null".getBytes(DefaultJsonFormatMap.CHARSET));
        } else if (booleanModel.getBoolean()) {
            out.write("true".getBytes(DefaultJsonFormatMap.CHARSET));
        } else {
            out.write("false".getBytes(DefaultJsonFormatMap.CHARSET));
        }
    }
}