package com.nexirius.framework.datamodel.json.format;

import com.nexirius.framework.datamodel.DataModel;
import com.nexirius.framework.datamodel.IntModel;
import com.nexirius.framework.datamodel.ModelFlag;
import com.nexirius.framework.datamodel.json.DefaultJsonFormatMap;
import com.nexirius.framework.datamodel.json.IJsonFormat;
import com.nexirius.framework.datamodel.json.JsonParseException;
import com.nexirius.util.TextToken;

import java.io.IOException;
import java.io.OutputStream;
import java.io.PushbackInputStream;

public class IntJsonFormat implements IJsonFormat {
    public void parseJson(PushbackInputStream in, DataModel target) throws IOException, JsonParseException {
        IntModel intModel = (IntModel)target;
        TextToken token = TextToken.nextToken(in);

        if (token.isInt()) {
            intModel.setInt(token.getInt());
        } else if (token.isIdentifier("null")) {
            intModel.setEmpty();
        } else {
            throw new JsonParseException(token, "int value", "");
        }
    }

    public void writeJson(DataModel source, OutputStream out) throws Exception {
        IntModel intModel = (IntModel)source;

        if (source.getFlag(ModelFlag.EMPTY)) {
            out.write("null".getBytes(DefaultJsonFormatMap.CHARSET));
        } else {
            out.write(Integer.toString(intModel.getInt()).getBytes(DefaultJsonFormatMap.CHARSET));
        }
    }
}