package com.nexirius.framework.datamodel.json.format;

import com.nexirius.framework.datamodel.DataModel;
import com.nexirius.framework.datamodel.ModelFlag;
import com.nexirius.framework.datamodel.StringModel;
import com.nexirius.framework.datamodel.json.DefaultJsonFormatMap;
import com.nexirius.framework.datamodel.json.IJsonFormat;
import com.nexirius.framework.datamodel.json.JsonParseException;
import com.nexirius.util.TextToken;

import java.io.IOException;
import java.io.OutputStream;
import java.io.PushbackInputStream;

public class StringJsonFormat implements IJsonFormat {
    public void parseJson(PushbackInputStream in, DataModel target) throws IOException, JsonParseException {
        StringModel stringModel = (StringModel)target;
        TextToken token = TextToken.nextToken(in);

        if (token.isString()) {
            stringModel.setText(token.getString());
        } else if (token.isIdentifier("null")) {
            stringModel.setEmpty();
        } else {
            throw new JsonParseException(token, "string literal", "");
        }
    }

    public void writeJson(DataModel source, OutputStream out) throws Exception {
        StringModel stringModel = (StringModel)source;

        if (source.getFlag(ModelFlag.EMPTY)) {
            out.write("null".getBytes(DefaultJsonFormatMap.CHARSET));
        } else {
            out.write(TextToken.quoteString(stringModel.getText()).getBytes(DefaultJsonFormatMap.CHARSET));
        }
    }
}