//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.framework.datamodel.parser;

import com.nexirius.framework.datamodel.*;

/**
 * A class which holds any DataModel type
 *
 * @author Marcel Baumann
 */
public abstract class DataModelType {
    public static final String TYPE_STRING = "String";
    public static final String TYPE_INTEGER = "Integer";
    public static final String TYPE_BOOLEAN = "Boolean";
    public static final String TYPE_DATE = "Date";

    String name;
    String memberName;

    public DataModelType(String name) {
        this.name = name;
    }

    public static SimpleModel createPrimitivInstance(String type, String init)
            throws ParserException {
        try {

            if (type.equals(TYPE_STRING)) {

                return new StringModel(init, null);
            } else if (type.equals(TYPE_INTEGER)) {
                IntModel i = new IntModel(0);

                if (init != null) {
                    i.setText(init);
                }

                return i;
            } else if (type.equals(TYPE_BOOLEAN)) {
                BooleanModel b = new BooleanModel(false);

                if (init != null) {
                    b.setText(init);
                }

                return b;
            } else if (type.equals(TYPE_DATE)) {
                DateModel d = new DateModel(new java.util.Date());

                if (init != null) {
                    d.setText(init);
                }
                return d;
            }
        } catch (Exception ex) {
            throw new ParserException("Can't create primitive instance of type '" + type + "': " + ex.getMessage());
        }

        throw new ParserException("Can't create primitive instance of type " + type);
    }

    public void setMemberName(String memberName) {
        this.memberName = memberName;
    }

    public String getName() {
        return name;
    }

    public String getMemberName() {
        return memberName;
    }

    public boolean isArray() {
        return false;
    }

    public boolean isMember() {
        return true;
    }

    public DataModel newInstance(DataModelClasses classes)
            throws ParserException {
        return null;
    }

    public DataModel newInstance(DataModelClasses classes, String init)
            throws ParserException {
        DataModel ret = newInstance(classes);

        if (init != null) {
            try {
                ret.dropData(init);
            } catch (Exception ex) {
                throw new RuntimeException(ex);
            }
        }

        return ret;
    }

    public abstract void appendDTD(StringBuffer sb);
}
