//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.framework.datamodel.persistence;

import com.nexirius.framework.datamodel.DataModel;
import com.nexirius.framework.datamodel.DataModelEnumeration;
import com.nexirius.framework.datamodel.DataModelVector;
import com.nexirius.util.StringVector;

/**
 * This interface is used a for storing and loading datamodels. It also holds
 * a list of PersistenceEnvironment.
 *
 * @author Marcel Baumann
 */
public interface PersistenceManager {
    public PersistenceEnvironment getEnvironment(String name) throws Exception;

    public void addEnvironment(PersistenceEnvironment e) throws Exception;

    public boolean isOpen(String environmentName, String identifier);

    /**
     * Get a model from the list of open models
     *
     * @return null if model is not open
     */
    public DataModel getOpenModel(String environmentName, String identifier);

    public DataModelEnumeration getNotSaved();

    public void store(DataModel model, String environmentName) throws Exception;

    public DataModel load(String environmentName, String identifier) throws Exception;

    /**
     * do not keep in open models
     * @param environmentName
     * @param identifier
     * @return
     */
    public DataModel read(String environmentName, String identifier) throws Exception;

    public void remove(String environmentName, String identifier) throws Exception;

    public void close(String environmentName, DataModel model, boolean looseChanges) throws Exception;

    public void storeAll() throws Exception;

    /**
     * do not keep in open models
     * @param environmentName
     * @return
     * @throws Exception
     */
    public DataModelVector loadAll(String environmentName) throws Exception;

    public StringVector getEnvironments();

    void save(DataModel model, String environment) throws Exception;

    void setVerbose(boolean verbose);
}
