//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.framework.datamodel.persistence;

import com.nexirius.framework.application.DialogManager;
import com.nexirius.framework.datamodel.*;

/**
 * This class holds a reference to another persistant instance which is loaded at runtime
 */
public abstract class PersistenceReferenceModel extends StructModel {
    static PersistenceManager persistenceManager = null;
    DataModel model;
    PersistenceChooserModel chooser;
    StringModel environmentName;
    StringModel identifier;

    public PersistenceReferenceModel() {
        super();
        init(null, null);
    }

    public PersistenceReferenceModel(String environmentName, String identifier, String fieldName) {
        super(fieldName);
        init(environmentName, identifier);
    }

    private void init(String environmentName, String identifier) {
        this.environmentName = new StringModel(environmentName, "environmentName");
        append(this.environmentName);
        this.identifier = new StringModel(identifier, "identifier");
        append(this.identifier);
        this.model = initDataModel();
        model.setTransient(true);
        append(model);

        appendMethod(new DefaultDataModelCommand("popupChooser"));
        this.identifier.addDataModelListener(new IdentifierListener());
    }

    public abstract DataModel initDataModel();

    public static void initPersistenceManager(PersistenceManager pe) {
        persistenceManager = pe;
    }

    public void popupChooser() {
        chooseModel();
    }

    public DataModel getDataModel() {
        if (model == null) {
            updateModel();
        }

        return model;
    }

    public void initChooser() {
        if (chooser == null) {
            chooser = new PersistenceChooserModel(persistenceManager, new SelectOpenAdaptor(), model, environmentName.getText());
        }
    }

    public void chooseModel() {
        initChooser();
        DialogManager.getPopupEditorAdaptor().noDuplicatePopupEdit(chooser, null, null);
    }

    public void updateModel() {
        if (persistenceManager == null) {
            return;
        }

        try {
            DataModel newModel = persistenceManager.load(environmentName.getText(), identifier.getText());

            if (model == null) {
                model = newModel;
            } else {
                model.dropData(newModel.dragData());
                model.setTransient(true);
            }
        } catch (Exception ex) {
            throw new RuntimeException(ex);
        }

        //this.fireValueChange(model);
    }

    class IdentifierListener extends DataModelAdaptor {
        public void dataModelChangeValue(DataModelEvent event) {
            updateModel();
        }
    }

    class SelectOpenAdaptor implements PersistenceOpenAdapter {
        public void openPersistentModel(PersistenceManager manager, String environmentName, String ident) {
            identifier.setText(ident);
        }
    }

    public static PersistenceManager getPersistenceManager() {
        return persistenceManager;
    }

    public DataModel getModel() {
        return model;
    }

    public PersistenceChooserModel getChooser() {
        return chooser;
    }

    public StringModel getEnvironmentName() {
        return environmentName;
    }

    public StringModel getIdentifier() {
        return identifier;
    }

    public void setIdentifier(String identifier) {
        this.identifier.setText(identifier);
    }

    public void setChooser(PersistenceChooserModel chooser) {
        this.chooser = chooser;
    }
}
