//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.framework.dataviewer;

import com.nexirius.framework.datamodel.ArrayModel;
import com.nexirius.framework.datamodel.ModelFlag;
import com.nexirius.framework.datamodel.Viewable;

/**
 * This class is used by the factory to create new instances of type ArrayViewer
 *
 * @author Marcel Baumann
 * @see ArrayViewer
 */
public class ArrayViewerCreator implements ViewerCreator {
    ModelFlag filteredFlag = null;
    ModelFlag selectedFlag = null;
    private boolean showPopupMenus;

    public ArrayViewerCreator() {
    }

    /**
     * If the given filteredFlag is != null then a FilteredArrayViewer will be created.
     *
     * @param filteredFlag
     */
    public ArrayViewerCreator(ModelFlag filteredFlag) {
        this.filteredFlag = filteredFlag;
    }

    /**
     * Creates a filtered array viewer if filteredFlag is != null and creates check box column if selectedFlag is != null
     * @param filteredFlag
     * @param selectedFlag
     */
    public ArrayViewerCreator(ModelFlag filteredFlag, ModelFlag selectedFlag) {
        this.filteredFlag = filteredFlag;
        this.selectedFlag = selectedFlag;
    }

    /**
     * Creates a filtered array viewer if filteredFlag is != null and creates check box column if selectedFlag is != null
     * @param filteredFlag for filtering
     * @param selectedFlag for selection (display checkboxes)
     * @param showPopupMenus for each model the methods are displayed in a poup menu
     */
    public ArrayViewerCreator(ModelFlag filteredFlag, ModelFlag selectedFlag, boolean showPopupMenus) {
        this.filteredFlag = filteredFlag;
        this.selectedFlag = selectedFlag;
        this.showPopupMenus = showPopupMenus;
    }

    public void setSelectedFlag(ModelFlag selectedFlag) {
        this.selectedFlag = selectedFlag;
    }

    /**
     * Returns false (The array viewer is not able to change its viewable values interactively).
     */
    public boolean isEditor() {
        return false;
    }

    /**
     * Returns a new viewer instance which is associated to the given viewable.
     */
    public DataViewer createViewer(Viewable model) {
        if (model == null) {
            throw new IllegalArgumentException("model may not be null");
        }

        if (!(model instanceof ArrayModel)) {
            throw new IllegalArgumentException("model must be of type ArrayModel but have: " + model.getClass().getName());
        }

        ArrayViewer ret;

        if (filteredFlag == null) {
            ret = createArrayViewer((ArrayModel)model);
            //ret = new ArrayViewer((ArrayModel) model);
        } else {
            ret = createFilteredArrayViewer((ArrayModel) model, filteredFlag);
            //ret = new FilteredArrayViewer((ArrayModel) model, filteredFlag);
        }

        ret.setSelectedFlag(selectedFlag);
        ret.setShowPopupMenus(showPopupMenus);

        return ret;
    }
    
    private ArrayViewer createFilteredArrayViewer(ArrayModel model,
            ModelFlag filteredFlag2) {
        
        return new FilteredArrayViewer(model, filteredFlag2);
    }

    protected ArrayViewer createArrayViewer(ArrayModel model) {
        return new ArrayViewer(model);
    }
    
    
}
