package com.nexirius.framework.dataviewer;

import java.util.Stack;

import javax.swing.table.AbstractTableModel;

import com.nexirius.framework.datamodel.*;

public class ArrayViewerTableModel<T extends DataModel> extends AbstractTableModel
{
    private static final long serialVersionUID = -2839240504260296752L;

    protected ArrayViewer<T> arrayViewer;
    private FieldNameVector fields;
    private int columnCount = -1;
    private String columnFieldNames[] = null;

    public ArrayViewerTableModel(ArrayViewer<T> arrayViewer)
    {
        super();

        this.arrayViewer = arrayViewer;

        LayoutItem layout = arrayViewer.getLayout();
        if (layout != null && !(layout instanceof TableLayoutInfo))
        {
            throw new RuntimeException("Expecting TableLayoutInfo but have " + layout.getClass().getName());
        }

//      getFactory().getClientResource().addResourceChangeListener(this);
    }

    public ArrayViewerTableModel(ArrayViewer<T> arrayViewer, ArrayModel<T> model) {
        this.arrayViewer = arrayViewer;
        this.fields = new FieldNameVector();

        model.getMemberFieldNames().getFieldNameVector(this.fields);
    }

    public boolean hasLayout() {
        return arrayViewer.getLayout() != null;
    }

    private TableLayoutInfo getTableLayout() {
        return (TableLayoutInfo) arrayViewer.getLayout();
    }

    @Override
    public int getColumnCount() {
        if (columnCount < 0) {
            if (hasLayout()) {
                columnCount = getTableLayout().getColumnNumber();
            } else {
                columnCount = fields.size();
            }
        }

        return columnCount + (arrayViewer.getSelectedFlag() == null ? 0 : 1);
    }

    @Override
    public int getRowCount() {
        return arrayViewer.getModel().getChildren().size();
    }

    @Override
    public Class<?> getColumnClass(int columnIndex)
    {
        if (hasLayout())
        {
            return getTableLayout().getColumnClass(columnIndex);
        }
        return super.getColumnClass(columnIndex);
    }

    @Override
    public Object getValueAt(int rowIndex, int colIndex)
    {
        try
        {
            DataModel model = getRow(rowIndex);

            return getValueAt(model, colIndex);
        }
        catch (Exception ex)
        {
            throw new RuntimeException(ex);
        }
    }

    public DataModel getRow(int rowIndex)
    {
        try
        {
            DataModel model = arrayViewer.getModel().getChildren().getItem(rowIndex);

            return model;
        }
        catch (Exception ex)
        {
            throw new RuntimeException(ex);
        }

    }

    protected Object getValueAt(DataModel rowDataModel, int colIndex) throws Exception
    {
        if (colIndex < 0)
        {
            return rowDataModel;
        }

        if (arrayViewer.getSelectedFlag() != null)
        {
            if (colIndex == 0)
            {
                return new Boolean(rowDataModel.getFlag(arrayViewer.getSelectedFlag()));
            }
            colIndex -= 1;
        }

        String columnFieldName = getColumnFieldName(colIndex);

        if (rowDataModel.hasMethod(columnFieldName))
        {
            // Methoden/Commands haben keinen eigentlichen Wert ==> Das Model zurück geben!
            return (rowDataModel);
        }

        if (DataModel.FIELD_NAME_INSTANCE_NAME.equals(columnFieldName))
        {
            return rowDataModel.getInstanceName();
        }

        DataModel child = rowDataModel.getChild(columnFieldName);

        return child;
    }

    @Override
    public String getColumnName(int x) {
        if (arrayViewer.getSelectedFlag() != null) {
            if (x == 0) {
                return arrayViewer.getSelectedFlag().getName();
            }
            x -= 1;
        }

        String ret;

        if (hasLayout()) {
            ret = getTableLayout().getColumnLabel(x);

        } else {
            ret = fields.getFieldName(x).toString();
        }

        ret = arrayViewer.getFactory().getText(ret);

        if (ret == null) {

            return "xxx";
        }

        return ret;
    }

    public String getColumnFieldName(int colIndex) {
        if (columnFieldNames != null) {
            if (colIndex < 0 || colIndex >= columnFieldNames.length) {

                return null;
            }

            return columnFieldNames[colIndex];
        }

        columnFieldNames = new String[getColumnCount()];

        for (int i = 0; i < columnCount; ++i) {
            if (hasLayout()) {
                String childName = getTableLayout().getModelFieldName(i);
                columnFieldNames[i] = childName;
            } else {
                FieldName fieldName = fields.getFieldName(i);
                Stack<String> stack = new Stack<String>();

                while (fieldName != null) {
                    stack.push(fieldName.toString());
                    fieldName = fieldName.getParent();
                }

                stack.pop();

                if (stack.size() == 0) {
                    columnFieldNames[i] = null;
                } else {
                    StringBuffer sb = new StringBuffer(stack.pop());

                    while (stack.size() > 0) {
                        String subchildname = stack.pop();

                        sb.append(DataModel.FIELD_NAME_SEPARATOR_CHAR);
                        sb.append(subchildname);
                    }

                    columnFieldNames[i] = sb.toString();
                }
            }
        }

        return getColumnFieldName(colIndex);
    }
}
