//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.framework.dataviewer;

import com.nexirius.framework.dataeditor.*;
import com.nexirius.framework.layout.AssistantFormLayout;
import com.nexirius.framework.layout.DefaultTextSelectorLayout;
import com.nexirius.framework.swing.SwingViewerCreator;

/**
 * The viewer creator map for all the data models which are defined in the framework package.
 * It maps class names to viewer creators.
 *
 * @author Marcel Bauamnn
 */
public class BasicViewerCreatorMap extends ViewerCreatorMap {


    private CreatorMapEntry entries[] =
            {
                    new CreatorMapEntry(com.nexirius.framework.datamodel.ArrayModel.class, new ArrayViewerCreator()),

                    new CreatorMapEntry(com.nexirius.framework.datamodel.BooleanModel.class, new BooleanViewerCreator()),
                    new CreatorMapEntry(com.nexirius.framework.datamodel.BooleanModel.class, new BooleanEditorCreator()),

                    new CreatorMapEntry(com.nexirius.framework.datamodel.ComboBoxModel.class, new ComboBoxViewerCreator()),

                    new CreatorMapEntry(com.nexirius.framework.datamodel.DefaultDataModelCommand.class, new CommandViewerCreator()),

                    new CreatorMapEntry(com.nexirius.framework.datamodel.SimpleArrayModel.class, new SimpleArrayViewerCreator()),

                    new CreatorMapEntry(com.nexirius.framework.datamodel.SimpleModel.class, new SimpleViewerCreator()),
                    new CreatorMapEntry(com.nexirius.framework.datamodel.SimpleModel.class, new SimpleEditorCreator()),

                    new CreatorMapEntry(com.nexirius.framework.datamodel.StructModel.class, new StructViewerCreator()),
                    new CreatorMapEntry(com.nexirius.framework.datamodel.StructModel.class, new StructEditorCreator()),

                    new CreatorMapEntry(com.nexirius.framework.datamodel.PasswordModel.class, new PasswordEditorCreator()),

                    new CreatorMapEntry(com.nexirius.framework.datamodel.FileNameModel.class, new FileNameEditorCreator()),

                    new CreatorMapEntry(com.nexirius.framework.datamodel.DirNameModel.class, new FileNameEditorCreator()),

                    new CreatorMapEntry(com.nexirius.framework.datamodel.TextModel.class, new TextAreaEditorCreator()),

                    new CreatorMapEntry(com.nexirius.framework.datamodel.TrendModel.class, new TrendViewerCreator()),

                    new CreatorMapEntry(com.nexirius.framework.datamodel.AskModel.class, new AskViewerCreator()),

                    new CreatorMapEntry(com.nexirius.framework.datamodel.ImageModel.class, new ImageViewerCreator()),

                    new CreatorMapEntry(com.nexirius.framework.datamodel.AssistantModel.class, new AssistantEditorCreator()),

                    new CreatorMapEntry(com.nexirius.framework.datamodel.AssistantFormModel.class, null, new AssistantFormLayout(), true),

                    new CreatorMapEntry(com.nexirius.framework.textselector.DefaultTextSelector.class, null, new DefaultTextSelectorLayout(), true),

                    new CreatorMapEntry(com.nexirius.framework.datamodel.SelectorModel.class, new SelectorEditorCreator()),

                    new CreatorMapEntry(com.nexirius.framework.datamodel.StepperModel.class, new StepperEditorCreator()),

                    new CreatorMapEntry(com.nexirius.framework.datamodel.ColorModel.class, new SwingViewerCreator(ColorViewer.class)),
                    new CreatorMapEntry(com.nexirius.framework.datamodel.ColorModel.class, new SwingViewerCreator(ColorEditor.class)),

                    new CreatorMapEntry(com.nexirius.framework.datamodel.PopupModel.class, new SwingViewerCreator(PopupSwingViewer.class)),

                    new CreatorMapEntry(com.nexirius.framework.datamodel.DateModel.class, new SwingViewerCreator(DateEditor.class)),
                    new CreatorMapEntry(com.nexirius.framework.datamodel.TimeModel.class, new SwingViewerCreator(TimeEditor.class)),

                    new CreatorMapEntry(com.nexirius.framework.datamodel.DocumentModel.class, new SwingViewerCreator(DocumentViewer.class)),

                    new CreatorMapEntry(com.nexirius.framework.datamodel.ProgressAdaptorModel.class, new SwingViewerCreator(ProgressAdaptorViewer.class)),

                    new CreatorMapEntry(com.nexirius.framework.datamodel.InstanceNameComboBoxModel.class, new ComboBoxViewerCreator()),
            };

    /**
     * Creates and initialises the map
     */
    public BasicViewerCreatorMap() {
        super();

        for (int i = 0; i < entries.length; ++i) {
            register(entries[i]);
        }
    }
}
