//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.framework.dataviewer;

import com.nexirius.framework.datamodel.Viewable;

import java.lang.reflect.Constructor;

/**
 * This class is used as a general ViewerCreator which instanciates DataViewer instances
 * which have a simple public constructor with one argument of the used DataModel type
 * <PRE>
 * use it like this:
 * new DefaultViewerCreator(MyViewer.class, true)
 * </PRE>
 */
public class DefaultViewerCreator implements ViewerCreator {



    Class viewerClass;
    boolean isEditorClass;

    /**
     * This constructor generates a general viewer creator for a given DataViewer class
     *
     * @param viewerClass the class of the DataViewer (this class must have a public constructor taking one argument of the associated DataModel type)
     * @param isEditor    reflects the general behaviour of the DataViewer
     */
    public DefaultViewerCreator(Class viewerClass, boolean isEditor) {
        this.viewerClass = viewerClass;
        this.isEditorClass = isEditor;
    }

    /**
     * reflects the general behaviour of the DataViewer
     *
     * @return true if the associated DataViewer is a editor
     */
    public boolean isEditor() {
        return isEditorClass;
    }

    /**
     * This method is looking for the constructor with one argument of type model.getClass()
     * and then calls constructor.newInstance(param) where param contains a single element (the model) Object array
     *
     * @param model this is the model which should be connected to a newly created DataViewer
     * @return null on failure
     */
    public DataViewer createViewer(Viewable model) {
        try {
            Class param[] = new Class[1];

            param[0] = model.getClass();

            Constructor constructor = viewerClass.getConstructor(param);

            Object args[] = new Object[1];

            args[0] = model;

            return (DataViewer) constructor.newInstance(args);
        } catch (NoSuchMethodException ex0) {
            throw new RuntimeException("No public Constructor which takes one argument of type " + model.getClass().getName() + " in class " + viewerClass.getName());
        } catch (Exception ex1) {
            throw new RuntimeException(ex1);
        }
    }
}
