package com.nexirius.framework.dataviewer;

import java.util.ArrayList;
import java.util.List;

import com.nexirius.framework.datamodel.*;

/**
 * Eine Spezialisierung der  ArrayViewerTableModel Klase, Im Gegensatz zu ArrayViewerTableModel
 * beachtet diese Klasse das 'Invisible' Flag der einzelnen Array Elemente!
 * @author bubenma1
 *
 * @param <T>
 */
public class ExtArrayViewerTableModel<T extends DataModel> extends ArrayViewerTableModel<T>
{
    private static final long serialVersionUID = 6954406652251227349L;

    protected List<T> visibleItems;
    protected int[] viewerToModel;
    protected int[] modelToViewer;

    public ExtArrayViewerTableModel(ArrayViewer<T> arrayViewer)
    {
        super(arrayViewer);

        visibleItems = new ArrayList<T>();

        arrayViewer.getModel().addDataModelListener(new DataModelAdaptor()
        {

            @Override
            public void dataModelChangeValue(DataModelEvent event)
            {
                buildList();
            }
        });

        buildList();
    }

    protected void buildList()
    {
        int hashCode = visibleItems.hashCode();

        visibleItems.clear();
        DataModelVector<T> children = arrayViewer.getModel().getChildren();
        viewerToModel = new int[children.size()];
        modelToViewer = new int[children.size()];

        int viewerIndex = 0;
        int modelIndex = 0;

        for (T dataModel : children)
        {
            if (dataModel.isInvisible() == false)
            {
                visibleItems.add(dataModel);
                viewerToModel[viewerIndex] = modelIndex;
                modelToViewer[modelIndex] = viewerIndex;
                viewerIndex++;
            }
            modelIndex++;
        }

        if (hashCode != visibleItems.hashCode())
        {
            fireTableRowsUpdated(0, getRowCount() - 1);
        }
    }

    public int mapViewerToModel(int viewerIndex)
    {
        int ret = viewerIndex;
        if (viewerIndex >=0 && viewerIndex < viewerToModel.length)
        {
            ret = viewerToModel[viewerIndex];
        }
        return (ret);
    }

    public int mapModelToViewer(int modelIndex)
    {
        int ret = modelIndex;
        if (modelIndex >= 0 && modelIndex < modelToViewer.length)
        {
            ret = modelToViewer[modelIndex];
        }
        return (ret);
    }

    @Override
    public int getRowCount()
    {
        return(visibleItems.size());
    }

    @Override
    public DataModel getRow(int rowIndex)
    {
        if (rowIndex < 0 || rowIndex >= getRowCount())
        {
            // This may happen if this TableModel is used together with a Jide SortableTableModel!
            return (null);
        }

        DataModel model = visibleItems.get(rowIndex);
        return model;
    }

    @Override
    public Object getValueAt(int rowIndex, int colIndex)
    {
        if (rowIndex < 0 || rowIndex >= getRowCount())
        {
            // This may happen if this TableModel is used together with a Jide SortableTableModel!
            return (null);
        }

        try
        {
            DataModel model = getRow(rowIndex);
            return getValueAt(model, colIndex);
        }
        catch (Exception ex)
        {
            throw new RuntimeException(ex);
        }
    }


}
