//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.framework.dataviewer;

import com.nexirius.framework.datamodel.StringModel;
import com.nexirius.framework.datamodel.DataModelEvent;

import javax.swing.*;
import javax.swing.event.HyperlinkListener;
import javax.swing.event.HyperlinkEvent;
import javax.swing.border.EtchedBorder;
import java.awt.*;

/**
 * This viewer is designed to show HTML files
 *
 * @author Marcel Baumann
 */

public class HTMLViewer extends DataViewer {


    public static final String s_viewername = "HTMLViewer";

    protected JEditorPane editorPane = null;
    protected boolean asURL;

    /**
     * @param model The StringModel is used as a file name or as a URL of the HTML file.
     */
    public HTMLViewer(StringModel model, boolean asURL) {
        super(model);
        this.asURL = asURL;
    }

    /**
     * Returns the associated string model
     */
    public StringModel getModel() {
        return (StringModel) getDataModel();
    }

    /**
     * Returns the Swing JPanel which is used to display JEditorPane
     */
    public JPanel getJPanel() {
        return (JPanel) getJComponent();
    }

    /**
     * Creates the actual JPanel
     */
    public void create() {
        JPanel panel = new JPanel(new BorderLayout());

        panel.setBorder(new EtchedBorder());
        setJComponent(panel);
        editorPane = new JEditorPane("text/html", "");
        editorPane.setEditable(false);
        panel.add(BorderLayout.CENTER, editorPane);
        editorPane.setSize(400, 400);
        editorPane.addHyperlinkListener(new MyHyperlinkListener());
        update();
    }

    /**
     * Use the new string value of the assotiated model as an existing file name or as a URL
     * and load and paint the associated image.
     */
    public void update() {
        if (isCreated()) {
            if (getModel().isExceptional() || getModel().getText().length() == 0) {
                editorPane.setText("");
            } else {
                String s = getModel().getText();

                try {
                    if (asURL) {
                        editorPane.setPage(s);
                    } else {
                        editorPane.setText(s);
                    }
                } catch (Exception ex) {
                    throw new RuntimeException(ex);
                }
            }
        }
    }

    /**
     * Only for debugging
     */
    public String getViewerName() {
        return s_viewername;
    }

    /**
     * Not implemented
     */
    public void grabFocus() {
    }

    class MyHyperlinkListener implements HyperlinkListener {

        public void hyperlinkUpdate(HyperlinkEvent e) {
            if (e.getEventType() == HyperlinkEvent.EventType.ACTIVATED) {
                DataModelEvent event = new DataModelEvent(getModel(), DataModelEvent.HYPERLINK);

                event.setTransmitter(getModel());
                event.setData(e.getURL().toString());
                getModel().fireEvent(event);
            }
        }
    }
}
