//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.framework.dataviewer;

import com.nexirius.framework.datamodel.BooleanModel;
import com.nexirius.framework.datamodel.LimitIntModel;
import com.nexirius.framework.datamodel.Viewable;

/**
 * This class is used by the factory to create new instances of type RadioBoxViewer
 *
 * @author Marcel Baumann
 * @see RadioBoxViewer
 */
public class RadioBoxViewerCreator implements ViewerCreator {
    boolean swap = false;
    int hide[] = null;
    boolean horizontal = false;

    public RadioBoxViewerCreator() {
    }

    public RadioBoxViewerCreator(int hide[]) {
        this.hide = hide;
    }

    public RadioBoxViewerCreator(boolean swap) {
        this.swap = swap;
    }

    /**
     * the layout of the radio button (only active for ComboBoxModel, BooleanModels are always horizonal)
     * @param horizontal
     */
    public void setHorizontal(boolean horizontal) {
        this.horizontal = horizontal;
    }

    public boolean isEditor() {
        return true;
    }

    public DataViewer createViewer(Viewable model) {
        if (model == null) {
            throw new IllegalArgumentException("model may not be null");
        }

        if (!(model instanceof LimitIntModel) && !(model instanceof BooleanModel)) {
            throw new IllegalArgumentException("model must be of type LimitIntModel or BooleanModel but have: " + model.getClass().getName());
        }

        if (model instanceof LimitIntModel) {

            RadioBoxViewer radioBoxViewer = new RadioBoxViewer((LimitIntModel) model, hide);
            radioBoxViewer.setHorizontal(horizontal);

            return radioBoxViewer;
        }

        return new RadioBoxViewer((BooleanModel) model, swap);
    }
}
