//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.framework.dataviewer;

import com.nexirius.framework.datamodel.DataModelTreeNode;

import javax.swing.*;
import javax.swing.tree.TreeCellRenderer;
import java.awt.*;

/**
 * This class is used to render a tree node in a swing tree which displays a data model tree node
 *
 * @author Marcel Baumann
 */
public class TreeViewerCellRenderer implements TreeCellRenderer {
    int columnWidth = 200;
    ViewerFactory factory = null;

    public TreeViewerCellRenderer(ViewerFactory factory) {
        this.factory = factory;

        try {
//FIX	this.columnWidth = factory.getInt("TreeColumnWidth");
        } catch (Exception ex) {
            throw new RuntimeException(ex);
        }
    }

    /**
     * Define the width of the columns in pixels
     */
    public void setColumnWidth(int column_width) {
        this.columnWidth = column_width;
    }

    /**
     * Returns the current column width
     */
    public int getColumnWidth() {
        return this.columnWidth;
    }

    /**
     * Returns the text which is eventually locale dependtly translated
     */
    public String getText(String s) {
        return this.factory.getText(s);
    }

    /**
     * Translates a string (resource key) into an icon
     */
    public Icon getIcon(String icon_id) {
        Icon ret = factory.getIcon(icon_id);

        return ret;
    }

    /**
     * Translates a string (resource key) into a color
     */
    public Color getColor(String color_id) {
        return this.factory.getColor(color_id);
    }

    /**
     * Translates a string (resource key) into an font
     */
    public Font getFont(String font_id) {
        return this.factory.getFont(font_id);
    }

    /**
     * This is messaged from JTree whenever it needs to get the size
     * of the component or it wants to draw it.
     */
    public Component getTreeCellRendererComponent(JTree tree
                                                  , Object value
                                                  , boolean selected
                                                  , boolean expanded
                                                  , boolean leaf
                                                  , int row
                                                  , boolean hasFocus) {
        DataModelTreeNode item = (DataModelTreeNode) value;

        return new TreeViewerCellRendererComponent(this
                , item
                , selected
                , expanded
                , leaf
                , row
                , hasFocus);
    }
}
