//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.framework.gadgets;

import com.nexirius.framework.datamodel.BooleanModel;
import com.nexirius.framework.datamodel.DataModelAdaptor;
import com.nexirius.framework.datamodel.DataModelEvent;
import com.nexirius.framework.datamodel.IntModel;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

/**
 * @author Marcel Baumann
 *         <p/>
 *         A panel which lets the user define the actual layout
 */
public class LayoutPanel extends JPanel {

    Component glassPane;
    LayoutPanelModel model = new LayoutPanelModel();

    public LayoutPanel() {
        setLayout(new NullLayout());
        model.addDataModelListener(new ComponentModelListener());
    }

    public void paint(Graphics g) {
        if (glassPane == null) {
            glassPane = new MyGlassPane();
            getRootPane().setGlassPane(glassPane);
//            glassPane.addKeyListener(new MyKeyListener());
            glassPane.addMouseListener(new MyMouseListener());
            glassPane.addMouseMotionListener(new MyMouseMotionListener());
        }

        super.paint(g);
    }

    public Component add(Component component) {
        JComponent jc = (JComponent) component;

        jc.setSize(jc.getPreferredSize());
        model.addComponent(jc);

        return super.add(component, 0);
    }

    public void updateLayout() {
        Component components[] = getComponents();

        for (int i = 0; i < components.length; ++i) {
            model.layout(i, (JComponent) this.getComponent(i));
        }

        repaint();
    }

    public LayoutPanelModel getModel() {
        return model;
    }

    class ComponentModelListener extends DataModelAdaptor {
        public void dataModelChangeValue(DataModelEvent event) {
            System.out.println("CHANGE:" + event);
            if (event.getDataModel() instanceof IntModel) {
                updateLayout();
            }

            if (event.getDataModel().getFieldName().equals("glassPaneVisible")) {
                if (glassPane != null) {
                    glassPane.setVisible(((BooleanModel) event.getDataModel()).getBoolean());
                }
            }
        }

        public void dataModelEdit(DataModelEvent event) {
            System.out.println("EDIT:" + event);
            if (event.getId() == DataModelEvent.CALL_METHOD && event.getData().equals(LayoutPanelModel.TO_TOP_COMMAND)) {
                toTop();
            }
        }
    }

    public void toTop() {
        int index = model.getHighlightedIndex();

        if (index > 0) {
            Component comp = getComponent(index);

            if (comp != null) {
                this.remove(comp);
                model.toTopHighlightedItem();
                this.add(comp, 0);

                repaint();
            }
        }
    }

/*
	class RightMouseListener extends MouseAdapter
	{
		public void mousePressed(MouseEvent event) {
			System.out.println(event);
			
			if (event.getButton() == 3) {
				model.toggleGlass();
			}
		}

		public void mouseReleased(MouseEvent event) {
			System.out.println(event);
		}
	}
*/
    class MyMouseListener extends MouseAdapter {
        public void mouseClicked(MouseEvent event) {
            System.out.println(event);
        }

        public void mousePressed(MouseEvent event) {
			if (event.getButton() == 3) {
				model.toggleGlass();

                return;
			}

            if (event.getButton() == 1) {
                System.out.println(event);
                Point pmouse = new Point(event.getX(), event.getY());
                Point p0 = getLocationOnScreen();
                Point p1 = glassPane.getLocationOnScreen();

                model.setHighlightedComponent(pmouse.x + p1.x - p0.x, pmouse.y + p1.y - p0.y);
                repaint();
            }
        }

        public void mouseReleased(MouseEvent event) {
            System.out.println(event);
        }
    }

    class MyMouseMotionListener implements MouseMotionListener {
        public void mouseDragged(MouseEvent event) {
            System.out.println(event);

            Point pmouse = new Point(event.getX(), event.getY());
            Point p0 = getLocationOnScreen();
            Point p1 = glassPane.getLocationOnScreen();

            model.moveHighlightedComponent(pmouse.x + p1.x - p0.x, pmouse.y + p1.y - p0.y);
        }

        public void mouseMoved(MouseEvent event) {
            //System.out.println(event);
        }
    }

//    class MyKeyListener implements KeyListener {
//        public void keyTyped(KeyEvent e) {
//            if (e.getKeyCode() == KeyEvent.VK_ESCAPE) {
//                getModel().toggleGlass();
//            }
//        }
//
//        public void keyPressed(KeyEvent e) {
//            if (e.getKeyCode() == KeyEvent.VK_ESCAPE) {
//                getModel().toggleGlass();
//            }
//        }
//
//        public void keyReleased(KeyEvent e) {
//            //TODO
//        }
//    }

    class MyGlassPane extends JPanel {
        MyGlassPane() {
            this.setOpaque(false);
        }

        public void paint(Graphics g) {
            Dimension dim = LayoutPanel.this.getSize();
            Point p0 = LayoutPanel.this.getLocationOnScreen();
            Point p1 = getLocationOnScreen();
            Point offset = new Point(p0.x - p1.x, p0.y - p1.y);
            model.paint(g, offset);
            g.setColor(Color.blue);
            g.drawRect(offset.x, offset.y, dim.width - 1, dim.height - 1);
        }
    }
}
