//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.framework.gadgets;

import java.awt.*;
import java.util.Enumeration;
import java.util.Vector;

/**
 * A grid layout where the objects can occupy an arbitrary number
 * of grid cells.
 */
public class SnapGridLayout implements LayoutManager {



    class Comp {
        Component m_c;
        int m_x;
        int m_y;
        int m_w;
        int m_h;

        Comp(Component c, int x, int y, int w, int h) {
            m_c = c;
            m_x = x;
            m_y = y;
            m_w = w;
            m_h = h;
        }

        void reposition(int dx, int dy, int mx, int my) {
            m_c.setBounds(m_x * dx + mx, m_y * dy + my, m_w * dx - 2 * mx, m_h * dy - 2 * my);
        }
    }

    int m_cols;
    int m_rows;
    int m_w;
    int m_h;
    int m_x_margin = 10;
    int m_y_margin = 10;
    Vector m_components = new Vector();

    /**
     * cols and rows define the grid, width and height define the
     * initial size in pixels
     */
    public SnapGridLayout(int cols, int rows, int width, int height) {
        m_cols = cols;
        m_rows = rows;
        m_w = width;
        m_h = height;
    }

    /**
     * this function is not implemented!
     */
    public void addLayoutComponent(String s, Component c) {
    }

    /**
     * adds a component to the layout manager. The size and position of the component is
     * calculated in grid cells!
     */
    public void addLayoutComponent(Component c, int x, int y, int w, int h) {
        m_components.addElement(new Comp(c, x, y, w, h));
    }

    /**
     * remove the component from the layout manager
     */
    public void removeLayoutComponent(Component comp) {
        Enumeration e = m_components.elements();

        for (; e.hasMoreElements();) {
            Comp c = (Comp) e.nextElement();

            if (c.m_c == comp) {
                m_components.removeElement(c);
                break;
            }
        }
    }

    /**
     * @return the initial size
     */
    public Dimension preferredLayoutSize(Container parent) {
        return new Dimension(m_w, m_h);
    }

    /**
     * @return the initial size
     */
    public Dimension minimumLayoutSize(Container parent) {
        return new Dimension(m_w, m_h);
    }

    /**
     * Lays out the container in the specified panel.
     *
     * @param parent the specified component being laid out
     * @see Container
     */
    public void layoutContainer(Container parent) {
        Dimension dim = parent.getSize();
        int dx = dim.width / m_cols;
        int dy = dim.height / m_rows;
        int mx = (dx * m_x_margin) / 100;
        int my = (dy * m_y_margin) / 100;
        Enumeration e = m_components.elements();

        for (; e.hasMoreElements();) {
            Comp c = (Comp) e.nextElement();

            c.reposition(dx, dy, mx, my);
        }
    }

    /**
     * Sets the margin between the registered components.
     * mx and my are percent values (percent of the width
     * or height of one grid cell!).
     */
    public void setMargins(int mx, int my) {
        m_x_margin = mx;
        m_y_margin = my;
    }

    /**
     * Returns the String representation of this SnapGridLayout's values.
     */
    public String toString() {
        return getClass().getName() + "[rows=" + m_rows + ",cols=" + m_cols + "]";
    }
}
