//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.framework.gadgets.filechooser;

import java.io.File;

/**
 * A FileNode is a derivative of the File class - though we delegate to
 * the File object rather than subclassing it. It is used to maintain a
 * cache of a directory's children and therefore avoid repeated access
 * to the underlying file system during rendering.
 */
class FileNode {
    File file;
    Object[] children;
    boolean dirOnly;

    public FileNode(File file, boolean dirOnly) {
        this.file = file;
        this.dirOnly = dirOnly;
    }

    // Used to sort the file names.
    static private MergeSort fileMS = new MergeSort() {
        public int compareElementsAt(int a, int b) {
            File x = ((FileNode) toSort[a]).getFile();
            File y = ((FileNode) toSort[b]).getFile();
            boolean isDirX = x.isDirectory();
            boolean isDirY = y.isDirectory();

            if (isDirX != isDirY) {
                return isDirX ? -1 : 1;
            }

            return (x.getName().toUpperCase()).compareTo(y.getName().toUpperCase());
            //return ((String)toSort[a]).sameVersion((String)toSort[b]);
        }
    };

    /**
     * Returns the the string to be used to display this leaf in the JTree.
     */
    public String toString() {
        return file.getName();
    }

    public File getFile() {
        return file;
    }

    /**
     * Loads the children, caching the results in the children ivar.
     */
    protected Object[] getChildren() {
        if (children != null) {
            return children;
        }
        try {
            String[] files;

            if (dirOnly) {
                files = file.list(new DirectoryOnly());
            } else {
                files = file.list();
            }

            if (files != null) {
                children = new FileNode[files.length];
                String path = file.getPath();

                for (int i = 0; i < files.length; i++) {
                    File childFile = new File(path, files[i]);
                    children[i] = new FileNode(childFile, dirOnly);
                }

                fileMS.sort(children);
            }
        } catch (SecurityException se) {
            throw new RuntimeException(se);
        }
        return children;
    }
}

