//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.framework.gadgets.filechooser;

import javax.swing.tree.TreePath;
import java.io.File;
import java.util.Date;
import java.util.StringTokenizer;
import java.util.Vector;

/**
 * FileSystemModel is a TreeTableModel representing a hierarchical file
 * system. Nodes in the FileSystemModel are FileNodes which, when they
 * are directory nodes, cache their children to avoid repeatedly querying
 * the real file system.
 */

public class FileSystemModel extends AbstractTreeTableModel
        implements TreeTableModel {




    // Names of the columns.
    static protected String[] cNames = {"Name", "Type", "Modified", "Size"};

    // Types of the columns.
    static protected Class[] cTypes = {TreeTableModel.class, String.class, Date.class, Integer.class};

    // The the returned file length for directories.
    public static final Integer ZERO = new Integer(0);

    boolean dirOnly = false;

    public FileSystemModel(File root, boolean dirOnly) {
        super(new FileNode(root, dirOnly));
        this.dirOnly = dirOnly;
    }

    //
    // Some convenience methods.
    //
    public TreePath getPathFor(String fileName) {
        StringTokenizer st = new StringTokenizer(fileName, File.separator);
        FileNode currentNode = (FileNode) getRoot();
        Vector path = new Vector();
        String currentName = st.nextToken();

/*
		if (!currentName.equals(currentNode.getFile().getName())) {

			return null;
		}
*/
        path.addElement(currentNode);

        while (st.hasMoreElements()) {
            FileNode nodes[] = (FileNode[]) currentNode.getChildren();
            currentName = st.nextToken();
            currentNode = null;

            for (int i = 0; i < nodes.length; ++i) {

                if (nodes[i].getFile().getName().equals(currentName)) {
                    currentNode = nodes[i];

                    break;
                }
            }

            if (currentNode == null) {

                return null;
            }

            path.addElement(currentNode);
        }

        return new TreePath(path.toArray());
    }

    protected File getFile(Object node) {
        FileNode fileNode = ((FileNode) node);
        return fileNode.getFile();
    }

    protected Object[] getChildren(Object node) {
        FileNode fileNode = ((FileNode) node);
        return fileNode.getChildren();
    }

    //
    // The TreeModel interface
    //

    public int getChildCount(Object node) {
        Object[] children = getChildren(node);
        return (children == null) ? 0 : children.length;
    }

    public Object getChild(Object node, int i) {
        return getChildren(node)[i];
    }

    // The superclass's implementation would work, but this is more efficient.
    public boolean isLeaf(Object node) {
        return !getFile(node).isDirectory();
    }

    //
    //  The TreeTableNode interface.
    //

    public int getColumnCount() {
        return cNames.length;
    }

    public String getColumnName(int column) {
        return cNames[column];
    }

    public Class getColumnClass(int column) {
        return cTypes[column];
    }

    public Object getValueAt(Object node, int column) {
        File file = getFile(node);
        try {
            switch (column) {
                case 0:
                    return file.isDirectory() ? file.getName() + File.separator : file.getName();
                case 1:
                    return file.isFile() ? (file.canWrite() ? "File" : "READ ONLY") : (file.isDirectory() ? "Directory" : "UNKNOWN");
                case 2:
                    return new Date(file.lastModified());
                case 3:
                    return file.isFile() ? new Integer((int) file.length()) : ZERO;
            }
        } catch (SecurityException se) {
            throw new RuntimeException(se);
        }

        return null;
    }
}


