//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.framework.htmlview;

import com.nexirius.framework.datamodel.DataModel;

import java.util.Hashtable;

/**
 * This class is used to find a DataModelVariableCreator which is registered apon its class name
 */
public class DataModelVariableHandler {



    static Hashtable dataModelVariableCreatorTable = new Hashtable();
    static DataModelVariableCreator defaultCreator = new DataModelVariableCreator();

    /**
     * stores all needed variables into a VariableStore and saves the original values in the old values store
     *
     * @param model
     * @param variableStore the target variable store
     * @param oldValues     where the old values are saved to
     * @param translator    the translator which is used to translate values
     */
    public static void setVariablesFor(DataModel model, VariableStore variableStore, VariableStore oldValues, HTMLTranslator translator) {
        // look if there is a handler registered for the specific type
        DataModelVariableCreator creator = (DataModelVariableCreator) dataModelVariableCreatorTable.get(model.getClass());

        if (creator == null) {
            creator = defaultCreator;
        }

        creator.setVariablesFor(model, variableStore, oldValues, translator);
    }

    /**
     * registerTemplate a variable creator for a specific data model class. The calss for which a variable creator is registered
     * is extracted from the creator with the getDataModelClass() method.
     *
     * @param creator the variable creator which will be registered
     */
    public void register(DataModelVariableCreator creator) {
        dataModelVariableCreatorTable.put(creator.getDataModelClass(), creator);
    }
}
