//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.framework.htmlview.function;

import com.nexirius.framework.datamodel.DataModel;
import com.nexirius.framework.htmlview.HTMLParser;
import com.nexirius.framework.htmlview.HTMLSessionVariable;

/**
 * This class handles the $!field("childname", view|edit, "template") function
 */
public class FieldHTMLFunction extends DefaultHTMLFunction {

    public String getFunctionName() {
        return "field";
    }

    /**
     * access the first parameter
     *
     */
    public String getChildName() {
        return getArgument(0);
    }

    /**
     * access the second parameter
     *
     * @return true for edit and false for view (else null)
     */
    public Boolean getIsEditor() {
        String arg = getArgument(1);

        if (arg == null) {
            return null;
        }

        if (arg.equalsIgnoreCase(HTMLParser.EDIT)) {
            return new Boolean(true);
        }

        if (arg.equalsIgnoreCase(HTMLParser.VIEW)) {
            return new Boolean(false);
        }

        return null;
    }

    /**
     * access the third parameter
     *
     */
    public String getTemplate() {
        return getArgument(2);
    }

    /**
     * translate the function into HTML and write the result back to the parser
     *
     * @param sessionVariable
     * @param parser the parser which translates the embedded commands
     * @throws java.lang.Exception
     */
    public void translate(HTMLSessionVariable sessionVariable, HTMLParser parser)
            throws Exception {
        String childName = getChildName();
        String template = getTemplate();
        Boolean type = getIsEditor();
        boolean childIsEditor = parser.isEditor();
        DataModel childModel = null;

        if (type != null) {
            childIsEditor = type.booleanValue();
        }

        DataModel model = parser.getModel();

        if (childName == null || childName.equals("this")) {
            childModel = model;
        } else if (model != null) {
            childModel = model.getChild(childName);
        }

        byte result[] = parser.getResolver().resolve(sessionVariable, childModel, template, childIsEditor);

        parser.getOut().write(result);
    }
}

