//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.framework.htmlview.function;

import com.nexirius.framework.htmlview.HTMLCommand;
import com.nexirius.framework.htmlview.HTMLState;

public class HTMLTransition {

    private HTMLState state;
    private String event;
    private HTMLCommand command;
    private HTMLState targetState;

    /**
     * Create a transition, which reacts on special events, which are not assigned to a specific state
     * @param event
     * @param command
     */
    public HTMLTransition(String event, HTMLCommand command) {
        this.state = null;
        this.event = event;
        this.command = command;
        this.targetState = null;
    }

    /**
     * Create a transition, which reacts on special events, which are not assigned to a specific state
     * @param event
     * @param command
     * @param targetState
     */
    public HTMLTransition(String event, HTMLCommand command, HTMLState targetState) {
        this.state = null;
        this.event = event;
        this.command = command;
        this.targetState = targetState;
    }

    /**
     * State transition with command (the transition is only performed if the command returns true)
     * mapping only when command needs it (requiresMapping())
     * @param state
     * @param event
     * @param command
     * @param targetState
     */
    public HTMLTransition(HTMLState state, String event, HTMLCommand command, HTMLState targetState) {
        this.state = state;
        this.event = event;
        this.command = command;
        this.targetState = targetState;
    }

    /**
     * State transition without command (no mapping)
     * @param state
     * @param event
     * @param targetState
     */
    public HTMLTransition(HTMLState state, String event, HTMLState targetState) {
        this(state, event, null, targetState);
    }

    public HTMLState getState() {
        return state;
    }

    public String getEvent() {
        return event;
    }

    public HTMLCommand getCommand() {
        return command;
    }

    public HTMLState getTargetState() {
        return targetState;
    }
}
