//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 *
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 *
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 *
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 *
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.framework.htmlview.function;

import com.nexirius.framework.htmlview.HTMLParser;
import com.nexirius.framework.htmlview.HTMLResolver;
import com.nexirius.framework.htmlview.HTMLSessionVariable;

/**
 * This class handles the $!IF("true or false parameter")---thenBlock---$ELSE$----elseBlock----$ENDIF$ function
 * <p/>
 * example: $!IF("$!isBold()")<b>bold</b>$ELSE$plain$ENDIF$
 * returns "<b>bold</b>" or "plain"
 */
public class IfHTMLFunction extends DefaultHTMLFunction {
    public static final String FUNCTION_NAME = "IF";
    public static final String ELSE = "$ELSE$";
    public static final String ENDIF = "$ENDIF$";

    public String getFunctionName() {
        return FUNCTION_NAME;
    }

    /**
     * access the first parameter
     *
     * @return
     */
    public boolean getCondition() {
        return getArgument(0).equalsIgnoreCase("true") || getArgument(0).equalsIgnoreCase("1");
    }

    /**
     * access the second parameter
     *
     * @return
     */
    public String getIfBlock() {
        return getArgument(1);
    }

    /**
     * access the third parameter
     *
     * @return
     */
    public String getElseBlock() {
        return getArgument(2);
    }

    /**
     * translate the function into HTML and write the result back to the parser
     *
     * @param sessionVariable
     * @param parser          the parser which translates the embedded commands
     * @throws Exception
     */
    public void translate(HTMLSessionVariable sessionVariable, HTMLParser parser)
            throws Exception {
        String result = getCondition() ? getIfBlock() : getElseBlock();

        result = parser.resolve(sessionVariable, result);

        parser.getOut().write(result.getBytes(HTMLResolver.CHARSET));
    }
}
