//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.framework.htmlview.function;

import com.nexirius.framework.htmlview.HTMLParser;
import com.nexirius.framework.htmlview.HTMLResolver;
import com.nexirius.framework.htmlview.HTMLSessionVariable;

/**
 * creates a checkbox and initializes a HTML variable called selected@[childname]
 * this class handles $!select("childname"[, checked]) functions. If the checked parameter is present then the checkbox
 * will be displayed as a ticked square.
 * To access the list of selelcted children use the method getSelectedChildren() from HTMLSessionVariable
 */
public class SelectHTMLFunction extends DefaultHTMLFunction {

    public String getFunctionName() {
        return "select";
    }

    /**
     * access the first parameter
     *
     */
    public String getChildName() {
        return getArgument(0);
    }

    public boolean getCheckedFlag() {
        String argument = getArgument(1);

        if (HTMLParser.CHECKED.equalsIgnoreCase(argument)) {
            return true;
        }

        return false;
    }

    public void translate(HTMLSessionVariable sessionVariable, HTMLParser parser)
            throws Exception {
        String childName = getChildName();

        childName = getFullChildName(parser, childName);

        String param = sessionVariable.getRequestParameterForFieldName(childName);

        StringBuffer buf = new StringBuffer();

        buf.append("<INPUT TYPE='checkbox' name='");
        buf.append(PARAMETER_SELECTED);
        buf.append(param);
        buf.append("'");
        if (getCheckedFlag()) {
            buf.append(" CHECKED");
        }
        buf.append("></INPUT> ");

        parser.getOut().write(buf.toString().getBytes(HTMLResolver.CHARSET));
    }
}
