//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.framework.jdbc;

import com.nexirius.framework.datamodel.DataModel;
import com.nexirius.util.StringVector;

import java.sql.ResultSet;
import java.sql.SQLException;

/**
 * Translates DataModel attributes into sql command parameters or translates ResultSet columns into DataModel attributes.
 */
public interface DatabaseFieldTranslator {
    /**
     * Access the list of database fields, which is managed by this translator
     */
    public StringVector getFieldList();

    /**
     * This method is called before the execute method of the given statement is called. Each parameter of the field array must be
     * set by this method (e.g. statement.setInt(indexOfFirstField + 1, themodel.getValue())
     *
     * @param mapping           the actual mapping which is calling this method
     * @param statement         the statement which neeed additional parameters (use statement.set...()
     * @param indexOfFirstField the index of the first field which is to be mapped by this method
     * @param model             the model from ehich the attributes are taken (and put into the statement)
     * @throws SQLException
     */
    public void setModelValues(DatabaseTableMapping mapping, JnexPreparedStatement statement, int indexOfFirstField, DataModel model) throws SQLException;

    /**
     * this method is called by the DatabaseTableMapping to translate ResultSet columns into model attributes.
     *
     * @param mapping           the actual mapping, which called this method
     * @param resultSet         the Result Set which holds the fields which need to be translated into the model instance (use resultSet.get...())
     * @param indexOfFirstField the index of th first field in the ResultSet
     * @param model             the model which geta the field values from the ResultSet
     * @throws SQLException
     */
    public void getModelValues(DatabaseTableMapping mapping, ResultSet resultSet, int indexOfFirstField, DataModel model) throws SQLException;
}
