//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.framework.jdbc;

import com.nexirius.framework.datamodel.DataModel;
import com.nexirius.util.StringVector;

import java.sql.ResultSet;
import java.sql.SQLException;

/**
 * Maps attributes from the DataModel to a single field in the DB table
 */
public abstract class SingleFieldTranslator implements DatabaseFieldTranslator {
    StringVector fieldNames;

    public SingleFieldTranslator(String fieldName) {
        fieldNames = new StringVector(new String[]{fieldName});
    }

    public StringVector getFieldList() {
        return fieldNames;
    }

    public void setModelValues(DatabaseTableMapping mapping, JnexPreparedStatement statement, int indexOfFirstField, DataModel model) throws SQLException {
        String s = getModelAttribute(model);

        statement.setString(indexOfFirstField, s, false);
    }

    public void getModelValues(DatabaseTableMapping mapping, ResultSet resultSet, int indexOfFirstField, DataModel model) throws SQLException {
        String s = resultSet.getString(indexOfFirstField);

        setModelAttribute(model, s);
    }

    public abstract String getModelAttribute(DataModel model);

    public abstract void setModelAttribute(DataModel model, String value);
}
