//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.framework.layout;

import com.nexirius.framework.FWLog;

import javax.swing.*;
import java.awt.*;
import java.lang.reflect.Method;

/**
 * This class is used to set the Alignment of a SWING JComponent.
 *
 * @author Marcel Baumann
 */
public class AlignmentChanger implements PropertyChanger {



    public static final String LEFT = "-1";
    public static final String CENTER = "0";
    public static final String RIGHT = "1";

    int alignment = SwingConstants.LEFT;

    static class TableEntry {
        public int value;
        public String text;

        TableEntry(int value, String text) {
            this.value = value;
            this.text = text;
        }
    }

    private static final TableEntry table[] = {
        new TableEntry(SwingConstants.LEFT, LEFT),
        new TableEntry(SwingConstants.CENTER, CENTER),
        new TableEntry(SwingConstants.RIGHT, RIGHT),
        new TableEntry(SwingConstants.LEFT, "LEFT"),
        new TableEntry(SwingConstants.CENTER, "CENTER"),
        new TableEntry(SwingConstants.RIGHT, "RIGHT"),
    };

    /**
     * @param s Accepts LEFT, RIGTH and CENTER (default is LEFT)
     */
    public AlignmentChanger(String s) {
        for (int i = 0; i < table.length; ++i) {
            if (table[i].text.equals(s)) {
                alignment = table[i].value;
                break;
            }
        }
    }

    /**
     * @param alignment Is one of SwingConstants.LEFT, SwingConstants.CENTER, SwingConstants.RIGHT
     */
    public AlignmentChanger(int alignment) {
        this.alignment = alignment;
    }

    /**
     * Set the horizontal alignment of the component (only if a method
     * setHorizontalAligment(int) is defined for the component)
     */
    public void change(JComponent jcomp) {
        Component comp = jcomp;

        if (jcomp instanceof JScrollPane) {
            comp = ((JScrollPane) comp).getViewport().getView();
        }

        // see if the component has a function called setHorizontalAlignment
        try {
            String name = "setHorizontalAlignment";
            Method meth[] = comp.getClass().getMethods();
            boolean found = false;

            for (int i = 0; i < meth.length; ++i) {
                if (meth[i].getName().equals(name)) {
                    Object par[] = {new Integer(alignment)};

                    found = true;
                    meth[i].invoke(comp, par);
                    break;
                }
            }

            if (!found) {
                FWLog.debug("METHOD " + name + " not found in " + comp.getClass().getName());
            }
        } catch (Exception ex) {
            throw new RuntimeException(ex);
        }
    }
}
