//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.framework.layout;

import java.awt.*;

/**
 * This layout manager is resizing to the lower right corner of the
 * outer most component. Each component is layed out according to its
 * preferred size.
 *
 * @author Marcel Baumann
 */
public class DefaultLayoutManager implements LayoutManager {



    public DefaultLayoutManager() {
    }

    /**
     * No implementation
     */
    public void addLayoutComponent(String s, Component c) {
    }

    /**
     * No implementation
     */
    public void removeLayoutComponent(Component c) {
    }

    /**
     * Calculates the size of the layout using the preferred size and location of all the
     * subcomponents and add their bounding rectangles.
     *
     * @return the initial size
     */
    public Dimension preferredLayoutSize(Container parent) {
        Rectangle rect = new Rectangle(10, 10);
        Component comp[] = parent.getComponents();

        if (comp == null || comp.length == 0) {
            return minimumLayoutSize(null);
        }

        for (int i = 0; i < comp.length; ++i) {
            Rectangle r = comp[i].getBounds();
            r.setSize(comp[i].getPreferredSize());
            rect.add(r);
        }

        return new Dimension(rect.width + 10, rect.height);
    }

    /**
     * @return The minimal size (returns Dimension(10, 10))
     */
    public Dimension minimumLayoutSize(Container parent) {
        return new Dimension(10, 10);
    }

    /**
     * Lays out the container in the specified panel.
     *
     * @param parent the specified component being laid out
     * @see Container
     */
    public void layoutContainer(Container parent) {
        Component comp[] = parent.getComponents();

        for (int i = 0; i < comp.length; ++i) {
            comp[i].setBounds(comp[i].getBounds());
        }
    }
}
