//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.framework.layout;

import javax.swing.*;
import java.lang.reflect.Method;


/**
 * This class is used to call a specific method of a SWING JComponent
 *
 * @author Marcel Baumann
 */
public class DefaultPropertyChanger implements PropertyChanger {



    String methodName;
    Object params[];

    public DefaultPropertyChanger(String methodName, Object params[]) {
        this.methodName = methodName;
        this.params = params;
    }

    /**
     * Set the property of the given component
     */
    public void change(JComponent comp) {
        try {
            if (comp instanceof JScrollPane) {
                comp = (JComponent) ((JScrollPane) comp).getViewport().getView();
            }

            Method method = null;

            if (params == null || params.length == 0) {
                method = comp.getClass().getMethod(methodName, (Class<?>)null);
            } else {
                Method methods[] = comp.getClass().getMethods();

                for (int i = 0; i < methods.length; ++i) {
                    if (methods[i].getName().equals(methodName) && methods[i].getParameterTypes().length == params.length) {
                        method = methods[i];

                        break;
                    }
                }
            }

            if (method == null) {
                throw new Exception("No such method");
            }

            method.invoke(comp, params);
        } catch (Exception ex) {
            throw new RuntimeException("DefaultPropertyChanger is trying to invoke method " + methodName + " on " + comp.getClass().getName());
        }
    }
}
