//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.framework.layout;

import com.nexirius.framework.dataviewer.ViewerCreator;

import java.awt.*;

/**
 * This class defines the layout of a structure field in a gridbag layout container
 *
 * @author Marcel Baumann
 */
public class GridBagLayoutItem extends StructureFieldLayout {



    /**
     * @param field         The associated structure field name
     * @param x             the gridx position of the component
     * @param y             the gridy position of the component
     * @param w             the gridwidth of the component
     * @param h             the gridheight of the component
     * @param viewerCreator the viewer creator which is used to create the component or null (default viewer from the model)
     * @param layout        the layout which is used to layout the component or null (default layout from the model)
     * @param prop          a SinglePropertyEntry or a MultiplePropertyEntry
     * @param constraints   the GridBagConstraints
     * @see SinglePropertyEntry
     * @see MultiplePropertyEntry
     */
    public GridBagLayoutItem(String field, int x, int y, int w, int h, ViewerCreator viewerCreator, DefaultLayoutItem layout, boolean createLabel, String viewerId, PropertyEntry prop, GridBagConstraints constraints) {
        super(field, 0, 0, 0, 0, viewerCreator, layout, createLabel, viewerId, prop);

        if (constraints != null) {
            constraints = (GridBagConstraints) constraints.clone();

            if (x >= 0) {
                constraints.gridx = x;
            }

            if (y >= 0) {
                constraints.gridy = y;
            }

            if (w >= 0) {
                constraints.gridwidth = w;
            }

            if (h >= 0) {
                constraints.gridheight = h;
            }

            setGridBagConstraints(constraints);
        }
    }
}
