//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.framework.layout;

import com.nexirius.framework.datamodel.DataModel;
import com.nexirius.framework.dataviewer.DataViewer;
import com.nexirius.framework.swing.CFJLabel;

import javax.swing.*;

/**
 * Based on a resourceId this class creates a label (JLabel)
 */
public class LabelLayoutItem extends DefaultLayoutItem {



    public LabelLayoutItem() {
        super();
    }

    /**
     * @param labelResourceId The associated resource ID which is associated with the created label
     * @param x               the x position of the component
     * @param y               the y position of the component
     * @param w               the width of the component (if twidth and theight are both 0 then they are ignored)
     * @param h               the height of the component (if twidth and theight are both 0 then they are ignored)
     */
    public LabelLayoutItem(String labelResourceId, int x, int y, int w, int h) {
        super(labelResourceId, x, y, w, h);
    }

    /**
     * @param labelResourceId The associated resource ID which is associated with the created label
     * @param x               the x position of the component
     * @param y               the y position of the component
     * @param w               the width of the component (if twidth and theight are both 0 then they are ignored)
     * @param h               the height of the component (if twidth and theight are both 0 then they are ignored)
     * @param prop            Additional properties for the label
     */
    public LabelLayoutItem(String labelResourceId, int x, int y, int w, int h, PropertyEntry prop) {
        this(labelResourceId, x, y, w, h);

        setProperties(prop);
    }

    /**
     * Create the label and add it to the parent viewer
     *
     * @param parent_viewer The parent structure viewer
     */
    public void doLayout(DataViewer parent_viewer) {
        try {
            DataModel parent_model = parent_viewer.getDataModel();
            String resourceID = getModelFieldName();

            // create the label
            JLabel label = new CFJLabel(parent_viewer.getFactory().getClientResource(), resourceID);

            label.setOpaque(true);

            parent_viewer.add(label);

            defaultLayout(label, parent_viewer.getFactory());
        } catch (Exception ex) {
            throw new RuntimeException(ex);
            // ignore
        }
    }
}

