package com.nexirius.framework.layout;

import java.awt.Color;
import java.awt.Component;
import java.awt.SystemColor;

import javax.swing.JLabel;
import javax.swing.JTable;
import javax.swing.border.EmptyBorder;
import javax.swing.table.TableCellRenderer;

import com.nexirius.framework.datamodel.DataModel;
import com.nexirius.framework.datamodel.ModelFlag;
import com.nexirius.framework.datamodel.SimpleModel;

public class LayoutTableCellRenderer extends JLabel implements TableCellRenderer {
    private static final long serialVersionUID = -5889307533892817363L;

    public int prevRow = -1;
    public int prevColumn = -1;
    public boolean prevIsSelected = false;

    protected TableColumnLayoutItem layoutItem;

    public LayoutTableCellRenderer(TableColumnLayoutItem layoutItem) {
        this.layoutItem = layoutItem;
        setOpaque(true);
        setBorder(new EmptyBorder(0, 3, 0, 3));
    }

    @Override
    public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int column) {
        try {
            prevRow = row;

            // Icon Handling
            if (value instanceof DataModel) {
                String iconId = ((DataModel) value).getIconId();

                if (iconId != null) {
                    setIcon(layoutItem.getFactory().getClientResource().getIcon(iconId));
                } else {
                    setIcon(null);
                }
            } else {
                setIcon(null);
            }

            // Text Handling
            if (value instanceof SimpleModel) {
                SimpleModel model = (SimpleModel) value;

                if (model.isInvisible()) {
                    setText("");
                }
                else if (model.getFlag(ModelFlag.TRANSLATE_VALUE)) {
                    setText(layoutItem.getFactory().getText(model.getText()));
                } else {
                    setText(value.toString());
                }
            } else {
                setText(value == null ? "" : value.toString());
            }

            // Color Handling
            prevColumn = column;
            prevIsSelected = isSelected;

            if (!isSelected) {
                setBackground(SystemColor.text);
                setForeground(SystemColor.textText);
            }

            if (layoutItem.getProperties() != null) {
                layoutItem.getProperties().change(this, layoutItem.getFactory());
            }

            if (!isSelected) {
                Color background = null;

                if (value instanceof DataModel) {
                    DataModel model = (DataModel) value;

                    String c = model.getBackgroundColorId();

                    if (c != null) {
                        background = layoutItem.getFactory().getClientResource().getColor(c);
                    } else {
                        Object rowModel = table.getModel().getValueAt(row, -1);

                        if (rowModel instanceof DataModel) {
                            c = ((DataModel)rowModel).getBackgroundColorId();

                            if (c != null) {
                                background = layoutItem.getFactory().getClientResource().getColor(c);
                            }
                        }
                    }
                }

                if (background != null) {
                    setBackground(background);
                }
            }

            if (isSelected)
            {
                setBackground(table.getSelectionBackground());
                setForeground(table.getSelectionForeground());
//                setBackground(SystemColor.textHighlight);
//                setForeground(SystemColor.textHighlightText);
            }
        } catch (Exception ex) {
            throw new RuntimeException(ex);
        }

        return this;
    }
}