//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 *
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 *
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 *
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 *
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.framework.layout;

import com.nexirius.framework.dataviewer.DataViewer;
import com.nexirius.framework.dataviewer.LayoutItem;

public class TableColumnLayoutArray extends LayoutArray<TableColumnLayoutItem> {

    private static final long serialVersionUID = 2824007603249514203L;

    public TableColumnLayoutArray() {
    }

    public int getColumnNumber() {
        return size();
    }

    public String getColumnLabel(int x) {
        TableColumnLayoutItem item = getLayoutItem(x);

        return item.getColumnLabel();
    }

    public Class<?> getColumnClass(int colindex)
    {
        TableColumnLayoutItem item = getLayoutItem(colindex);
        if (item == null) {
            return Object.class;
        }
        return (item.getColumnClass());
    }

    public String getModelFieldName(int x) {
        TableColumnLayoutItem item = getLayoutItem(x);

        return item.getModelFieldName();
    }

    public TableColumnLayoutItem getLayoutItem(int x)
    {
        TableColumnLayoutItem item = (TableColumnLayoutItem) getObject(x);
        return item;
    }

    public TableColumnLayoutItem getLayoutItem(String fieldName)
    {
        for (LayoutItem layoutItem : this)
        {
            TableColumnLayoutItem tableColLayoutItem = (TableColumnLayoutItem)layoutItem;
            if (tableColLayoutItem.getModelFieldName().equals(fieldName))
            {
                return(tableColLayoutItem);
            }
        }
        return(null);
    }

    /**
     * Returns the index of the TableLayoutItem with the fieldName.
     * @param fieldName
     * @return
     */
    public int getLayoutItemIndex(String fieldName)
    {
        int idx = 0;
        for (LayoutItem layoutItem : this)
        {
            TableColumnLayoutItem tableColLayoutItem = (TableColumnLayoutItem)layoutItem;
            if (tableColLayoutItem.getModelFieldName().equals(fieldName))
            {
                return(idx);
            }
            idx++;
        }
        return(-1);

    }

    public void doLayout(DataViewer parent_viewer) {
        // go thru the list of items and call doLayout

        LayoutEnumeration e = getEnumeration();

        while (e.hasMore()) {
            TableColumnLayoutItem i = (TableColumnLayoutItem) e.next();

            i.doLayout(parent_viewer);
        }
    }
}
