//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.framework.layout;

import com.nexirius.framework.datamodel.DataModel;
import com.nexirius.framework.dataviewer.DataViewer;
import com.nexirius.framework.swing.ClientResourceTitledBorder;

import javax.swing.*;

public class TitledBorderLayoutItem extends DefaultLayoutItem {



    public TitledBorderLayoutItem() {
        super();
    }

    /**
     * @param field The associated structure field name (or any resource key string)
     * @param x     the x position of the component
     * @param y     the y position of the component
     * @param w     the width of the component (if twidth and theight are both 0 then they are ignored)
     * @param h     the height of the component (if twidth and theight are both 0 then they are ignored)
     */
    public TitledBorderLayoutItem(String field, int x, int y, int w, int h) {
        super(field, x, y, w, h);
    }

    public void doLayout(DataViewer parent_viewer) {
        try {
            DataModel parent_model = parent_viewer.getDataModel();
            // look for the associated model
            String resourceKey;

            try {
                DataModel model = parent_model.getChild(getModelFieldName());
                resourceKey = model.getFieldName();
            } catch (Exception ex) {
                resourceKey = getModelFieldName();
            }

            // create the associated label
            JPanel comp = new JPanel();

            comp.setOpaque(false);

            // create the border
            ClientResourceTitledBorder border = new ClientResourceTitledBorder(parent_viewer.getFactory().getClientResource(), resourceKey);

            comp.setBorder(border);

            parent_viewer.add(comp);

            // move and resize the JComponent
            defaultLayout(comp, parent_viewer.getFactory());
        } catch (Exception ex) {
            throw new RuntimeException(ex);
            // ignore
        }
    }
}

