//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.framework.menu;

import com.nexirius.framework.datamodel.DataModel;
import com.nexirius.framework.datamodel.DataModelCommand;
import com.nexirius.framework.datamodel.DataModelCommandVector;
import com.nexirius.framework.dataviewer.CommandViewer;
import com.nexirius.framework.dataviewer.ViewerFactory;
import com.nexirius.framework.swing.CFJButton;
import com.nexirius.framework.swing.CFJMenu;
import com.nexirius.framework.swing.CFJMenuBar;
import com.nexirius.framework.swing.CFJMenuItem;

import javax.swing.*;
import java.util.Stack;

public class SwingMenuCreator implements MenuCreator {
    private ViewerFactory factory;
    private CFJMenuBar menuBar;
    private Stack actMenuStack;
    private JToolBar toolBar;
    private DataModelCommandVector commands;
    private DataModel applicationModel;

    public SwingMenuCreator(ViewerFactory factory, DataModel applicationModel, DataModelCommandVector commands) {
        this.factory = factory;
        this.applicationModel = applicationModel;
        this.commands = commands;
    }

    public CFJMenuBar getMenuBar() {
        return menuBar;
    }

    public JToolBar getToolBar() {
        return toolBar;
    }

    public void newMenuBar() {
        menuBar = new CFJMenuBar(factory.getClientResource(), "MenuBar");
        actMenuStack = new Stack();
    }

    public void newToolBar() {
        toolBar = new JToolBar();
    }

    public void addMenu(String menuName) {
        CFJMenu menu = new CFJMenu(factory.getClientResource(), menuName);

        if (actMenuStack.isEmpty()) {
            menuBar.add(menu);
        } else {
            CFJMenu actMenu = (CFJMenu) actMenuStack.peek();
            actMenu.add(menu);
        }
        actMenuStack.push(menu);
    }

    public void addToggle(String menuName) {
        JComponent toggleMenu = null;
        
        try {
            toggleMenu = factory.createDefaultEditor(applicationModel.getChild(menuName)).getJComponent();
        } catch (Exception e) {
            throw new RuntimeException(e);
            // toggleMenu = new CFJCheckBox(factory.getClientResource(), menuName);
        }

        if (actMenuStack != null) {
            CFJMenu actMenu = (CFJMenu) actMenuStack.peek();

            actMenu.add(toggleMenu);
        } else if (toolBar != null) {
            toolBar.add(toggleMenu);
        }
    }

    public void addMenuItem(String menuItem) {
        if (actMenuStack != null) {
            CFJMenu actMenu = (CFJMenu) actMenuStack.peek();

            actMenu.add(createJMenuItem(menuItem));
        } else if (toolBar != null) {
            toolBar.add(createToolBarButton(menuItem));
        }
    }

    public void addSeparator() {
        if (actMenuStack != null) {
            CFJMenu actMenu = (CFJMenu) actMenuStack.peek();
            actMenu.add(new JSeparator());
        } else if (toolBar != null) {
            JSeparator separator = new JToolBar.Separator();
            separator.setOrientation(JSeparator.VERTICAL);
            //separator.setMaximumSize(new Dimension(8, 30));
            toolBar.add(separator);
        }
    }

    public void end() {
        if (actMenuStack != null) {
            if (!actMenuStack.isEmpty()) {
                actMenuStack.pop();
            } else {
                actMenuStack = null;
            }
        }
    }

    private JMenuItem createJMenuItem(String menuItem) {
        JMenuItem jMenuItem = null;

        if (commands != null) {
            for (DataModelCommand m = commands.firstItem(); m != null; m = commands.nextItem()) {
                if (m.getCommandName().equals(menuItem)) {
                    CommandViewer viewer = null;
                    try {
                        viewer = (CommandViewer) factory.createDefaultViewer(m);
                        jMenuItem = viewer.createJMenuItem();
                    } catch (Exception e) {
                        throw new RuntimeException(e);  //TODO
                    }
                }
            }
        }

        if (jMenuItem == null) {
            jMenuItem = new CFJMenuItem(factory.getClientResource(), menuItem);
        }
        return jMenuItem;
    }

    private JButton createToolBarButton(String text) {
        JButton ret = null;

        if (commands != null) {
            for (DataModelCommand m = commands.firstItem(); m != null; m = commands.nextItem()) {
                if (m.getCommandName().equals(text)) {
                    CommandViewer viewer = null;
                    try {
                        viewer = (CommandViewer) factory.createDefaultViewer(m);

                        ret = (JButton) viewer.createJButton();
                    } catch (Exception e) {
                        throw new RuntimeException(e);  //TODO
                    }

                    break;
                }
            }
        }

        if (ret == null) {
            ret = new CFJButton(factory.getClientResource(), text);
        }

        Icon icon = factory.getIcon(text);

        if (icon != null) {
            ret.setIcon(icon);
            ret.setText("");
        }
        return ret;
    }
}
