package com.nexirius.framework.rest;

import com.nexirius.framework.rest.interfaces.IJnexRestManager;
import com.nexirius.framework.rest.interfaces.IJnexRestParameterExtractor;
import com.nexirius.framework.rest.interfaces.IJnexRestRequest;

import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.ServletException;
import javax.servlet.ServletConfig;
import java.io.IOException;

public class JnexRestServlet extends HttpServlet {
    private static final String IJNEX_REST_MANAGER = "IJnexRestManager";
    private static final String IJNEX_REST_PARAMETER_EXTRACTOR = "IJnexRestParameterExtractor";
    IJnexRestManager restManager;
    IJnexRestParameterExtractor parameterExtractor;

    public void init() throws ServletException {
        ServletConfig servletConfig = getServletConfig();
        String className = servletConfig.getInitParameter(IJNEX_REST_MANAGER);

        if (className == null) {
            throw new ServletException("Servlet parameter " + IJNEX_REST_MANAGER + " == null");
        }

        try {
            Class clazz = getClass().getClassLoader().loadClass(className);
            restManager = (IJnexRestManager) clazz.newInstance();
            restManager.init(servletConfig);
        } catch (Exception e) {
            throw new ServletException(e);
        }

        className = servletConfig.getInitParameter(IJNEX_REST_PARAMETER_EXTRACTOR);

        if (className != null) {
            try {
                Class clazz = getClass().getClassLoader().loadClass(className);
                parameterExtractor = (IJnexRestParameterExtractor) clazz.newInstance();
            } catch (Exception e) {
                throw new ServletException(e);
            }
        }
    }

    protected void doGet(HttpServletRequest httpServletRequest, HttpServletResponse httpServletResponse) throws ServletException, IOException {
        doRest(httpServletRequest, httpServletResponse);
    }

    protected void doPost(HttpServletRequest httpServletRequest, HttpServletResponse httpServletResponse) throws ServletException, IOException {
        doRest(httpServletRequest, httpServletResponse);
    }

    private void doRest(HttpServletRequest httpServletRequest, HttpServletResponse httpServletResponse) {
        IJnexRestRequest restRequest = restManager.createRequest(httpServletRequest, httpServletResponse);

        try {
            restManager.rest(restRequest, parameterExtractor);
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }
}

