package com.nexirius.framework.rest.example;

import com.nexirius.framework.datamodel.ArrayModel;
import com.nexirius.framework.datamodel.DataModel;
import com.nexirius.framework.datamodel.StringModel;
import com.nexirius.framework.datamodel.json.DefaultJsonFormatMap;
import com.nexirius.framework.jdbc.DatabaseTableMapping;
import com.nexirius.framework.jdbc.JdbcConnectionHandler;
import com.nexirius.framework.rest.example.parameter.*;
import com.nexirius.framework.rest.interfaces.IJnexRestManager;
import com.nexirius.framework.rest.interfaces.IJnexRestParameterExtractor;
import com.nexirius.framework.rest.interfaces.IJnexRestParameters;
import com.nexirius.framework.rest.interfaces.IJnexRestRequest;
import com.nexirius.framework.rest.JnexRestRequest;

import javax.servlet.ServletConfig;
import javax.servlet.ServletContext;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.Enumeration;

public class JdbcJnexRestManager implements IJnexRestManager {
    public static final String FUNCTION_CREATE = "create";
    public static final String FUNCTION_READ = "read";
    public static final String FUNCTION_PUT_BLOB = "putBlob";
    public static final String FUNCTION_UPDATE = "update";
    public static final String FUNCTION_DELETE = "delete";
    public static final String FUNCTION_SEARCH = "search";
    public static final String CHARSET = "UTF-8";

    public JdbcJnexRestManager() {
    }

    public JdbcJnexRestManager(String url, String username, String password) {
        System.setProperty("url", url);
        System.setProperty("username", username);
        System.setProperty("password", password);

        init(new ServletConfig() {

            public String getServletName() {
                return null;
            }

            public ServletContext getServletContext() {
                return null;
            }

            public String getInitParameter(String s) {
                return System.getProperty(s);
            }

            public Enumeration getInitParameterNames() {
                return System.getProperties().keys();
            }
        });
    }

    public void init(ServletConfig servletConfig) {
        String url = getProperty(servletConfig, "url");
        String username = getProperty(servletConfig, "username");
        String password = getProperty(servletConfig, "password");

        JdbcConnectionHandler.init(new JdbcConnectionHandler(url, username, password));
    }

    private String getProperty(ServletConfig servletConfig, String key) {
        String ret = servletConfig.getInitParameter(key);

        if (ret == null) {
            throw new RuntimeException("Missing parameter: " + key);
        }

        return ret;
    }

    public void rest(IJnexRestRequest request, IJnexRestParameterExtractor parameterExtractor) throws Exception {
        IJnexRestParameters jnexRestParameters = request.getParameters(parameterExtractor);

        String function = jnexRestParameters.getParameter("function");

        if (FUNCTION_PUT_BLOB.equals(function)) {
        } else {
            String argument = jnexRestParameters.getParameter("argument");

            if (FUNCTION_READ.equals(function)) {
                ReadParameterModel readParameter = new ReadParameterModel();
                DefaultJsonFormatMap.readDataModel(readParameter, argument);
                DataModel result = readParameter.getResultTemplate();
                result.setInstanceName(readParameter.getKey());
                DatabaseTableMapping databaseTableMapping = JdbcConnectionHandler.instance().getDatabaseTableMapping(result.getClass().getName());
                databaseTableMapping.read(result);
                request.replyJson(result, DefaultJsonFormatMap.getInstance());
            } else if (FUNCTION_DELETE.equals(function)) {
                DeleteParameterModel deleteParameter = new DeleteParameterModel();
                DefaultJsonFormatMap.readDataModel(deleteParameter, argument);
                DatabaseTableMapping databaseTableMapping = JdbcConnectionHandler.instance().getDatabaseTableMapping(deleteParameter.getClassName());
                databaseTableMapping.delete(deleteParameter.getKey());
                request.replyJson(new StringModel("success"), DefaultJsonFormatMap.getInstance());
            } else if (FUNCTION_CREATE.equals(function)) {
                CreateParameterModel createParameter = new CreateParameterModel();
                DefaultJsonFormatMap.readDataModel(createParameter, argument);
                DataModel dataModel = createParameter.getDataModel();
                DatabaseTableMapping databaseTableMapping = JdbcConnectionHandler.instance().getDatabaseTableMapping(dataModel.getClass().getName());
                databaseTableMapping.create(dataModel);
                request.replyJson(dataModel, DefaultJsonFormatMap.getInstance());
            } else if (FUNCTION_UPDATE.equals(function)) {
                UpdateParameterModel updateParameter = new UpdateParameterModel();
                DefaultJsonFormatMap.readDataModel(updateParameter, argument);
                DataModel dataModel = updateParameter.getDataModel();
                DatabaseTableMapping databaseTableMapping = JdbcConnectionHandler.instance().getDatabaseTableMapping(dataModel.getClass().getName());
                databaseTableMapping.update(dataModel);
                request.replyJson(dataModel, DefaultJsonFormatMap.getInstance());
            } else if (FUNCTION_SEARCH.equals(function)) {
                SearchParameterModel searchParameter = new SearchParameterModel();
                DefaultJsonFormatMap.readDataModel(searchParameter, argument);
                DataModel template = searchParameter.getResultTemplate();
                ArrayModel result = new ArrayModel(template, "Result");
                DatabaseTableMapping databaseTableMapping = JdbcConnectionHandler.instance().getDatabaseTableMapping(template.getClass().getName());
                databaseTableMapping.search(searchParameter.getWhereClause(), template, result);
                request.replyJson(result, DefaultJsonFormatMap.getInstance());
            }
        }
    }

    public IJnexRestRequest createRequest(HttpServletRequest request, HttpServletResponse response) {
        return new JnexRestRequest(request, response);
    }
}
