//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.framework.swing;

import com.nexirius.framework.application.DialogManager;

import javax.swing.*;
import java.io.File;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.*;
import java.util.HashMap;

public class JnexFileChooser {

    static final int OPEN = 1;
    static final int SAVE = 2;
    public static String openText = "Open";
    public static String saveText = "Save";

    private static HashMap instanceMap = new HashMap();

    protected JFileChooser openChooser;
    protected JFileChooser saveChooser;
    protected JDialog openDialog;
    protected JDialog saveDialog;
    protected int actionOption;
    protected String directoryName;

    private JnexFileChooser(String directoryName) {
        this.directoryName = directoryName;
    }

    /**
     * access open file instance
     * @return the chooser for file open
     */
    public JFileChooser getJFileChooser() {
        if (openChooser == null) {
            initChooser(OPEN, null);
        }
        return openChooser;
    }

    /**
     * access save file instance
     * @return the chooser for file save
     */
    public JFileChooser getJFileChooserForSave() {
        if (saveChooser == null) {
            initChooser(SAVE, null);
        }
        return saveChooser;
    }

    /**
     *
     * @param directoryName if (null) then System.getProperty("user.dir") will be used
     */
    public static JnexFileChooser getInstance(String directoryName) {
        if (directoryName == null) {
            directoryName = System.getProperty("user.dir");
        }

        JnexFileChooser instance = (JnexFileChooser) instanceMap.get(directoryName);

        if (instance == null) {
            instance = new JnexFileChooser(directoryName);
            instanceMap.put(directoryName, instance);
        }

        return instance;
    }

    /**
     * display the open button
     * @param fileName
     */
    public File popupOpenDialog(String fileName) {
        initChooser(OPEN, fileName);
        return popup(openDialog, openChooser);
    }

    /**
     * display the save button
     * @param fileName
     */
    public File popupSaveDialog(String fileName) {
        initChooser(SAVE, fileName);
        return popup(saveDialog, saveChooser);
    }

    private File popup(JDialog dialog, JFileChooser chooser) {
        actionOption = JFileChooser.CANCEL_OPTION;
        dialog.setVisible(true);
        int retval = actionOption;

        if (retval == JFileChooser.APPROVE_OPTION) {
            return chooser.getSelectedFile();
        }

        return null;
    }

    public void initChooser(int type, String filename) {
        File directory = new File(directoryName);
        JFileChooser chooser;

        switch (type) {
            case SAVE:
                if (saveChooser == null) {
                    saveChooser = new JFileChooser(directory);
                    saveChooser.setDialogType(JFileChooser.SAVE_DIALOG);

                    saveDialog = DialogManager.createJDialog("", true);
                    saveDialog.getContentPane().setLayout(new BorderLayout());
                    saveDialog.getContentPane().add(saveChooser);

                    saveDialog.pack();

                    DialogManager.center(saveDialog, false);

                    saveChooser.addActionListener(new ActionListener() {
                        public void actionPerformed(ActionEvent e) {
                            if (e.getActionCommand().equals(JFileChooser.APPROVE_SELECTION)) {
                                actionOption = JFileChooser.APPROVE_OPTION;
                            }
                            saveDialog.setVisible(false);
                        }
                    });
                }
                chooser = saveChooser;

                break;

            default:
                if (openChooser == null) {
                    openChooser = new JFileChooser(directory);
                    openChooser.setDialogType(JFileChooser.OPEN_DIALOG);

                    openDialog = DialogManager.createJDialog("", true);
                    openDialog.getContentPane().setLayout(new BorderLayout());
                    openDialog.getContentPane().add(openChooser);

                    openDialog.pack();

                    if (openDialog.getSize().getWidth() < 10 || openDialog.getSize().getHeight() < 10) {
                        openDialog.setSize(300, 500);
                    }

                    DialogManager.center(openDialog, false);

                    openChooser.addActionListener(new ActionListener() {
                        public void actionPerformed(ActionEvent e) {
                            if (e.getActionCommand().equals(JFileChooser.APPROVE_SELECTION)) {
                                actionOption = JFileChooser.APPROVE_OPTION;
                            }
                            openDialog.setVisible(false);
                        }
                    });
                }
                chooser = openChooser;

                break;
        }

        actionOption = JFileChooser.CANCEL_OPTION;

        chooser.setCurrentDirectory(directory);
        chooser.rescanCurrentDirectory();
        DialogManager.setDefaultCursor(chooser);

        try {
            if (filename != null) {
                if (filename.indexOf(File.separatorChar) < 0) {
                        chooser.setSelectedFile(new File(chooser.getCurrentDirectory().getAbsolutePath(), filename));
                } else {
                    File file = new File(filename);

                    if (file.isDirectory()) {
                        chooser.setCurrentDirectory(file);
                    } else {
                        chooser.setSelectedFile(file);
                    }
                }
            } else if (directoryName != null) {
                chooser.setSelectedFile(new File(directory, ""));
            }
        } catch (Exception t) {
            throw new RuntimeException(t);
        }
    }
}
