//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.framework.textselector;

import com.nexirius.util.StringVector;

/**
 * This interface handles a selector functionality which is used in conjunction with SelectorModel
 */
public interface TextSelector {
    /**
     * get the name which is used to reference this unique instance via TextSelectorManager
     */
    public String getName();

    /**
     * registerTemplate new text entry (must call setText(text))
     */
    public void addText(String text);

    /**
     * When a certain amount of characters is reached, then the popup window with maching entries is activated
     *
     * @return the threshhold
     */
    public int getMinCharsForPopup();

    /**
     * When a certain amount of characters is reached, then the popup window with maching entries is activated
     */
    public void setMinCharsForPopup(int c);

    /**
     * get a possible completion for a given string
     *
     * @return the actual possible completion or null if none of the entries starts with the given string
     */
    public String getCompletionFor(String s);

    /**
     * get all possible completions for a given string
     *
     * @param s      the string which will be tested against
     * @param result the resulting list of strings will be appended to this list
     * @return true if there are more than the returned number of possible completions
     */
    public boolean getCompletionListFor(String s, StringVector result);

    /**
     * access the text which has been selected
     */
    public String getText();

    /**
     * set the text which will be selected
     */
    public void setText(String text);

    /**
     * popup a dialog which is used to edit the actual text which will be selected
     *
     * @return true if selection has been performed
     */
    public boolean popup();
}
