//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.framework.trend;

import java.util.Date;
import java.util.GregorianCalendar;

public class DateRange {



    public static long MS_PER_DAY = 24 * 3600 * 1000;
    long value;
    long width;

    public DateRange(Date from, Date to) {
        value = from.getTime();
        width = to.getTime() - value;

        if (width < 0) {
            width = 0;
        }
    }

    public DateRange(Date from, double days) {
        value = from.getTime();

        if (days < 0) {
            width = 0;
        } else {
            width = (long) (days * MS_PER_DAY);
        }
    }

    public DateRange(int year, int month, int day, double days) {
        GregorianCalendar cal = new GregorianCalendar(year, month - 1, day);

        value = cal.getTime().getTime();

        if (days < 0) {
            width = 0;
        } else {
            width = (long) (days * MS_PER_DAY);
        }
    }

    public long min() {
        return value;
    }

    public long max() {
        return value + width;
    }

    public long width() {
        return width;
    }

    public double getRelativeLocation(long position) {
        if (width == 0) {

            return 0;
        }

        return (double) (position - value) / (double) width;
    }

    public boolean contains(Date d) {
        long t = d.getTime();

        if (t >= value && t <= max()) {

            return true;
        }

        return false;
    }

    public long getDays() {
        return width / MS_PER_DAY;
    }

    public boolean isDay() {
        return getDays() <= 2;
    }

    public boolean isWeek() {
        return getDays() <= 14;
    }

    public boolean isMonth() {
        return getDays() <= 62;
    }

    public boolean isYear() {
        return getDays() <= 731;
    }

}
