//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.framework.trend;

import java.awt.Graphics;
import java.awt.Rectangle;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;

import com.nexirius.util.TimeMachine;

public class TrendGridDate extends TrendGrid {



    DateRange range;

    public TrendGridDate(DateRange r) {
        range = r;
        xAchsis = true;
    }

    public LabelPositionVector getPositions(Rectangle rect) {
        Calendar cal = TimeMachine.getCalendar();
        LabelPositionVector v = new LabelPositionVector();

        cal.setTime(new Date(range.min()));
        //cal = new GregorianCalendar(cal.get(Calendar.YEAR), cal.get(Calendar.MONTH), cal.get(Calendar.DAY_OF_MONTH));

        if (range.isDay()) {
            // every 3 hours

            while (cal.getTime().getTime() < range.min()) {
                cal.add(Calendar.HOUR, 3);
            }

            while (cal.getTime().getTime() <= range.max()) {
                v.append(new LabelPosition(getPosition(rect, cal.getTime().getTime()), Integer.toString(cal.get(Calendar.HOUR_OF_DAY))));
                cal.add(Calendar.HOUR, 3);
            }
        } else if (range.isWeek()) {
            // every day
            while (cal.getTime().getTime() < range.min()) {
                cal.add(Calendar.DAY_OF_YEAR, 1);
            }

            while (cal.getTime().getTime() <= range.max()) {
                v.append(new LabelPosition(getPosition(rect, cal.getTime().getTime()), Integer.toString(cal.get(Calendar.DAY_OF_MONTH))));
                cal.add(Calendar.DAY_OF_YEAR, 1);
            }
        } else if (range.isMonth()) {
            // every week

            cal.add(Calendar.DAY_OF_YEAR, -cal.get(Calendar.DAY_OF_WEEK));

            while (cal.getTime().getTime() < range.min()) {
                cal.add(Calendar.WEEK_OF_YEAR, 1);
            }

            while (cal.getTime().getTime() <= range.max()) {
                v.append(new LabelPosition(getPosition(rect, cal.getTime().getTime()), Integer.toString(cal.get(Calendar.DAY_OF_MONTH))));
                cal.add(Calendar.WEEK_OF_YEAR, 1);
            }
        } else if (range.isYear()) {
            // every month

            cal.add(Calendar.DAY_OF_YEAR, -(cal.get(Calendar.DAY_OF_MONTH) - 1));

            while (cal.getTime().getTime() < range.min()) {
                cal.add(Calendar.MONTH, 1);
            }

            while (cal.getTime().getTime() <= range.max()) {
                v.append(new LabelPosition(getPosition(rect, cal.getTime().getTime()), Integer.toString(cal.get(Calendar.MONTH))));
                cal.add(Calendar.MONTH, 1);
            }
        } else {
            // every year
            cal.add(Calendar.DAY_OF_YEAR, -(cal.get(Calendar.DAY_OF_YEAR) - 1));

            while (cal.getTime().getTime() < range.min()) {
                cal.add(Calendar.YEAR, 1);
            }

            while (cal.getTime().getTime() <= range.max()) {
                v.append(new LabelPosition(getPosition(rect, cal.getTime().getTime()), Integer.toString(cal.get(Calendar.YEAR))));
                cal.add(Calendar.YEAR, 1);
            }
        }

        return v;
    }

    public int getPosition(Rectangle rect, long d) {
        return (int) rect.getX() + (int) (rect.getWidth() * range.getRelativeLocation(d));
    }

    public void paintRange(Graphics g, Rectangle rect) {
        int yoff = (int) g.getFontMetrics().getAscent();
        Calendar cal =  TimeMachine.getCalendar();
//        LabelPositionVector v = new LabelPositionVector();

        SimpleDateFormat format = new SimpleDateFormat("dd.MM.yyyy HH:mm");
        String t1 = format.format(new Date(range.min()));
        cal.setTime(new Date(range.max()));
        String t2 = format.format(new Date(range.max()));

        int y = rect.y + yoff;

        g.drawString(t1, rect.x + 2, y += 2);
        g.drawString(t2, rect.x + 2, y += yoff + 2);
    }
}
