//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.print;

import javax.swing.*;
import java.awt.*;
import java.awt.print.PageFormat;

public class DefaultFixPrintable extends FixPrintableTable {
    String left;
    String center;
    String right;

    class HeaderTemplate extends PrintTableTemplate {
        public HeaderTemplate() {
            super(new PrintTableCellProperties());

            defaultCellProperties.setProperty(PrintTableCellProperties.FONTFACE, "Default");
            defaultCellProperties.setProperty(PrintTableCellProperties.FONTSIZE, 2);
            defaultCellProperties.setProperty(PrintTableCellProperties.MARGINLEFT, 1);
            defaultCellProperties.setProperty(PrintTableCellProperties.MARGINRIGHT, 1);
            defaultCellProperties.setProperty(PrintTableCellProperties.NO_FRAME, 1);

            PrintTableCellProperties cp1 = new PrintTableCellProperties();
            PrintTableCellProperties cp2 = new PrintTableCellProperties();
            PrintTableCellProperties cp3 = new PrintTableCellProperties();

            cp1.setProperty(PrintTableCellProperties.WIDTH_PERCENT, 20);
            cp1.setProperty(PrintTableCellProperties.COLUMN_NAME, "LEFT");
            cp1.setProperty(PrintTableCellProperties.ALIGNMENT, SwingConstants.LEFT);

            cp2.setProperty(PrintTableCellProperties.WIDTH_PERCENT, 60);
            cp2.setProperty(PrintTableCellProperties.COLUMN_NAME, "CENTER");
            cp2.setProperty(PrintTableCellProperties.USES_PAGE_INDEX, 1);
            cp2.setProperty(PrintTableCellProperties.ALIGNMENT, SwingConstants.CENTER);

            cp3.setProperty(PrintTableCellProperties.WIDTH_PERCENT, 20);
            cp3.setProperty(PrintTableCellProperties.COLUMN_NAME, "RIGHT");
            cp3.setProperty(PrintTableCellProperties.ALIGNMENT, SwingConstants.RIGHT);

            appendCellToRow(cp1);
            appendCellToRow(cp2);
            appendCellToRow(cp3);
        }
    }

    class HeaderData implements PrintTableData {
        public int getRowCount() {
            return 1;
        }

        public String getHeader(String colName) {
            return "Header";
        }

        public String getData(String colName, int row, int pageIndex) {
            if (colName.equals("LEFT")) {
                return left;
            } else if (colName.equals("CENTER")) {
                return center + " Page:" + (pageIndex + 1);
            } else if (colName.equals("RIGHT")) {
                return right;
            }
            return "";
        }

        public String getFooter(String colName) {
            return "Footer";
        }
    }

    public DefaultFixPrintable(PageFormat pf, String left, String center, String right) {
        super(new Rectangle(0, 0, 100, 10)
                , pf
                , (PrintTableTemplate) null
                , (PrintTableDataAccessor) null);
        this.left = left;
        this.center = center;
        this.right = right;
        set(new HeaderTemplate(), new PrintTableDataAccessor(new HeaderData(), null));
    }
}
