//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.print;

import java.awt.*;
import java.awt.print.PageFormat;
import java.awt.print.Printable;
import java.awt.print.PrinterException;

public class PageTemplate implements Printable {



    protected Rectangle flowBounds;
    protected FixPrintableList fixPrintables = new FixPrintableList();
    protected PageFormat pageFormat;

    public PageTemplate(Rectangle flowBounds, PageFormat pageFormat) {
        this.flowBounds = flowBounds;
        this.pageFormat = pageFormat;
    }

    public PageFormat getPageFormat() {
        return pageFormat;
    }

    public void add(FixPrintable p) {
        fixPrintables.append(p);
    }

    public FixPrintableList getFixPrintables() {
        return fixPrintables;
    }

    public Rectangle getFlowBounds() {
        int xPage = (int) pageFormat.getImageableX();
        int yPage = (int) pageFormat.getImageableY();
        int wPage = (int) pageFormat.getImageableWidth();
        int hPage = (int) pageFormat.getImageableHeight();

        return new Rectangle(recalc(flowBounds.x, xPage, wPage), recalc(flowBounds.y, yPage, hPage), recalc(flowBounds.width, 0, wPage), recalc(flowBounds.height, 0, hPage));
    }

    public static int recalc(int v, int offset, int width) {
        return (v * width) / 100 + offset;
    }

    public int print(Graphics gc, PageFormat pageFormat, int pageIndex)
            throws PrinterException {
        for (Printable p = fixPrintables.first(); p != null; p = fixPrintables.next()) {
            p.print(gc, pageFormat, pageIndex);
        }

        return PAGE_EXISTS;
    }
}
