//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.print;

import java.awt.*;
import java.awt.print.PageFormat;
import java.awt.print.Printable;

public class PrintTablePart implements Printable {



    protected PrintTableRowList rows;
    protected Point location;

    protected PrintTable printTable;
    protected int firstRow;
    protected int height = 0;
    protected int pageIndex;

    public PrintTablePart(Point location, PrintTable printTable, int firstRow, int pageIndex) {
        this.printTable = printTable;
        this.rows = new PrintTableRowList();
        this.location = location;
        this.firstRow = firstRow;
        this.pageIndex = pageIndex;
    }

    public PrintTableDataAccessor getData() {
        return printTable.getData();
    }

    public int getHeight() {
        return height;
    }

    public int getPageIndex() {
        return pageIndex;
    }

    public int getWidth() {
        return printTable.getWidth();
    }

    public int print(Graphics gc, PageFormat pageFormat, int pageIndex) {
        this.pageIndex = pageIndex;

        printPage(location, gc);

        return PAGE_EXISTS;
    }

    public void printPage(Point location, Graphics gc) {
        Point p = new Point(location);

        for (PrintTableRow row = rows.first(); row != null; row = rows.next()) {
            row.print(p, gc);
            p.translate(0, row.getHeight());
        }
    }

    public int preparePage(int nextRow, Point location, int maxY) {
        // prepare header
        PrintTableRow row = printTable.template.createHeader(printTable);
        height = 0;

        if (row != null) {
            int hrow = row.preparePrint(0, pageIndex);
            // does header fit on page?

            if (location.y + hrow > maxY) {
                if (nextRow == 0) {

                    return nextRow;
                }

                return -1;
            }

            rows.append(row);
            height += hrow;
        }

        for (; nextRow < getData().getRowCount(); ++nextRow) {
            // prepare next row
            row = printTable.template.createRow(printTable);

            int hrow = row.preparePrint(nextRow, pageIndex);

            // does row fit on this page?
            if (location.y + height + hrow > maxY) {

                return nextRow;
            }

            rows.append(row);
            height += hrow;
        }

        // print footer

        return -1;
    }
}

