//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.tcp;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.Socket;

// This class implements a simple TCP client which answers messages

public class ClientImpl implements Client {



    public static final char END_OF_MSG_CHAR = '\r';
    public static final char ESC_CHAR = '\\';

    Thread myThread;
    Socket socket = null;

    public ClientImpl(Socket socket) {
        setSocket(socket);
    }

    public void setSocket(Socket socket) {
        this.socket = socket;
    }

    public String getAnswer(String message) {
        return message;
    }

    private void delay() {
        try {
            Thread.currentThread().sleep(100);
        } catch (Exception ex) {
        }
    }

    public void run() {
        InputStream in;
        OutputStream out;

        try {
            in = socket.getInputStream();
            out = socket.getOutputStream();

            socket.setTcpNoDelay(false);
        } catch (Exception e) {
            throw new RuntimeException(e);
            // System.err.println("While creating streams");

            // leave the thread
            // return;
        }

        try {
            while (true) {
                String msg = readMessageFromStream(in);

                if (msg == null) {
                    // give up this thread
                    return;
                }

                System.out.println("RECEIVED '" + msg + "'");
                // send answer
                writeMessageToStream(out, getAnswer(msg));
            }
        } catch (Exception e) {
            throw new RuntimeException("While reading or writing to socket stream");
        } finally {
            try {
                System.out.println("Closing streams and socket");
                in.close();
                out.close();
                socket.close();
            } catch (Exception ex) {
                throw new RuntimeException(ex);
            }
        }
    }

    public static String readMessageFromStream(InputStream in)
            throws IOException {
        int state = 0;
        StringBuffer msg = new StringBuffer();

        // read characters from the socket

        while (true) {
            int c = in.read();

            if (c == -1) {
                if (msg.length() > 0) {
                    System.err.println("Unexpected end of stream after reading '" + msg.toString() + "'");

                    return null;
                }

                return null;
            }

            switch (state) {
                case 0:
                    switch (c) {
                        case END_OF_MSG_CHAR:

                            return msg.toString();

                        case ESC_CHAR:
                            state = 1;
                            break;

                        default:
                            msg.append((char) c);
                    }
                    break;

                case 1:
                    state = 0;
                    msg.append('\\');
                    msg.append((char) c);
                    break;
            }
        }
    }

    public static void writeMessageToStream(OutputStream out, String msg)
            throws IOException {
        int len = msg.length();

        for (int i = 0; i < len; ++i) {
            char c = msg.charAt(i);

            System.out.print(c);

            if (c == ESC_CHAR || c == END_OF_MSG_CHAR) {
                out.write(ESC_CHAR);
            }

            out.write(c);
        }

        out.write(END_OF_MSG_CHAR);
        System.out.println("\n.done.");
    }

    public void start() {
        myThread = new Thread(this);
        myThread.start();
    }
}
