//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.util;

import java.io.File;
import java.util.StringTokenizer;

/**
 * a class that helps to find a file in an arbitrary number of directories (path)
 */
public class DirPath extends Object {



    private StringVector m_path = new StringVector();

    public DirPath() {

    }

    /**
     * creates a path from a single directory or from
     * a path name
     */
    public DirPath(String path) {
        set(path);
    }

    /**
     * get the path name
     *
     * @return directory names separated by path separator character
     */
    public String getPath() {
        StringBuffer ret = new StringBuffer();
        String s;

        for (s = m_path.firstItem(); s != null;) {
            ret.append(s);
            s = m_path.nextItem();

            if (s == null) {
                break;
            } else {
                ret.append(File.pathSeparator);
            }
        }

        return ret.toString();
    }

    /**
     * is the specified directory part of the existing path?
     */
    public boolean contains(String directory) {
        String s = null;

        for (s = m_path.firstItem(); s != null; s = m_path.nextItem()) {
            if (s.equals(directory)) {
                return true;
            }
        }

        return false;
    }

    /**
     * look for the specified file in all the directories
     *
     * @return the directory name, where the file exists or null if the file was not found
     */
    public String getDirectory(String file) {
        String ret = null;
        String s = null;

        for (s = m_path.firstItem(); s != null; s = m_path.nextItem()) {
            File f = new File(s, file);

            if (f.exists()) {
                ret = s;
                break;
            }
        }

        return ret;
    }

    /**
     * look for the specified file in all the directories
     *
     * @return the file, or null if the file was not found
     */
    public File getFile(String file) {
        File ret = null;
        String s = null;

        for (s = m_path.firstItem(); s != null; s = m_path.nextItem()) {
            File f = new File(s, file);

            if (f.exists()) {
                ret = f;
                break;
            }
        }

        return ret;
    }

    /**
     * look for the specified file in all the directories
     *
     * @return the file, or null if the file was not found
     */
    public XFile getXFile(String file) {
        XFile ret = null;
        String s = null;

        for (s = m_path.firstItem(); s != null; s = m_path.nextItem()) {
            XFile f = new XFile(s, file);

            if (f.exists()) {
                ret = f;
                break;
            }
        }

        return ret;
    }

    /**
     * look for the specified file in all the directories
     *
     * @return the file name, or null if the file was not found
     */
    public String getFileName(String file) {
        File f = getFile(file);
        String ret = null;

        if (f != null) {
            ret = f.getPath();
        }

        return ret;
    }

    /**
     * @return true if the file was found in one of the path directories
     */
    public boolean exists(String file) {
        return null != getFile(file);
    }

    /**
     * deletes the existing path (if set) and creates a path from a single directory or from
     * a path name
     */
    public void set(String path) {
        insert(path, true, true);
    }

    /**
     * appends a single directory or a path name to the end of the existing path
     */
    public void append(String path) {
        insert(path, true, false);
    }

    /**
     * inserts a single directory or a path name to the beginning of the
     * existing path
     */
    public void prepend(String path) {
        insert(path, true, false);
    }

    private void insert(String path, boolean at_end, boolean override) {
        if (override) {
            m_path.removeAllElements();
        }

        int index = (at_end ? m_path.size() : 0);

        // check if there are path name separators in path
        if (path.indexOf(File.pathSeparatorChar) != -1) {
            // separate directories
            StringTokenizer st = new StringTokenizer(path, File.pathSeparator);

            while (st.hasMoreTokens()) {
                m_path.insertElementAt(st.nextToken(), index++);
            }

        } else {
            m_path.insertElementAt(path, index++);
        }
    }

    public static void main(String argv[])
            throws Exception {
        DirPath p = new DirPath(argv[0]);
        String file = argv[1];

        if (p.exists(file)) {
            System.out.println(file + " --> exists in " + p.getDirectory(file));
        } else {
            System.out.println(file + " --> does not exist in " + p.getPath());
        }

        throw new Exception("END.");
    }

    public String toString() {
        return getPath();
    }

    public boolean equals(Object o) {
        return o != null && o instanceof DirPath && o.toString().equals(toString());
    }

    public Object clone() {
        return new DirPath(toString());
    }
}
