//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.util;

import com.nexirius.framework.datamodel.DataModel;

/**
 * This type was created in VisualAge
 *
 * @author Marcel Baumann
 * @invariant
 */
public interface Editable {

    /**
     * Return whether or not finish edit can complete successfully with an invalid value
     *
     * @return boolean
     * @author Marcel Baumann
     * @post result = this.allowInvalid
     */
    public boolean allowInvalid();

    /**
     * Terminate edit mode and restore old value to current value
     *
     * @author Marcel Baumann
     * @pre isEditing()
     * @post !isEditing() && getValue() == getOldValue()
     */
    public void cancelEdit();

    /**
     * If value is valid then exit editing status, else throw an exception
     *
     * @author Marcel Baumann
     * @pre isEditing()
     * @post !isEditing() && ( allowInvalid() || isValid() )
     */
    public void finishEdit();

    /**
     * Return the value before editing status was entered
     *
     * @author Marcel Baumann
     * @post result == this.oldValue
     */
    public Object getOldValue();

    /**
     * Return the current value
     *
     * @return java.lang.Object
     * @author Marcel Baumann
     * @post result == this.value
     */
    public Object getValue();

    /**
     * Return true if in editing status
     *
     * @return boolean
     * @author Marcel Baumann
     * @post result == this.isEditing
     */
    public boolean isEditing();

    /**
     * This method was created in VisualAge
     *
     * @return boolean
     * @author Marcel Baumann
     * @post true iif getValue() conforms to some condition (determined by implementor)
     */
    public boolean isValid();

    /**
     * Set the current value of the object
     *
     * @author Marcel Baumann
     * @post getValue() == value
     */
    public void setValue(Object value);

    /**
     * This method has to be redifined for classes which distinguish between
     * assignment of a new value and replacement of intance
     *
     * @author Marcel Baumann
     * @post getValue().equals(value)
     */
    public void assignValue(Object value);

    /**
     * Place the editable object in editing 'status'
     * and copy the current value to oldValue
     *
     * @author Marcel Baumann
     * @pre !isEditing()
     * @post isEditing() && getOldValue() == getValue()
     */
    public void startEdit();

    /**
     * create an exact copy instance of the actual value
     *
     * @author Marcel Baumann
     * @pre getValue() != null
     * @post result.equals(getValue())
     */
    public Object cloneValue(CopyPairs<DataModel> copyPairs);
}
