//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.util;

import java.io.*;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.MissingResourceException;
import java.util.PropertyResourceBundle;

public class ResourceFile extends PropertyResourceBundle implements StringResource {



    String m_filename;
    FileInputStream m_stream = null;
    Hashtable m_override = new Hashtable();

    public ResourceFile(String filename) throws ResourceFileException, IOException {
        super(makeStream(filename));
        m_filename = filename;
    }

    public ResourceFile(FileInputStream stream) throws ResourceFileException, IOException {
        super(stream);
        m_stream = stream;
    }

    public String getAsString(String key) {
        String ret = null;

        try {
            ret = getString(key);
        } catch (MissingResourceException ex) {
            return key;
        }

        return ret;
    }

    public synchronized boolean getAsBoolean(String key) throws ResourceFileException {
        try {
            String s = getString(key);

            if (s.equalsIgnoreCase("false")) {
                return false;
            } else if (s.equalsIgnoreCase("true")) {
                return true;
            }

            throw new ResourceFileException("Resource '" + key + "' must be true or false");
        } catch (MissingResourceException ex) {
            throw new ResourceFileException("Resource '" + key + "' was not defined in " + m_filename);
        }
    }

    public synchronized int getAsInt(String key) throws ResourceFileException {
        try {
            String s = getString(key);

            try {
                return Integer.parseInt(s);
            } catch (Exception ex) {
                throw new ResourceFileException("Resource '" + key + "' must be an integer");
            }
        } catch (MissingResourceException ex) {
            throw new ResourceFileException("Resource '" + key + "' was not defined in " + m_filename);
        }
    }

    public synchronized long getAsLong(String key) throws ResourceFileException {
        try {
            String s = getString(key);

            try {
                return Long.parseLong(s);
            } catch (Exception ex) {
                throw new ResourceFileException("Resource '" + key + "' must be a long integer");
            }
        } catch (MissingResourceException ex) {
            throw new ResourceFileException("Resource '" + key + "' was not defined in " + m_filename);
        }
    }

    public synchronized double getAsDouble(String key) throws ResourceFileException {
        try {
            String s = getString(key);

            try {
                Double d = Double.valueOf(s);

                return d.doubleValue();
            } catch (Exception ex) {
                throw new ResourceFileException("Resource '" + key + "' must be a floating point number");
            }
        } catch (MissingResourceException ex) {
            throw new ResourceFileException("Resource '" + key + "' was not defined in " + m_filename);
        }
    }

    public String getFileName() {
        return m_filename;
    }

    public void overrideResource(String key, String value)
            throws ResourceFileException {
        try {
            String s = getString(key);

            m_override.put(key, value);
        } catch (Exception ex) {
            throw new ResourceFileException("Can't override unknown resource '" + key + "'");
        }
    }

    public Object handleGetObject(String key) {
        Object value = m_override.get(key);

        if (value != null) {
            return value;
        }

        return super.handleGetObject(key);
    }

    public void print() {
        // display the resources
        try {
            for (Enumeration e = getKeys(); e.hasMoreElements();) {
                String key = (String) e.nextElement();
                String val = getString(key);

                System.out.println(key + "=" + val);
            }
        } catch (Exception ex) {
            System.out.println(ex);
        }
    }


    private static InputStream makeStream(String filename) throws ResourceFileException {
        File file = new File(filename);
        FileInputStream stream = null;

        if (!file.exists() || !file.canRead()) {
            throw new ResourceFileException("Can't read resource file:" + filename);
        }

        try {
            stream = new FileInputStream(file);
        } catch (FileNotFoundException ex) {
            throw new ResourceFileException("Resource file not found:" + file.getName());
        }

        return stream;
    }
}
