package com.nexirius.util;

import java.awt.*;
import java.awt.event.*;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;

import javax.swing.*;

/**
 * Die Klasse TimeMachine stellt einen Zeitstempel in der Form "new Date()" oder "System.currentTimeMillis" zur Verfügung. Im Gegensatz zu den Originalroutinen
 * liefert TimeMachine aber nicht zwingend die wirkliche Uhrzeit und das wirkliche Datum sondern kann auf einen beliebigen Wert gesetzt werden. Diese
 * Möglichkeit wird zu Testzwecken verwendet um z.B. fünf jährige Revisionen zu testen ohne fünf Jahre warten zu müssen.
 * @author bubenma1
 */
public class TimeMachine
{
    // Der Offset in Millisekunden welcher zur wirklichen Zeit addiert wird.
    public static long offset = 0L;
    // public static long offset = 365 * 24 * 60 * 60 * 1000L; // Ein Jahr
    // public static long offset = 5 * 365 * 24 * 60 * 60 * 1000L; // Drei Jahre

    // Für das User Interface
    protected static JFrame window;
    protected static JTextField jTextField;;
    protected static JLabel info;

    static
    {
        //
        try
        {
            // File dir1 = new File (".");
            // System.out.println ("Current dir : " + dir1.getCanonicalPath());

            // Sofern eine Datei "TimeMachine.txt" existiert wird diese gelesen und die Zahl in der ersten Zeile
            // als Offset verwendet!
            File tmFile = new File("TimeMachine.txt");
            if (tmFile.exists())
            {
                BufferedReader reader = new BufferedReader(new FileReader(tmFile));
                String aLine = reader.readLine();
                offset = (long) (Double.parseDouble(aLine.trim()) * 1000.0 * 60 * 60 * 24);
                reader.close();
            }
        }
        catch (Exception exp)
        {
            // Ignorieren!
        }

        // Wenn der Offset am Anfang ungleich 0 ist, dann wird ein kleines User Interface angezeigt!
        if (offset != 0L)
        {
            showUserInterface();
        }
    }

    /**
     * Das 'aktuelle' Datum / Uhrzeit als Date Objekt.
     * @return
     */
    public static Date getDate()
    {
        return (new Date(currentTimeMillis()));
    }

    /**
     * Das 'aktuelle' Datum / Uhrzeit als Anzahl Millisekunden seit 1. Januar 1970!
     * @return
     */
    public static long currentTimeMillis()
    {
        return (System.currentTimeMillis()) + offset;
    }

    public static Calendar getCalendar()
    {
        // Zunächst einen Kalender mit dem aktuellen Datum erstellen.
        Calendar cal = new GregorianCalendar();

        // Und dann den Offset dazu addieren.
        // Weil das add(.., ..) nur einen int Wert als amount erlaubt, teilen wir das Addieren in Sekunden und Millisekunden auf!
        cal.add(Calendar.SECOND, (int) (offset / 1000L));
        cal.add(Calendar.MILLISECOND, (int) (offset % 1000L));

        return (cal);
    }

    /**
     * Ein kleines User Interface
     */
    public static void showUserInterface()
    {
        if (window == null)
        {
            JPanel panel = new JPanel(new GridBagLayout());
            GridBagConstraints c = new GridBagConstraints();

            c.gridx = 0;
            c.gridy = 0;
            panel.add(new JLabel("Offset in days: "), c);

            jTextField = new JTextField();
            jTextField.setText(Double.toString(offset / 1000.0 / 60 / 60 / 24));
            jTextField.addFocusListener(new FocusAdapter()
            {
                @Override
                public void focusLost(FocusEvent focusevent)
                {
                    textHasChanged();
                }
            });
            jTextField.setPreferredSize(new Dimension(50, 22));
            c.gridx = 1;
            c.gridy = 0;
            panel.add(jTextField, c);

            info = new JLabel();
            c.gridx = 0;
            c.gridy = 1;
            panel.add(info, c);
            updateInfoLabel(info);

            JButton button = new JButton("Apply");
            button.addActionListener(new ActionListener()
            {
                @Override
                public void actionPerformed(ActionEvent arg0)
                {
                    textHasChanged();
                }
            });
            c.gridx = 0;
            c.gridy = 2;
            panel.add(button, c);

            window = new JFrame("Time Machine");
            window.setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
            window.setResizable(false);

            window.add(panel);

            window.setPreferredSize(new Dimension(200, 110));
            window.setMinimumSize(new Dimension(200, 110));
            window.setLocation(4, 4);
        }
        window.setVisible(true);
        window.setState(Frame.NORMAL);
        window.toFront();
    }

    private static void updateInfoLabel(JLabel infoLabel)
    {
        SimpleDateFormat sdf = new SimpleDateFormat("dd.MM.yyyy");

        infoLabel.setText(sdf.format(getDate()));
    }

    protected static void textHasChanged()
    {
        try
        {
            offset = (long) Double.parseDouble(jTextField.getText()) * 1000L * 60 * 60 * 24;
            updateInfoLabel(info);
        }
        catch (Exception e)
        {
            // Ignore all
        }
    }
}
