//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.util;

import java.io.File;
import java.io.FilenameFilter;

/**
 * A string class with additional functionality
 */
public class XString extends Object implements FilenameFilter {

    private String sValue;

    public XString(String s) {
        sValue = s;
    }

    public int length() {
        return sValue.length();
    }

    /**
     * set the current string
     */
    public void set(String s) {
        sValue = s;
    }

    /**
     * get the current string
     */
    public String get() {
        return sValue;
    }

    public String toString() {
        return sValue;
    }

    public boolean equals(Object s) {
        return sValue.equals(s);
    }

    /**
     * this function compares a pattern which eventually contains
     * wildcards '*' and '?' with the current string
     *
     * @return true if the pattern matches the current string
     */
    public boolean match(String pattern) {
        if (pattern == null) {
            return false;
        }

        boolean ret = false;
        ret = check(pattern, 0, 0);

        return ret;
    }

    /**
     * this function compares a pattern which eventually contains
     * wildcards '*' and '?' with a string
     *
     * @return true if the pattern matches the string
     */
    public static boolean match(String pattern, String s) {
        XString xs = new XString(s);

        return xs.match(pattern);
    }

    public boolean accept(File dir, String filename) {
        String pattern = get();

        set(filename.toUpperCase());

        boolean ret = match(pattern.toUpperCase());

        set(pattern);

        return ret;
    }

    private boolean check(String pattern, int p_index, int s_index) {
        if (p_index == pattern.length()) {
            return s_index == sValue.length();
        }

        switch (pattern.charAt(p_index)) {
            case '*':
                if (p_index + 1 == pattern.length()) {
                    // break when the last char is '*'
                    return true;
                }
                if (pattern.charAt(p_index + 1) == '*') {
                    // skip multiple '*'
                    return check(pattern, p_index + 1, s_index);
                }
                return (s_index != sValue.length() && check(pattern, p_index, s_index + 1)) || check(pattern, p_index + 1, s_index);

            case '?':
                return s_index != sValue.length() && check(pattern, p_index + 1, s_index + 1);
        }

        if (s_index == sValue.length()) {
            return false;
        }

        return sValue.charAt(s_index) == pattern.charAt(p_index) && check(pattern, p_index + 1, s_index + 1);
    }

    /**
     * replace every occurence of 'from' with 'to'
     *
     * @return the number of replacements
     */
    public int replace(String from, String to) {
        int ret = 0;
        int index = 0;
        StringBuffer result = new StringBuffer();
        String rest = sValue;

        if (from == null) {
            return 0;
        }

        while (true) {
            index = rest.indexOf(from);

            if (index >= 0) {
                char buf[] = new char[index];

                ++ret;
                rest.getChars(0, index, buf, 0);
                result.append(buf);

                if (to != null) {
                    result.append(to);
                }

                index += from.length();
                int rest_bytes = rest.length() - index;

                if (rest_bytes > 0) {
                    // new rest
                    buf = new char[rest_bytes];
                    rest.getChars(index, index + rest_bytes, buf, 0);
                    rest = new String(buf);
                } else {
                    break;
                }
            } else {
                result.append(rest);
                break;
            }
        }

        sValue = result.toString();

        return ret;
    }

    /**
     * get a substring out of the current string
     *
     * @param from starting index
     * @param to   end index (if -1 then the rest of the string is returned)
     * @return the substring or "" (if from and to are bigger than the actual string)
     */
    public String subString(int from, int to) {
        if (to == -1) {
            to = sValue.length();
        }

        if (from >= to) {
            return "";
        }

        char buf[] = new char[to - from];

        sValue.getChars(from, to, buf, 0);

        return new String(buf);
    }

    /**
     * any occurence of any character in carr is removed from the string
     */
    public String removeCharacters(char carr[]) {
        for (int i = 0; i < carr.length; ++i) {
            removeCharacter(carr[i]);
        }

        return sValue;
    }

    /**
     * any occurence of character c is removed from the string
     */
    public String removeCharacter(char c) {
        if (-1 == sValue.indexOf(c)) {
            return sValue;
        }

        StringBuffer result = new StringBuffer();

        for (int i = 0; i < sValue.length(); ++i) {
            char sc = sValue.charAt(i);

            if (sc != c) {
                result.append(sc);
            }
        }

        sValue = result.toString();

        return sValue;
    }

    /**
     * @return if o1>o2 -> -1 if o1<o2 -> 1 if o1==o2 -> 0
     */
    public static int compare(Object o1, Object o2) {
        try {
            String item1 = (String) o1;
            String item2 = (String) o2;
            String s1 = item1.toUpperCase();
            String s2 = item2.toUpperCase();

            return s2.compareTo(s1);
        } catch (Exception ex) {
            return 1;
        }
    }

    public static void main(String argv[])
            throws Exception {
        XString s = new XString(argv[0]);

        System.out.println("pattern " + argv[3] + (s.match(argv[3]) ? " found" : " not found"));
        System.out.println("replace returns:" + s.replace(argv[1], argv[2]) + " result=" + s);
    }
}
