//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.util.locale;

import java.util.Locale;

public class LocaleManagerImpl
        implements LocaleManager {



    private Locale locale = null;
    protected transient LocaleListener aLocaleListener = null;

    public LocaleManagerImpl() {
        super();
    }

    /**
     * @param newListener com.nexirius.framework.LocaleListener
     */
    public void addLocaleListener(LocaleListener newListener) {
        aLocaleListener = LocaleEventMulticaster.add(aLocaleListener, newListener);
        return;
    }

    /**
     * Method to support listener events.
     *
     * @param event com.nexirius.framework.LocaleEvent
     */
    private void fireLocaleChangedEvent(LocaleEvent event) {
        if (aLocaleListener == null) {
            return;
        }
        ;
        aLocaleListener.localeChanged(event);
    }

    /**
     * Gets the locale property (java.util.Locale) value.
     *
     * @return The locale property value.
     * @see #setLocale
     */
    public Locale getLocale() {
        if (this.locale == null) {
            this.locale = Locale.getDefault();
        }
        return this.locale;
    }

    /**
     * @param newListener com.nexirius.framework.LocaleListener
     */
    public void removeLocaleListener(LocaleListener newListener) {
        aLocaleListener = LocaleEventMulticaster.remove(aLocaleListener, newListener);
        return;
    }

    /**
     * Sets the locale property (java.util.Locale) value.
     *
     * @param locale The new value for the property.
     * @see #getLocale
     */
    public void setLocale(java.util.Locale locale) {
        if (locale == null) {
            throw new IllegalArgumentException("Parameter locale cannot be null");
        }
        this.locale = locale;
        LocaleEvent event = new LocaleEvent(this, this.locale);
        fireLocaleChangedEvent(event);
    }

    /**
     * This method was created in VisualAge.
     *
     */
    public void updateLocaleListeners() {
        fireLocaleChangedEvent(new LocaleEvent(this, getLocale()));
    }
}
