//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.util.resource;

import com.nexirius.util.locale.LocaleEvent;
import com.nexirius.util.textresource.TextResource;
import com.nexirius.util.textresource.TextResourceListener;

import java.awt.*;
import java.net.URL;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Locale;

/**
 * @author Marcel Baumann
 * @see com.nexirius.util.resource.ClientResource for documentation
 */
public class DefaultClientResource
        implements ClientResource {



    protected static DateFormat DEFAULT_DATE_FORMAT = new SimpleDateFormat("dd.MM.yyyy HH:mm:ss");
    protected static Color DEFAULT_COLOR = Color.black;
    protected static int DEFAULT_FONT_SIZE = 12;
    protected static String DEFAULT_FONT_FAMILY = "SansSerif";
    protected static Font DEFAULT_FONT = new Font(DEFAULT_FONT_FAMILY, Font.PLAIN, DEFAULT_FONT_SIZE);

    /**
     * ClientResourceImpl constructor comment.
     */
    public DefaultClientResource() {
        super();
    }


    /**
     * Do nothing
     *
     * @param newListener a listener
     */
    public void addResourceChangeListener(ResourceChangeListener newListener) {
    }


    /**
     * @return false
     */
    public boolean isDefined(String resourceKey) {
        return false;
    }


    /**
     * @return 0
     */
    public int getInt(String resourceKey) {
        return 0;
    }


    /**
     * @return 0
     */
    public double getDouble(String resourceKey) {
        return 0;
    }


    /**
     * @return null
     */
    public URL getHelpURL(String resourceKey) {
        return null;
    }

    /**
     * @return null
     */
    public String getHelpString(String resourceKey) {
        return null;
    }


    /**
     * Returns the text displayed for a button in the current language.
     *
     * @return java.lang.String
     */
    public String getButtonText(String resourceKey) {
        return getText(resourceKey);
    }


    /**
     * NOT USED!!!!
     * @deprecated
     */
    /*
    public java.net.URL getHelpURL(String resourceKey)
    {
    	return null;
    }
    */


    /**
     * getIcon returns an Icon.
     */
    public javax.swing.Icon getIcon(String resourceKey) {
        return null;
    }

    /**
     * Same as getText() but returns the key itself instead of null when not defined
     */
    public String getLabel(String resourceKey) {
        return resourceKey;
    }

    /**
     * An accelerator key is a shortcut to a command or event.  Examples
     * are cut Ctrl+X, copy Ctrl+C, and paste Ctrl+V.
     * Note this is not the same as a mnemonic key, see getMnemonic().
     */
    public String getKeyAcceleratorText(String resourceKey) {
        return null;
    }


    /**
     * Return the text that should appear on a menu status in the current language.
     */
    public String getMenuText(String resourceKey) {
        return getText(resourceKey);
    }


    /**
     * return 0 (zero)
     */
    public int getMnemonic(String resourceKey, String ext) {
        return 0;
    }


    /**
     * Tool tips are help text that appears if you hold the cursor over a widget for a few seconds.
     * Returns the tool tip in the current language.
     */
    public String getToolTipText(String resourceKey) {
        return getText(resourceKey);
    }


    /**
     * Appends .dateformat to the resourceKey and returns a Date Format Instance from the resource bundel.
     * Return DEFAULT_DATE_FORMAT if resource not found.
     *
     * @return java.text.DateFormat
     */
    public DateFormat getDateFormat(String resourceKey) {
        return DEFAULT_DATE_FORMAT;
    }


    /**
     * @return DEFAULT_COLOR
     */
    public Color getColor(String resourceKey) {
        return DEFAULT_COLOR;
    }


    /**
     * @return DEFAULT_FONT
     */
    public Font getFont(String resourceKey) {
        return DEFAULT_FONT;
    }


    /**
     * @return DEFAULT_FONT_FAMILY
     */
    public String getFontFamily(String resourceKey) {
        return DEFAULT_FONT_FAMILY;
    }


    /**
     * @return DEFAULT_FONT_SIZE
     */
    public int getFontSize(String resourceKey) {
        return DEFAULT_FONT_SIZE;
    }


    /**
     * @return null
     */
    public String getCurrencyNumberFormat(String currencyCode) {
        return null;
    }


    /**
     * Returns 0.0
     *
     * @return the smallest denominator
     */
    public double getCurrencyRoundingDenominator(String currencyCode) {
        return 0.0;
    }


    /**
     * Do nothing
     *
     * @param newListener a new listener
     */
    public void removeResourceChangeListener(ResourceChangeListener newListener) {
    }


    /**
     * Do nothing
     *
     * @param newListener a new listener
     */
    public void removeTextResourceListener(TextResourceListener newListener) {
    }


    /**
     * Do nothing
     *
     * @param newListener a new listener
     */
    public void addTextResourceListener(TextResourceListener newListener) {
    }


    /**
     * @param resourceKey the key of the text resource
     * @return resourceKey
     */
    public String getText(String resourceKey) {
        return resourceKey;
    }


    /**
     * @param resourceKey the key of the text resource
     * @return resourceKey
     */
    public String getText(String resourceKey, Locale locale) {
        return resourceKey;
    }


    /**
     * Do nothing
     */
    public void setParent(TextResource resource) {
    }

    /**
     * Do nothing
     */
    public void setParent(ClientResource resource) {
    }


    /**
     * Do nothing
     */
    public void updateListeners() {
    }


    /**
     * Do nothing
     */
    public void localeChanged(LocaleEvent event) {
    }


}
