//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.util.resource;

import com.nexirius.framework.FWLog;

import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;
import java.lang.reflect.Field;

/**
 * This type was created in VisualAge.
 */
public class Util {



    /**
     * Util constructor comment.
     */
    public Util() {
        super();
    }

    /**
     * takes input of the form: Mask+Key (Ctrl+X) and returns mask constant.
     *
     * @param acceleratorText java.lang.String
     * @return int
     */
    public static int getMask(String acceleratorText) {
        int pos = acceleratorText.indexOf('+');
        if (pos == -1) return 0;
        String mask = acceleratorText.substring(0, pos);
        if ("META".equalsIgnoreCase(mask))
            return InputEvent.META_MASK;
        else if ("CTRL".equalsIgnoreCase(mask))
            return InputEvent.CTRL_MASK;
        else if ("SHIFT".equalsIgnoreCase(mask))
            return InputEvent.SHIFT_MASK;
        else if ("ALT".equalsIgnoreCase(mask))
            return InputEvent.ALT_MASK;
        else if ("BUTTON1".equalsIgnoreCase(mask))
            return InputEvent.BUTTON1_MASK;
        else if ("BUTTON2".equalsIgnoreCase(mask))
            return InputEvent.BUTTON2_MASK;
        else if ("BUTTON3".equalsIgnoreCase(mask))
            return InputEvent.BUTTON3_MASK;
        else
            return 0;
    }

    /**
     * takes input of the form: Mask+Key (Ctrl+X) and returns VK constant.
     *
     * @param acceleratorText java.lang.String
     * @return int
     */
    public static int getVirtualKey(String acceleratorText) {
        try {
            int pos = acceleratorText.indexOf('+');
            String key = (pos == -1) ? acceleratorText : acceleratorText.substring(pos + 1);
            key = key.toUpperCase();
            if ("LEFT".equals(key))
                key = "LEFT";
            else if ("RIGHT".equals(key))
                key = "RIGHT";
            else if ("DEL".equals(key))
                key = "DELETE";
            else if ("ESC".equals(key))
                key = "ESCAPE";
            else if ("PGDN".equals(key))
                key = "PAGE_UP";
            else if ("PGUP".equals(key))
                key = "PAGE_DOWN";
            else if ("/".equals(key))
                key = "SLASH"; //DIVIDE
            else if (";".equals(key))
                key = "SEMICOLON";
            else if ("=".equals(key))
                key = "EQUALS";
            else if (",".equals(key))
                key = "COMB"; //SEPARATER
            else if (".".equals(key))
                key = "PERIOD";
            else if ("(".equals(key))
                key = "OPEN_BRACKET";
            else if ("\\".equals(key))
                key = "BACK_SLASH";
            else if (")".equals(key))
                key = "CLOSE_BRACKET";
            else if ("*".equals(key))
                key = "MULTIPLY";
            else if ("+".equals(key))
                key = "ADD";
            else if ("-".equals(key))
                key = "SUBTRACT";
            else if (".".equals(key))
                key = "DECIMAL";
            else if ("`".equals(key))
                key = "BACK_QUOTE";
            else if ("'".equals(key)) key = "QUOTE"; //????
            Class c = KeyEvent.class;
            Field field = c.getDeclaredField("VK_" + key);

//System.out.println("VK_"+ key + "  - " + field.getName() + "  - " + field);
//new Exception().printStackTrace();
            return field.getInt(c);
        } catch (Exception e) {
            FWLog.error(e);
            return KeyEvent.VK_UNDEFINED; //0
        }
    }

    /**
     * This method was created in VisualAge.
     *
     * @param args java.lang.String[]
     */
    public static void main(String args[]) {
        System.out.println(getMask("Ctrl+X"));
        System.out.println(getVirtualKey("Ctrl+X"));
        System.out.println(getMask("Delete"));
        System.out.println(getVirtualKey("Delete"));
    }
}
