//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.util.simpletype;

import java.math.BigDecimal;

/**
 * The simple type class helps to change simple type values into Strings
 * and vice versa. It also helps to compare values in various ways.
 *
 * @autor Marcel Baumann
 */
public abstract class SimpleType extends Object implements Comparable<Object> 
{
    public static boolean isSimpleType(Object o) {
        if (o == null) {
            return false;
        }

        if (o instanceof Integer) {
            return true;
        }

        if (o instanceof Double) {
            return true;
        }

        if (o instanceof Float) {
            return true;
        }

        if (o instanceof Character) {
            return true;
        }

        if (o instanceof Byte) {
            return true;
        }

        if (o instanceof Long) {
            return true;
        }

        if (o instanceof Short) {
            return true;
        }

        if (o instanceof Boolean) {
            return true;
        }

        if (o instanceof String) {
            return true;
        }

        if (o instanceof BigDecimal) {
            return true;
        }

        return false;
    }

    public static SimpleType create(Object o)
            throws Exception {
        if (o instanceof Integer) {
            return create(((Integer) o).intValue());
        } else if (o instanceof Double) {
            return create(((Double) o).doubleValue());
        } else if (o instanceof Float) {
            return create(((Float) o).floatValue());
        } else if (o instanceof Character) {
            return create(((Character) o).charValue());
        } else if (o instanceof Byte) {
            return create(((Byte) o).byteValue());
        } else if (o instanceof Long) {
            return create(((Long) o).longValue());
        } else if (o instanceof Short) {
            return create(((Short) o).shortValue());
        } else if (o instanceof Boolean) {
            return create(((Boolean) o).booleanValue());
        } else if (o instanceof String) {
            return create((String) o);
        } else if (o instanceof BigDecimal) {
            return create((BigDecimal) o);
        }

        throw new Exception("Can't convert " + o + " into a SimpleType object");
    }

    public static SimpleType create(Class<?> c, String value)
            throws Exception {
        if (c.getName().equals("int")) {
            return new SimpleType_int(value);
        } else if (c.getName().equals("double")) {
            return new SimpleType_double(value);
        } else if (c.getName().equals("float")) {
            return new SimpleType_float(value);
        } else if (c.getName().equals("char")) {
            return new SimpleType_char(value);
        } else if (c.getName().equals("byte")) {
            return new SimpleType_byte(value);
        } else if (c.getName().equals("long")) {
            return new SimpleType_long(value);
        } else if (c.getName().equals("short")) {
            return new SimpleType_short(value);
        } else if (c.getName().equals("boolean")) {
            return new SimpleType_boolean(value);
        } else if (c.getName().equals("java.lang.String")) {
            return new SimpleType_String(value);
        } else if (c.equals(BigDecimal.class)) {
            return new SimpleType_BigDecimal(value);
        }

        throw new Exception("Can't convert " + c.getName() + " into a SimpleType object");
    }

    public static SimpleType create(byte value) {
        return new SimpleType_byte(value);
    }

    public static SimpleType create(char value) {
        return new SimpleType_char(value);
    }

    public static SimpleType create(double value) {
        return new SimpleType_double(value);
    }

    public static SimpleType create(float value) {
        return new SimpleType_float(value);
    }

    public static SimpleType create(int value) {
        return new SimpleType_int(value);
    }

    public static SimpleType create(long value) {
        return new SimpleType_long(value);
    }

    public static SimpleType create(short value) {
        return new SimpleType_short(value);
    }

    public static SimpleType create(boolean value) {
        return new SimpleType_boolean(value);
    }

    public static SimpleType create(String value) {
        return new SimpleType_String(value);
    }

    public static SimpleType create(BigDecimal value) {
        return new SimpleType_BigDecimal(value);
    }

    public abstract SimpleType duplicate();

    public byte getByte() {
        throw new RuntimeException("Can't produce a byte from " + getClass().getName());
    }

    public char getChar() {
        throw new RuntimeException("Can't produce a char from " + getClass().getName());
    }

    public double getDouble() {
        throw new RuntimeException("Can't produce a double from " + getClass().getName());
    }

    public float getFloat() {
        throw new RuntimeException("Can't produce a float from " + getClass().getName());
    }

    public int getInt() {
        throw new RuntimeException("Can't produce an int from " + getClass().getName());
    }

    public long getLong() {
        throw new RuntimeException("Can't produce a long from " + getClass().getName());
    }

    public short getShort() {
        throw new RuntimeException("Can't produce a short from " + getClass().getName());
    }

    public boolean getBoolean() {
        throw new RuntimeException("Can't produce a boolean from " + getClass().getName());
    }

    public BigDecimal getBigDecimal() {
        throw new RuntimeException("Can't produce a boolean from " + getClass().getName());
    }

    public boolean equals(Object other) {
        throw new RuntimeException("equals() not implemented for: " + getClass().getName());
    }

    public abstract String toString();

    public abstract void set(String value) throws Exception;

    /**
     * @return an appropriate type for the primitive underlying type (eg. Integer for int)
     */
    public abstract Object wrap();
}
