//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.util.simpletype;

import java.text.DateFormat;
import java.util.Date;

public class SimpleType_Date extends SimpleType {



    Date m_value;
    DateFormat format;

    public SimpleType_Date(Date value) {
        m_value = value;
    }

    public SimpleType_Date(Date value, DateFormat dateFormat)
            throws Exception {
        m_value = value;
        setDateFormat(dateFormat);
    }

    public SimpleType_Date(String value, DateFormat dateFormat)
            throws Exception {
        setDateFormat(dateFormat);
        set(value);
    }

    public void setDateFormat(DateFormat format) {
        this.format = format;
    }

    public DateFormat getDateFormat() {
        return this.format;
    }

    public SimpleType duplicate() {
        SimpleType_Date ret = new SimpleType_Date(getDate());

        ret.setDateFormat(getDateFormat());

        return ret;
    }

    public boolean equals(Object other) {
        if (other != null && other instanceof SimpleType_Date) {
            
            return m_value.getTime() == ((SimpleType_Date) other).m_value.getTime();
        }

        return false;
    }

    @Override
    public int compareTo(Object other)
    {
        if (other != null && other instanceof SimpleType_Date) {
            
            return   ((Long)m_value.getTime()).compareTo((Long)((SimpleType_Date) other).m_value.getTime());
        }

        return -1;
    }
    
    public void set(String value)
            throws Exception {
        if (format == null) {
            throw new RuntimeException("Can't use SimpleType_Date.set(String) without DateFormat");
        }

        m_value = format.parse(value);
    }

    public void setDate(Date value) {
        m_value = value;
    }

    public void setTime(long time) {
        m_value.setTime(time); 
    }

    public Date getDate() {
        return new Date(m_value.getTime());
    }

    public String toString() {
        if (format == null) {
            throw new RuntimeException("Can't use SimpleType_Date.toString() without DateFormat");
        }

        return format.format(m_value);
    }

    public Object wrap() {
        return m_value;
    }

    @Override
    public int hashCode()
    {
        return(m_value == null ? 0 : m_value.hashCode());
    }
}
