//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 *
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 *
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 *
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 *
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.util.simpletype;

import com.nexirius.util.TextToken;

import java.lang.reflect.Array;
import java.lang.reflect.Constructor;
import java.util.Arrays;

/**
 * Helps to hold arrays of primitive types
 *
 */
public class SimpleType_array extends SimpleType {

    public static final String SEPARATOR = "\n";
    Object array[] = null;

    public SimpleType_array() {
    }

    public SimpleType_array(String text) {
        set(text);
    }

    public SimpleType_array(Object... array) {
        setArray(array);
    }

    public void setArray(Object... array) {
        this.array = array;
    }

    public Object[] getArray() {
        return array;
    }

    @Override
    public SimpleType duplicate() {
        int len = Array.getLength(array);

        Object new_array[] = (Object[]) Array.newInstance(getComponentType(), len);

        while (--len >= 0) {
            Array.set(new_array, len, getItem(len));
        }

        return new SimpleType_array(new_array);
    }

    public void setItem(int index, Object value) {
        Array.set(array, index, value);
    }

    public Object getItem(int index) {
        try {
            return Array.get(array, index);
        } catch (Exception ex) {
            throw new RuntimeException("getItem(" + index + "): " + ex.getMessage(), ex);
        }
    }

    public SimpleType getSimpleTypeItem(int index) {
        try {
            return SimpleType.create(getItem(index));
        } catch (Exception ex) {
            throw new RuntimeException("Unexpected type in array: " + ex.getMessage(), ex);
        }
    }

    public int getLength() {
        if (array == null) {
            return 0;
        }

        return Array.getLength(array);
    }

    public Class<?> getComponentType() {
        if (array == null) {
            return null;
        }

        return array.getClass().getComponentType();
    }

    @Override
    public boolean equals(Object other) {
        // FIX with Java 2.0 this should be replacable with Array.equals(array, ((SimpleType_array)other).array)

        if (other == null || !(other instanceof SimpleType_array)) {
            return false;
        }

        SimpleType_array oa = (SimpleType_array) other;


        int len = getLength();

        if (len != oa.getLength()) {
            return false;
        }

        if (!getComponentType().equals(oa.getComponentType())) {
            return false;
        }

        while (--len >= 0) {
            Object item1 = getItem(len);
            Object item2 = oa.getItem(len);
            if (item1 == null) {
                return item2 == null;
            } else if (!item1.equals(item2)) {
                return false;
            }
        }

        return true;
    }

    @Override
    public int compareTo(Object other)
    {
        if (other == null || !(other instanceof SimpleType_array))
        {
            return -1;
        }

        Object[] otherArray = ((SimpleType_array)other).array;
        if (Arrays.equals(array, otherArray))
        {
            // Beide Arrays sind identisch!
            return(0);
        }

        // Leider gibt es kein Arrays.compareTo(). Also müssen wir es selber machen!
        if (array.length != otherArray.length)
        {
            // Unterschiedliche Länge
            return(array.length - otherArray.length);
        }

        // Element für Element vergleichen!
        for (int i = 0; i < array.length; i++)
        {
            Object o1 = array[i];
            Object o2 = otherArray[i];

            if (o1.equals(o2) == false)
            {
                // Aha. Die zwei sind unterschiedlich. Jetzt müsse wir nur noch schauen wer 'kleiner' ist!
                if (o1 instanceof Comparable)
                {
                    return (((Comparable<Object>)o1).compareTo(o2));
                }

                // o1 ist nicht einmal Comparable ==> Wir geben dann einfach -1 aus
                return(-1);
            }
        }
        return(0);
    }

    @Override
    public String toString() {
        int len = getLength();
        StringBuffer sb = new StringBuffer();

        sb.append(new TextToken(getComponentType().getName()).toString());

        for (int i = 0; i < len; ++i) {
            Object o = getItem(i);

            if (o == null) {
                sb.append(new TextToken("").toString());
            } else {
                sb.append(new TextToken(o.toString()).toString());
            }

            if (i < len - 1) {
                sb.append(SEPARATOR);
            }
        }

        return sb.toString();
    }

    @Override
    public void set(String value) {
        try {
            TextToken t[] = TextToken.getTokensFrom(value);
            Class<?> type = Class.forName(t[0].getString());
            Object newValue[] = (Object[]) Array.newInstance(type, t.length - 1);
            Class<?> param[] = {new String("").getClass()};

            Constructor<?> constr = type.getConstructor(param);
            Object cparam[] = new String[1];

            for (int i = 1; i < t.length; ++i) {
                cparam[0] = t[i].getString();
                newValue[i - 1] = constr.newInstance(cparam);
            }

            setArray(newValue);
        } catch (Exception ex) {
            throw new RuntimeException("Can't parse: " + value);
        }
    }

    /**
     * @return an apropriate type for the primitive underlying type (eg. Integer for int)
     */
    @Override
    public Object wrap() {
        return array;
    }
}
