//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.util.statemachine;

import java.io.Serializable;

/**
 * The transition is the unidirectional connection between two states. It holds the event
 * information which maps on this transition
 */
public class Transition implements Serializable {



    State next;
    String event;

    /**
     * This constructor calls addTransition on the current state!
     *
     * @param current The state to which this transition will be attached
     * @param event   The event name which is used to step through this transition
     * @param next    The target state
     */
    public Transition(State current, String event, State next) {
        this.event = event;
        this.next = next;
        current.addTransition(this);
    }

    /**
     * creates a default transition (not attached to a specific event). The default
     * transition is used to handle unknown events (and null events).
     * The event name of the default transition is always null!
     */
    public Transition(State current, State next) {
        this.event = null;
        this.next = next;
        current.addTransition(this);
    }

    /**
     * the event is null for default transitions
     */
    public String getEvent() {
        return event;
    }

    /**
     * Access the target state
     *
     * @return
     */
    public State getNextState() {
        return next;
    }

    /**
     * Check whether this is the default transition (event name is null)
     *
     * @return
     */
    public boolean isDefault() {
        return event == null;
    }
}

