//{HEADER
/**
 * This class is part of jnex 'Nexirius Application Framework for Java'
 *	
 * Copyright (C) Nexirius GmbH, CH-4450 Sissach, Switzerland (www.nexirius.ch)
 * 
 * <p>This library is free software; you can redistribute it and/or<br>
 * modify it under the terms of the GNU Lesser General Public<br>
 * License as published by the Free Software Foundation; either<br>
 * version 2.1 of the License, or (at your option) any later version.</p>
 * 
 * <p>This library is distributed in the hope that it will be useful,<br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of<br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU<br>
 * Lesser General Public License for more details.</p>
 * 
 * <p>You should have received a copy of the GNU Lesser General Public<br>
 * License along with this library; if not, write to the Free Software<br>
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>
 * </blockquote>
 * 
 * <p>
 * Nexirius GmbH, hereby disclaims all copyright interest in<br>
 * the library jnex' 'Nexirius Application Framework for Java' written<br>
 * by Marcel Baumann.</p>
 */
//}HEADER
package com.nexirius.util.support;

import com.nexirius.framework.datamodel.SimpleArrayModel;
import com.nexirius.util.StringVector;
import com.nexirius.util.resource.ClientResource;

/**
 * This class is a singleton which acts as a server for all currency related informations.
 */
public class CurrencyManager {



    public static final String CURRENCY_CODES = "CurrencyCodes";

    protected static String DEFAULT_CURRENCY_NUMBER_FORMAT = "#,##0.00;-#,##0.00";
    protected static double DEFAULT_CURRENCY_ROUNDING_DENOMINATOR = 0.01;

    private static CurrencyManager instance = null;

    ClientResource resource;
    protected SimpleArrayModel currencyCodes = null;

    public static void init(ClientResource resource) {
        if (isInitialized()) {

            throw new RuntimeException("Can't initialize CurrencyManager more than once");
        }

        instance = new CurrencyManager(resource);
    }

    public static boolean isInitialized() {
        return instance != null;
    }

    public static CurrencyManager getInstance() {
        if (isInitialized()) {
            return instance;
        }

        throw new RuntimeException("Can't access CurrencyManager instance before initialisation");
    }

    public static void setInstance(CurrencyManager manager) {
        instance = manager;
    }

    public CurrencyManager() {

    }

    private CurrencyManager(ClientResource resource) {
        this.resource = resource;
    }

    public SimpleArrayModel getCurrencyCodes() {
        if (currencyCodes == null) {
            // get the list of available currency codes
            String codes = resource.getText(CURRENCY_CODES);

            if (codes == null) {
                throw new RuntimeException(CURRENCY_CODES + " must be defined in the resources (comma separated list)");
            }

            StringVector codesVector = new StringVector(codes, ", \t");

            currencyCodes = new SimpleArrayModel(codesVector.getArray(), CURRENCY_CODES);
        }

        return currencyCodes;
    }

    public String getNumberFormat(String currencyCode) {
        String ret = resource.getCurrencyNumberFormat(currencyCode);

        if (ret == null) {
            ret = DEFAULT_CURRENCY_NUMBER_FORMAT;
        }

        return ret;
    }

    public double getRoundingDenominator(String currencyCode) {
        double ret = resource.getCurrencyRoundingDenominator(currencyCode);

        if (ret == 0.0) {
            ret = DEFAULT_CURRENCY_ROUNDING_DENOMINATOR;
        }

        return ret;
    }
}
